(function(ja,ka,la,ma,u,na,M,z,B,R,N,p,oa,D,pa,X,Q,G,r,qa,U,da,ra,ea,fa,ga,sa,Z,ha,ia,K,ta,ua,ba,va,wa,xa,ya,aa,za,Aa,Ba,Ca,Da,Ea,Fa,Ga,Ha,Ia,Ja,Ka,La,Ma,Na,Oa,Pa,Qa,Ra,d,g){M("J.adapter.smarter");B(["JU.P3","$.SB"],"J.adapter.smarter.XtalSymmetry","java.lang.Boolean $.Float java.util.Hashtable JU.BS $.Lst $.M3 $.M4 $.P3i $.PT $.V3 J.adapter.smarter.Atom JS.Symmetry $.SymmetryOperation JU.BSUtil $.SimpleUnitCell".split(" "),function(){c$=N(function(){this.sym2=this.baseSymmetry=this.asc=this.acr=
null;this.doCentroidUnitCell=this.centroidPacked=this.applySymmetryToBonds=!1;this.doNormalize=!0;this.latticeOnly=this.doPackUnitCell=!1;this.unitCellTranslations=this.latticeCells=this.baseUnitCell=this.unitCellParams=this.trajectoryUnitCells=null;this.packingError=this.symmetryRange=0;this.bondsFound=this.filterSymop=null;this.ndims=3;this.nVib=this.noSymmetryCount=this.latticeOp=this.firstAtom=0;this.mTemp=this.ptTemp=null;this.rmaxz=this.rmaxy=this.rmaxx=this.rminz=this.rminy=this.rminx=0;this.maxXYZ0=
this.minXYZ0=this.maxXYZ=this.minXYZ=this.ptOffset=null;this.checkAll=!1;this.bondCount0=0;this.symmetry=null;R(this,arguments)},J.adapter.smarter,"XtalSymmetry");U(c$,function(){this.unitCellParams=r(6,0);this.bondsFound=new JU.SB;this.ptOffset=new JU.P3});D(c$,function(){});d(c$,"set",function(a){this.acr=a;this.asc=a.asc;this.getSymmetry();return this},"J.adapter.smarter.AtomSetCollectionReader");d(c$,"getSymmetry",function(){return null==this.symmetry?this.symmetry=this.acr.getInterface("JS.Symmetry"):
this.symmetry});d(c$,"setSymmetry",function(a){return this.symmetry=a},"J.api.SymmetryInterface");d(c$,"setSymmetryRange",function(a){this.symmetryRange=a;this.asc.setInfo("symmetryRange",Float.$valueOf(a))},"~N");d(c$,"setLatticeCells",function(){this.latticeCells=this.acr.latticeCells;var a=555>=this.latticeCells[0]&&555<=this.latticeCells[1]&&(0==this.latticeCells[2]||1==this.latticeCells[2]||-1==this.latticeCells[2]);this.doNormalize=0!=this.latticeCells[0]&&(!a||1==this.latticeCells[2]);this.applySymmetryToBonds=
this.acr.applySymmetryToBonds;this.doPackUnitCell=this.acr.doPackUnitCell&&!this.applySymmetryToBonds;this.doCentroidUnitCell=this.acr.doCentroidUnitCell;this.centroidPacked=this.acr.centroidPacked;this.filterSymop=this.acr.filterSymop});d(c$,"setUnitCell",function(a,b,c){this.unitCellParams=r(a.length,0);for(var f=0;f<a.length;f++)this.unitCellParams[f]=a[f];this.asc.haveUnitCell=!0;this.asc.setCurrentModelInfo("unitCellParams",this.unitCellParams);this.asc.isTrajectory&&(null==this.trajectoryUnitCells&&
(this.trajectoryUnitCells=new JU.Lst,this.asc.setInfo("unitCells",this.trajectoryUnitCells)),this.trajectoryUnitCells.addLast(this.unitCellParams));this.asc.setGlobalBoolean(2);this.getSymmetry().setUnitCell(this.unitCellParams,!1);null!=c&&(this.symmetry.setOffsetPt(c),this.asc.setCurrentModelInfo("unitCellOffset",c));null!=b&&(this.symmetry.initializeOrientation(b),this.asc.setCurrentModelInfo("matUnitCellOrientation",b))},"~A,JU.M3,JU.P3");d(c$,"addSpaceGroupOperation",function(a,b){b&&this.setLatticeCells();
this.symmetry.setSpaceGroup(this.doNormalize);return this.symmetry.addSpaceGroupOperation(a,0)},"~S,~B");d(c$,"setLatticeParameter",function(a){this.symmetry.setSpaceGroup(this.doNormalize);this.symmetry.setLattice(a)},"~N");d(c$,"applySymmetryFromReader",function(a){this.asc.setCoordinatesAreFractional(this.acr.iHaveFractionalCoordinates);this.setUnitCell(this.acr.unitCellParams,this.acr.matUnitCellOrientation,this.acr.unitCellOffset);this.setAtomSetSpaceGroupName(this.acr.sgName);this.setSymmetryRange(this.acr.symmetryRange);
if(this.acr.doConvertToFractional||this.acr.fileCoordinatesAreFractional){this.setLatticeCells();var b=!0;if(this.acr.ignoreFileSpaceGroupName||!this.acr.iHaveSymmetryOperators){if(!this.acr.merging||null==a)a=this.acr.getNewSymmetry();b=a.createSpaceGroup(this.acr.desiredSpaceGroupIndex,0<=this.acr.sgName.indexOf("!")?"P1":this.acr.sgName,this.acr.unitCellParams,this.acr.modDim)}else this.acr.doPreSymmetry(),a=null;this.packingError=this.acr.packingError;b&&(null!=a&&this.setSpaceGroupFrom(a),this.applySymmetryLattice(),
null!=a&&null==this.filterSymop&&this.setAtomSetSpaceGroupName(a.getSpaceGroupName()))}if(this.acr.iHaveFractionalCoordinates&&this.acr.merging&&null!=a){for(var b=this.asc.atoms,c=this.asc.getLastAtomSetAtomIndex(),f=this.asc.ac;c<f;c++)a.toCartesian(b[c],!0);this.asc.setCoordinatesAreFractional(!1);this.acr.addVibrations=!1}return this.symmetry},"J.api.SymmetryInterface");d(c$,"setSpaceGroupFrom",function(a){this.getSymmetry().setSpaceGroupFrom(a)},"J.api.SymmetryInterface");d(c$,"setAtomSetSpaceGroupName",
function(a){this.symmetry.setSpaceGroupName(a);this.asc.setCurrentModelInfo("spaceGroup",a+"")},"~S");d(c$,"applySymmetryLattice",function(){if(this.asc.coordinatesAreFractional&&null!=this.symmetry.getSpaceGroup()){this.sym2=null;var a=this.latticeCells[0],b=this.latticeCells[1],c=Math.abs(this.latticeCells[2]),f=this.latticeCells[3],h=p(this.symmetry.getUnitCellInfoType(6));this.firstAtom=this.asc.getLastAtomSetAtomIndex();var e=this.asc.bsAtoms;null!=e&&(this.updateBSAtoms(),this.firstAtom=e.nextSetBit(this.firstAtom));
this.rminx=this.rminy=this.rminz=3.4028235E38;this.rmaxx=this.rmaxy=this.rmaxz=-3.4028235E38;var F=null;null==this.acr.latticeType&&(this.acr.latticeType=this.symmetry.getLatticeType());this.acr.isPrimitive&&(this.asc.setCurrentModelInfo("isprimitive",Boolean.TRUE),(!"P".equals(this.acr.latticeType)||null!=this.acr.primitiveToCrystal)&&this.asc.setCurrentModelInfo("unitcell_conventional",this.symmetry.getConventionalUnitCell(this.acr.latticeType,this.acr.primitiveToCrystal)));if(null!=this.acr.latticeType&&
(this.asc.setCurrentModelInfo("latticeType",this.acr.latticeType),z(this.acr.fillRange,String))){var d=this.acr.fillRange;d.equals("conventional")?this.acr.fillRange=this.symmetry.getConventionalUnitCell(this.acr.latticeType,this.acr.primitiveToCrystal):d.equals("primitive")?(this.acr.fillRange=this.symmetry.getUnitCellVectors(),this.symmetry.toFromPrimitive(!0,this.acr.latticeType.charAt(0),this.acr.fillRange,this.acr.primitiveToCrystal)):this.acr.fillRange=null;null!=this.acr.fillRange&&this.acr.addJmolScript("unitcell "+
d)}if(null!=this.acr.fillRange){e=this.updateBSAtoms();this.acr.forcePacked=!0;this.doPackUnitCell=!1;this.minXYZ=new JU.P3i;this.maxXYZ=JU.P3i.new3(1,1,1);for(var d=Array(4),n=0;4>n;n++)d[n]=JU.P3.newP(this.acr.fillRange[n]);this.adjustRangeMinMax(d);null==this.sym2&&(this.sym2=new JS.Symmetry,this.sym2.getUnitCell(this.acr.fillRange,!1,null));this.applyAllSymmetry(this.acr.ms,e);for(var F=new JU.P3,j=this.asc.atoms,n=this.asc.ac;--n>=this.firstAtom;)F.setT(j[n]),this.symmetry.toCartesian(F,!1),
this.sym2.toFractional(F,!1),this.acr.fixJavaFloat&&JU.PT.fixPtFloats(F,1E5),this.isWithinCell(this.ndims,F,0,1,0,1,0,1,this.packingError)||e.clear(n)}else{this.nVib=0;var m=null,k=null,q=null;this.baseSymmetry=this.symmetry;var g=this.acr.strSupercell,d=null;null!=g&&0<=g.indexOf(",")&&(d=this.symmetry.getV0abc(g),null!=d&&(this.setMinMax(h,f,a,b,c),F=JU.P3.newP(d[0]),m=JU.P3.newP(d[1]),k=JU.P3.newP(d[2]),q=JU.P3.newP(d[3]),this.adjustRangeMinMax(d)));var x=this.asc.getLastAtomSetAtomIndex();null!=
e&&(x=e.nextSetBit(x));if(3.4028235E38==this.rminx)d=null;else{this.doPackUnitCell=n=this.doPackUnitCell;e=this.updateBSAtoms();this.applyAllSymmetry(this.acr.ms,null);this.doPackUnitCell=n;for(var j=this.asc.atoms,s=this.asc.ac,n=x;n<s;n++)this.symmetry.toCartesian(j[n],!0),e.set(n);this.symmetry=null;this.symmetry=this.getSymmetry();this.setUnitCell(r(-1,[0,0,0,0,0,0,m.x,m.y,m.z,k.x,k.y,k.z,q.x,q.y,q.z]),null,null);this.setAtomSetSpaceGroupName(null==d||null==g?"P1":"cell="+g);this.symmetry.setSpaceGroup(this.doNormalize);
this.symmetry.addSpaceGroupOperation("x,y,z",0);null!=F&&this.symmetry.toFractional(F,!0);for(n=x;n<s;n++)this.symmetry.toFractional(j[n],!0),null!=F&&j[n].sub(F);this.asc.haveAnisou=!1;this.asc.setCurrentModelInfo("matUnitCellOrientation",null)}this.setMinMax(h,f,a,b,c);if(null==d){this.applyAllSymmetry(this.acr.ms,e);if(!this.applySymmetryToBonds||!this.acr.doPackUnitCell)return;this.setMinMax(h,f,a,b,c)}(this.acr.forcePacked||this.acr.doPackUnitCell)&&this.trimToUnitCell(x)}}});d(c$,"setMinMax",
function(a,b,c,f,h){this.minXYZ=new JU.P3i;this.maxXYZ=JU.P3i.new3(c,f,h);JU.SimpleUnitCell.setMinMaxLatticeParameters(a,this.minXYZ,this.maxXYZ,b)},"~N,~N,~N,~N,~N");d(c$,"trimToUnitCell",function(a){var b=this.asc.atoms,c=this.updateBSAtoms();for(a=c.nextSetBit(a);0<=a;a=c.nextSetBit(a+1))this.isWithinCell(this.ndims,b[a],this.minXYZ.x,this.maxXYZ.x,this.minXYZ.y,this.maxXYZ.y,this.minXYZ.z,this.maxXYZ.z,this.packingError)||c.clear(a)},"~N");d(c$,"updateBSAtoms",function(){var a=this.asc.bsAtoms;
null==a&&(a=this.asc.bsAtoms=JU.BSUtil.newBitSet2(0,this.asc.ac));0>a.nextSetBit(this.firstAtom)&&a.setBits(this.firstAtom,this.asc.ac);return a});d(c$,"adjustRangeMinMax",function(a){var b=new JU.P3,c=new JU.P3,f=new JU.P3;this.acr.forcePacked&&(b.setT(a[1]),c.setT(a[2]),f.setT(a[3]),b.scale(this.packingError),c.scale(this.packingError),f.scale(this.packingError));a[0].scaleAdd2(this.minXYZ.x,a[1],a[0]);a[0].scaleAdd2(this.minXYZ.y,a[2],a[0]);a[0].scaleAdd2(this.minXYZ.z,a[3],a[0]);a[0].sub(b);a[0].sub(c);
a[0].sub(f);var h=JU.P3.newP(a[0]);this.symmetry.toFractional(h,!0);this.setSymmetryMinMax(h);a[1].scale(this.maxXYZ.x-this.minXYZ.x);a[2].scale(this.maxXYZ.y-this.minXYZ.y);a[3].scale(this.maxXYZ.z-this.minXYZ.z);a[1].scaleAdd2(2,b,a[1]);a[2].scaleAdd2(2,c,a[2]);a[3].scaleAdd2(2,f,a[3]);for(b=0;3>b;b++)for(c=b+1;4>c;c++)h.add2(a[b],a[c]),0!=b&&h.add(a[0]),this.symmetry.toFractional(h,!1),this.setSymmetryMinMax(h);this.symmetry.toCartesian(h,!1);h.add(a[1]);this.symmetry.toFractional(h,!1);this.setSymmetryMinMax(h);
this.minXYZ=JU.P3i.new3(u(Math.min(0,Math.floor(this.rminx+0.001))),u(Math.min(0,Math.floor(this.rminy+0.001))),u(Math.min(0,Math.floor(this.rminz+0.001))));this.maxXYZ=JU.P3i.new3(u(Math.max(1,Math.ceil(this.rmaxx-0.001))),u(Math.max(1,Math.ceil(this.rmaxy-0.001))),u(Math.max(1,Math.ceil(this.rmaxz-0.001))))},"~A");d(c$,"setSymmetryMinMax",function(a){this.rminx>a.x&&(this.rminx=a.x);this.rminy>a.y&&(this.rminy=a.y);this.rminz>a.z&&(this.rminz=a.z);this.rmaxx<a.x&&(this.rmaxx=a.x);this.rmaxy<a.y&&
(this.rmaxy=a.y);this.rmaxz<a.z&&(this.rmaxz=a.z)},"JU.P3");d(c$,"isWithinCell",function(a,b,c,f,h,e,F,d,n){return b.x>c-n&&b.x<f+n&&(2>a||b.y>h-n&&b.y<e+n)&&(3>a||b.z>F-n&&b.z<d+n)},"~N,JU.P3,~N,~N,~N,~N,~N,~N,~N");d(c$,"applyAllSymmetry",function(a,b){if(!(0==this.asc.ac||null!=b&&b.isEmpty())){var c=this.noSymmetryCount=0<this.asc.baseSymmetryAtomCount?this.asc.baseSymmetryAtomCount:null==b?this.asc.getLastAtomSetAtomCount():this.asc.ac-b.nextSetBit(this.asc.getLastAtomSetAtomIndex());this.asc.setTensors();
this.applySymmetryToBonds=this.acr.applySymmetryToBonds;this.doPackUnitCell=this.acr.doPackUnitCell&&!this.applySymmetryToBonds;this.bondCount0=this.asc.bondCount;this.finalizeSymmetry(this.symmetry);var f=this.symmetry.getSpaceGroupOperationCount(),h=null==this.acr.thisBiomolecule?null:new JU.BS;null!=h&&(this.asc.checkSpecial=!0);this.ndims=p(this.symmetry.getUnitCellInfoType(6));JU.SimpleUnitCell.setMinMaxLatticeParameters(this.ndims,this.minXYZ,this.maxXYZ,0);this.latticeOp=this.symmetry.getLatticeOp();
this.latticeOnly=this.asc.checkLatticeOnly&&0<=this.latticeOp;this.doCentroidUnitCell&&this.asc.setInfo("centroidMinMax",Q(-1,[this.minXYZ.x,this.minXYZ.y,this.minXYZ.z,this.maxXYZ.x,this.maxXYZ.y,this.maxXYZ.z,this.centroidPacked?1:0]));if(this.doCentroidUnitCell||this.acr.doPackUnitCell||0!=this.symmetryRange&&1==this.maxXYZ.x-this.minXYZ.x&&1==this.maxXYZ.y-this.minXYZ.y&&1==this.maxXYZ.z-this.minXYZ.z)switch(this.minXYZ0=JU.P3.new3(this.minXYZ.x,this.minXYZ.y,this.minXYZ.z),this.maxXYZ0=JU.P3.new3(this.maxXYZ.x,
this.maxXYZ.y,this.maxXYZ.z),null!=a&&(a.setMinMax0(this.minXYZ0,this.maxXYZ0),this.minXYZ.set(p(this.minXYZ0.x),p(this.minXYZ0.y),p(this.minXYZ0.z)),this.maxXYZ.set(p(this.maxXYZ0.x),p(this.maxXYZ0.y),p(this.maxXYZ0.z))),this.ndims){case 3:this.minXYZ.z--,this.maxXYZ.z++;case 2:this.minXYZ.y--,this.maxXYZ.y++;case 1:this.minXYZ.x--,this.maxXYZ.x++}for(var e=(this.maxXYZ.x-this.minXYZ.x)*(this.maxXYZ.y-this.minXYZ.y)*(this.maxXYZ.z-this.minXYZ.z),F=c*(this.latticeOnly?4:f),F=this.asc.checkSpecial||
null!=this.acr.thisBiomolecule?F*e:0<this.symmetryRange?F:1,d=Array(F),n=this.asc.atoms,j=0;j<c;j++)n[this.firstAtom+j].bsSymmetry=JU.BS.newN(f*(e+1));var m=0;this.unitCellTranslations=Array(e);var k=0,q=0,g=Math.abs(this.symmetryRange),x=0!=this.symmetryRange,s=0>this.symmetryRange,u=0<this.symmetryRange;x&&(this.rminx=this.rminy=this.rminz=3.4028235E38,this.rmaxx=this.rmaxy=this.rmaxz=-3.4028235E38);var t=this.symmetry,w=t;this.checkAll=this.latticeOnly||1==this.asc.atomSetCount&&this.asc.checkSpecial&&
0<=this.latticeOp;var y=this.acr.lstNCS;if(null!=y&&0==y.get(0).m33){for(var ca=t.getSpaceGroupOperationCount(),O=y.size(),j=O;0<=--j;){var A=y.get(j);A.m33=1;t.toFractionalM(A)}for(j=1;j<ca;j++)for(var H=t.getSpaceGroupOperation(j),T=0;T<O;T++)A=JU.M4.newM4(y.get(T)),A.mul2(H,A),this.doNormalize&&JS.SymmetryOperation.setOffset(A,n,this.firstAtom,this.noSymmetryCount),y.addLast(A)}j=null;y=t.getSpaceGroupOperation(0);this.doPackUnitCell&&(j=new JU.P3,this.ptOffset.set(0,0,0));ca=this.bondCount0>this.asc.bondIndex0&&
this.applySymmetryToBonds?Q(c,0):null;e=Q(e,0);for(O=this.minXYZ.x;O<this.maxXYZ.x;O++)for(A=this.minXYZ.y;A<this.maxXYZ.y;A++)for(H=this.minXYZ.z;H<this.maxXYZ.z;H++)if(this.unitCellTranslations[k]=JU.V3.new3(O,A,H),e[k++]=555+100*O+10*A+H,!(0!=O||0!=A||0!=H||0==d.length)){for(m=0;m<c;m++){q=n[this.firstAtom+m];null!=a&&(t=a.getAtomSymmetry(q,this.symmetry),t!==w&&(0==t.getSpaceGroupOperationCount()&&this.finalizeSymmetry(w=t),y=t.getSpaceGroupOperation(0)));T=JU.P3.newP(q);y.rotTrans(T);t.toCartesian(T,
!1);if(this.doPackUnitCell)if(t.toUnitCell(T,this.ptOffset),j.setT(T),t.toFractional(j,!1),this.acr.fixJavaFloat&&JU.PT.fixPtFloats(j,1E5),null==b)q.setT(j);else if(1E-4>q.distance(j))b.set(q.index);else{b.clear(q.index);continue}null!=b&&q.bsSymmetry.clearAll();q.bsSymmetry.set(k*f);q.bsSymmetry.set(0);x&&this.setSymmetryMinMax(T);m<F&&(d[m]=T)}s&&(this.rminx-=g,this.rminy-=g,this.rminz-=g,this.rmaxx+=g,this.rmaxy+=g,this.rmaxz+=g);q=m=this.symmetryAddAtoms(0,0,0,0,m,k*f,d,a,h,ca)}u&&(this.rminx-=
g,this.rminy-=g,this.rminz-=g,this.rmaxx+=g,this.rmaxy+=g,this.rmaxz+=g);k=0;for(O=this.minXYZ.x;O<this.maxXYZ.x;O++)for(A=this.minXYZ.y;A<this.maxXYZ.y;A++)for(H=this.minXYZ.z;H<this.maxXYZ.z;H++)if(k++,0!=O||0!=A||0!=H)m=this.symmetryAddAtoms(O,A,H,q,m,k*f,d,a,h,ca);k*c==this.asc.ac-this.firstAtom&&this.duplicateAtomProperties(k);this.setSymmetryOps();this.asc.setCurrentModelInfo("presymmetryAtomIndex",Integer.$valueOf(this.firstAtom));this.asc.setCurrentModelInfo("presymmetryAtomCount",Integer.$valueOf(c));
this.asc.setCurrentModelInfo("latticeDesignation",t.getLatticeDesignation());this.asc.setCurrentModelInfo("unitCellRange",e);this.asc.setCurrentModelInfo("unitCellTranslations",this.unitCellTranslations);this.baseUnitCell=this.unitCellParams;this.unitCellParams=r(6,0);this.reset()}},"J.adapter.smarter.MSInterface,JU.BS");d(c$,"symmetryAddAtoms",function(a,b,c,f,h,e,d,l,n,j){var m=0==f,k=null!=j;this.doPackUnitCell&&this.ptOffset.set(a,b,c);var g=this.symmetryRange*this.symmetryRange,v=0>this.symmetryRange,
x=0<this.symmetryRange,s=m&&x,x=(new Boolean(x&!m)).valueOf(),p=this.symmetry.getSpaceGroupOperationCount(),t=this.acr.lstNCS,w=null==t?0:t.size(),y=p+w,w=u(w/p),r=1==y&&!this.doPackUnitCell?!1:this.asc.checkSpecial,O=v||x,A=r||O,H=null!=n,T=r||s,z=this.acr.isMolecular?null:this.asc.bsAtoms,E=this.symmetry;v&&(f=this.noSymmetryCount);var v=this.firstAtom+this.noSymmetryCount,W=new JU.P3,D=null,Q=H?0.01:1E-4,M="\x00",R=null==z?this.firstAtom:z.nextSetBit(this.firstAtom),L=0;a:for(;L<y;L++)if(!(m&&
0==L||this.latticeOnly&&0<L&&0!=L%this.latticeOp||null!=n&&n.get(L))){for(var N=this.firstAtom+(r||null!=n?h:x?f:0),K=L>=p?0:0==this.asc.vibScale?E.getSpinOp(L):this.asc.vibScale,B=Math.max(this.firstAtom,null==z?0:z.nextSetBit(0)),G=A,Y=L>=p?u((L-p)/w):L,V=Y+e;B<v;B++){var S=this.asc.atoms[B];if(!(S.ignoreSymmetry||null!=z&&!z.get(B))){null==l?E.newSpaceGroupPoint(L,S,W,a,b,c,L>=p?t.get(L-p):null):(E=l.getAtomSymmetry(S,this.symmetry),E.newSpaceGroupPoint(L,S,W,a,b,c,null),D=E.getSpaceGroupOperationCode(L),
null!=D&&(M=D.charAt(0),E=l.getSymmetryFromCode(D),0==E.getSpaceGroupOperationCount()&&this.finalizeSymmetry(E)));this.acr.fixJavaFloat&&JU.PT.fixPtFloats(W,1E5);var P=JU.P3.newP(W);E.toCartesian(P,!1);if(this.doPackUnitCell&&(E.toUnitCell(P,this.ptOffset),W.setT(P),E.toFractional(W,!1),this.acr.fixJavaFloat&&JU.PT.fixPtFloats(W,1E5),!this.isWithinCell(this.ndims,W,this.minXYZ0.x,this.maxXYZ0.x,this.minXYZ0.y,this.maxXYZ0.y,this.minXYZ0.z,this.maxXYZ0.z,this.packingError)))continue;s&&this.setSymmetryMinMax(P);
var C=null;if(G){if(O&&(P.x<this.rminx||P.y<this.rminy||P.z<this.rminz||P.x>this.rmaxx||P.y>this.rmaxy||P.z>this.rmaxz))continue;for(var U=3.4028235E38,X=this.checkAll?this.asc.ac:N,Z=S.atomName,ba=null==D?S.altLoc:M,I=R;I<X;I++)if(null==z||z.get(I)){var $=d[I-this.firstAtom];if(null!=$){$=P.distanceSquared($);if(r&&$<Q){if(H){n.set(L);continue a}C=this.asc.atoms[I];if((null==C.atomName||C.atomName.equals(Z))&&C.altLoc==ba)break;C=null}x&&(I<f&&$<U)&&(U=$)}}if(x&&U>g)continue}H&&(G=!1);I=S.atomSite;
if(null!=C)k&&(j[I]=C.index),C.bsSymmetry.set(V),C.bsSymmetry.set(Y);else if(k&&(j[I]=this.asc.ac),C=S.copyTo(W,this.asc),null!=this.asc.bsAtoms&&this.asc.bsAtoms.set(C.index),0!=K&&null!=C.vib&&(E.getSpaceGroupOperation(L).rotate(C.vib),C.vib.scale(K)),C.atomSite=I,null!=D&&(C.altLoc=M),C.bsSymmetry=JU.BSUtil.newAndSetBit(V),C.bsSymmetry.set(Y),T&&(d[h++]=P),S=S.tensors,null!=S){C.tensors=null;for(I=S.size();0<=--I;)P=S.get(I),null!=P&&(1==p?C.addTensor(P.copyTensor(),null,!1):this.addRotatedTensor(C,
P,L,!1,E))}}}if(k)for(var N=this.asc.bonds,K=this.asc.atoms,aa,G=this.asc.bondIndex0;G<this.bondCount0;G++)if(Y=N[G],C=K[Y.atomIndex1],B=K[Y.atomIndex2],!(null==C||null==B||B.atomSetIndex<C.atomSetIndex))if(V=j[C.atomSite],I=j[B.atomSite],V>I&&(B=V,V=I,I=B),V!=I&&(V>=v||I>=v)&&0>this.bondsFound.indexOf(aa="-"+V+","+I))this.bondsFound.append(aa),this.asc.addNewBondWithOrder(V,I,Y.order)}return h},"~N,~N,~N,~N,~N,~N,~A,J.adapter.smarter.MSInterface,JU.BS,~A");d(c$,"duplicateAtomProperties",function(a){var b=
this.asc.getAtomSetAuxiliaryInfoValue(-1,"atomProperties");if(null!=b)for(var c,f=b.entrySet().iterator();f.hasNext()&&((c=f.next())||1);){var h=c.getKey(),e=c.getValue();if(z(e,String)){for(var d=e,l=new JU.SB,e=a;0<=--e;)l.append(d);b.put(h,l.toString())}else{h=e;d=r(h.length*a,0);for(e=a;0<=--e;)Zystem.arraycopy(h,0,d,e*h.length,h.length)}}},"~N");d(c$,"finalizeSymmetry",function(a){var b=this.asc.getAtomSetAuxiliaryInfoValue(-1,"spaceGroup");a.setFinalOperations(b,this.asc.atoms,this.firstAtom,
this.noSymmetryCount,this.doNormalize,this.filterSymop);(null!=this.filterSymop||null==b||b.equals("unspecified!"))&&this.setAtomSetSpaceGroupName(a.getSpaceGroupName())},"J.api.SymmetryInterface");d(c$,"setSymmetryOps",function(){var a=this.symmetry.getSpaceGroupOperationCount();if(0<a){for(var b=Array(a),c=0;c<a;c++)b[c]=""+this.symmetry.getSpaceGroupXyz(c,this.doNormalize);this.asc.setCurrentModelInfo("symmetryOperations",b);this.asc.setCurrentModelInfo("symmetryOps",this.symmetry.getSymmetryOperations())}this.asc.setCurrentModelInfo("symmetryCount",
Integer.$valueOf(a));this.asc.setCurrentModelInfo("latticeType",null==this.acr.latticeType?"P":this.acr.latticeType);this.asc.setCurrentModelInfo("intlTableNo",this.symmetry.getIntTableNumber());if(null==this.acr.sgName||0<=this.acr.sgName.indexOf("?")||0<=this.acr.sgName.indexOf("!"))this.setAtomSetSpaceGroupName(this.acr.sgName=this.symmetry.getSpaceGroupName())});d(c$,"getOverallSpan",function(){return null==this.maxXYZ0?JU.V3.new3(this.maxXYZ.x-this.minXYZ.x,this.maxXYZ.y-this.minXYZ.y,this.maxXYZ.z-
this.minXYZ.z):JU.V3.newVsub(this.maxXYZ0,this.minXYZ0)});d(c$,"applySymmetryBio",function(a,b,c){var f=a.get("biomts");if(!(2>f.size())){this.acr.lstNCS=null;this.setLatticeCells();var h=null!=this.latticeCells&&0!=this.latticeCells[0]?Q(3,0):null;if(null!=h)for(var e=0;3>e;e++)h[e]=this.latticeCells[e];this.latticeCells=null;var e=this.acr.getFilter("BMCHAINS"),d=null==e?-1:2>e.length?0:e.charCodeAt(1)-48,l=0<=c.indexOf("BYCHAIN")?1:0<=c.indexOf("BYSYMOP")?2:0;this.doNormalize=!1;var n=a.get("chains");
n.get(0).equals(n.get(1))&&(n=null);this.symmetry=null;this.getSymmetry().setSpaceGroup(this.doNormalize);this.addSpaceGroupOperation("x,y,z",!1);e=a.get("name");this.setAtomSetSpaceGroupName(this.acr.sgName=e);h=f.size();this.applySymmetryToBonds=b;this.bondCount0=this.asc.bondCount;this.firstAtom=this.asc.getLastAtomSetAtomIndex();var j=this.asc.ac,m=new java.util.Hashtable,k=0,g=this.asc.atoms;b=this.bondCount0>this.asc.bondIndex0&&b;switch(l){case 1:for(e=j;--e>=this.firstAtom;){var v=Integer.$valueOf(g[e].chainID);
b=m.get(v);null==b&&(k++,m.put(v,b=new JU.BS));b.set(e)}this.asc.bsAtoms=new JU.BS;for(e=0;e<k;e++)this.asc.bsAtoms.set(j+e),v=new J.adapter.smarter.Atom,v.set(0,0,0),v.radius=16,this.asc.addAtom(v);for(var x=0,s,m=m.entrySet().iterator();m.hasNext()&&((s=m.next())||1);){v=g[j+x++];b=s.getValue();for(e=b.nextSetBit(0);0<=e;e=b.nextSetBit(e+1))v.add(g[e]);v.scale(1/b.cardinality());v.atomName="Pt"+x;v.chainID=s.getKey().intValue()}this.firstAtom=j;j+=k;b=!1;break;case 2:this.asc.bsAtoms=new JU.BS;
this.asc.bsAtoms.set(j);v=g[j]=new J.adapter.smarter.Atom;v.set(0,0,0);for(e=j;--e>=this.firstAtom;)v.add(g[e]);v.scale(1/(j-this.firstAtom));v.atomName="Pt";v.radius=16;this.asc.addAtom(v);this.firstAtom=j++;b=!1}a=a.get("asemblyIdAtoms");0<=c.indexOf("#<")&&(h=Math.min(h,JU.PT.parseInt(c.substring(c.indexOf("#<")+2))-1),c=JU.PT.rep(c,"#<","_<"));k=g[this.firstAtom].chainID;v=0;for(m=this.firstAtom;m<j;m++)g[m].bsSymmetry=JU.BSUtil.newAndSetBit(0),e=g[m].chainID,e>v&&(v=e);for(var x=this.asc.bsAtoms,
p=b?Q(this.asc.ac,0):null,e=null==n?1:0;e<h;e++){if(0<=c.indexOf("!#")){if(0<=c.indexOf("!#"+(e+1)+";"))continue}else if(0<=c.indexOf("#")&&0>c.indexOf("#"+(e+1)+";"))continue;var t=f.get(e),w=null==n?null:n.get(e);if(null!=w&&null!=a){x=new JU.BS;for(m=a.entrySet().iterator();m.hasNext()&&((s=m.next())||1);)0<=w.indexOf(":"+s.getKey()+";")&&x.or(s.getValue());null!=this.asc.bsAtoms&&x.and(this.asc.bsAtoms);w=null}for(m=this.firstAtom;m<j;m++)if(!(null!=x&&!x.get(m)||null!=w&&0>w.indexOf(":"+this.acr.vwr.getChainIDStr(g[m].chainID)+
";")))try{var y=g[m].atomSite,r;b&&(p[y]=this.asc.ac);r=this.asc.newCloneAtom(g[m]);null!=this.asc.bsAtoms&&this.asc.bsAtoms.set(r.index);r.atomSite=y;t.rotTrans(r);r.bsSymmetry=JU.BSUtil.newAndSetBit(e)}catch(u){if(X(u,Exception))this.asc.errorMessage="appendAtomCollection error: "+u;else throw u;}if(0<e&&(this.symmetry.addBioMoleculeOperation(t,!1),b))for(m=this.asc.bondIndex0;m<this.bondCount0;m++)t=this.asc.bonds[m],this.asc.addNewBondWithOrder(p[g[t.atomIndex1].atomSite],p[g[t.atomIndex2].atomSite],
t.order)}null!=n&&(null==this.asc.bsAtoms&&(this.asc.bsAtoms=JU.BSUtil.newBitSet2(0,this.asc.ac)),this.asc.bsAtoms.clearBits(this.firstAtom,j));if(0==l&&0<=d){c=0<d?v-k-66:0;g=this.asc.atoms;e=j;for(f=this.asc.ac;e<f;e++)s=g[e].chainID,y=g[e].bsSymmetry.nextSetBit(0)+1,s=0==c?this.acr.vwr.getChainIDStr(s)+y:""+String.fromCharCode((s+c+y)%26+65),g[e].chainID=this.acr.vwr.getChainID(s,!0)}this.noSymmetryCount=j-this.firstAtom;this.asc.setCurrentModelInfo("presymmetryAtomIndex",Integer.$valueOf(this.firstAtom));
this.asc.setCurrentModelInfo("presymmetryAtomCount",Integer.$valueOf(this.noSymmetryCount));this.asc.setCurrentModelInfo("biosymmetryCount",Integer.$valueOf(h));this.asc.setCurrentModelInfo("biosymmetry",this.symmetry);this.finalizeSymmetry(this.symmetry);this.setSymmetryOps();this.reset()}},"java.util.Map,~B,~S");d(c$,"reset",function(){this.asc.coordinatesAreFractional=!1;this.asc.setCurrentModelInfo("hasSymmetry",Boolean.TRUE);this.asc.setGlobalBoolean(1)});d(c$,"addRotatedTensor",function(a,b,
c,f,h){null==this.ptTemp&&(this.ptTemp=new JU.P3,this.mTemp=new JU.M3);return a.addTensor(this.acr.getInterface("JU.Tensor").setFromEigenVectors(h.rotateAxes(c,b.eigenVectors,this.ptTemp,this.mTemp),b.eigenValues,b.isIsotropic?"iso":b.type,b.id,b),null,f)},"J.adapter.smarter.Atom,JU.Tensor,~N,~B,J.api.SymmetryInterface");d(c$,"setTensors",function(){for(var a=this.asc.ac,b=this.asc.getLastAtomSetAtomIndex();b<a;b++){var c=this.asc.atoms[b];null!=c.anisoBorU&&(c.addTensor(this.symmetry.getTensor(this.acr.vwr,
c.anisoBorU),null,!1),Float.isNaN(c.bfactor)&&(c.bfactor=100*c.anisoBorU[7]),c.anisoBorU=null)}});d(c$,"setTimeReversal",function(a,b){this.symmetry.setTimeReversal(a,b)},"~N,~N");d(c$,"setSpinVectors",function(){if(0<this.nVib||0>this.asc.iSet||!this.acr.vibsFractional)return this.nVib;for(var a=this.asc.getAtomSetAtomIndex(this.asc.iSet),b=this.getBaseSymmetry(),c=this.asc.ac;--c>=a;){var f=this.asc.atoms[c].vib;null!=f&&(0<f.modDim?f.setMoment():(f=f.clone(),b.toCartesian(f,!0),this.asc.atoms[c].vib=
f),this.nVib++)}return this.nVib});d(c$,"scaleFractionalVibs",function(){for(var a=this.getBaseSymmetry().getUnitCellParams(),a=JU.P3.new3(1/a[0],1/a[1],1/a[2]),b=this.asc.getAtomSetAtomIndex(this.asc.iSet),c=this.asc.ac;--c>=b;){var f=this.asc.atoms[c].vib;null!=f&&f.scaleT(a)}});d(c$,"getBaseSymmetry",function(){return null==this.baseSymmetry?this.symmetry:this.baseSymmetry});d(c$,"finalizeUnitCell",function(a){null!=a&&null!=this.baseUnitCell&&(this.baseUnitCell[22]=Math.max(1,p(a.x)),this.baseUnitCell[23]=
Math.max(1,p(a.y)),this.baseUnitCell[24]=Math.max(1,p(a.z)))},"JU.P3");G(c$,"PARTICLE_NONE",0,"PARTICLE_CHAIN",1,"PARTICLE_SYMOP",2)});M("J.api");ea(J.api,"SymmetryInterface");M("JS");B(["J.api.SymmetryInterface"],"JS.Symmetry","JU.BS $.Lst $.P3 J.api.Interface J.bspt.Bspt JS.PointGroup $.SpaceGroup $.SymmetryInfo $.SymmetryOperation $.UnitCell JU.Escape $.Logger $.SimpleUnitCell".split(" "),function(){c$=N(function(){this.unitCell=this.symmetryInfo=this.spaceGroup=this.pointGroup=null;this.$isBio=
!1;this.cip=this.desc=null;R(this,arguments)},JS,"Symmetry",null,J.api.SymmetryInterface);g(c$,"isBio",function(){return this.$isBio});D(c$,function(){});g(c$,"setPointGroup",function(a,b,c,f,h,e,d,l){this.pointGroup=JS.PointGroup.getPointGroup(null==a?null:a.pointGroup,b,c,f,h,e,d,l);return this},"J.api.SymmetryInterface,JU.T3,~A,JU.BS,~B,~N,~N,~B");g(c$,"getPointGroupName",function(){return this.pointGroup.getName()});g(c$,"getPointGroupInfo",function(a,b,c,f,h,e){return null==b&&!c&&null!=this.pointGroup.textInfo?
this.pointGroup.textInfo:null==b&&this.pointGroup.isDrawType(f,h,e)?this.pointGroup.drawInfo:c&&null!=this.pointGroup.info?this.pointGroup.info:this.pointGroup.getInfo(a,b,c,f,h,e)},"~N,~S,~B,~S,~N,~N");g(c$,"setSpaceGroup",function(a){null==this.spaceGroup&&(this.spaceGroup=JS.SpaceGroup.getNull(!0,a,!1))},"~B");g(c$,"addSpaceGroupOperation",function(a,b){return this.spaceGroup.addSymmetry(a,b,!1)},"~S,~N");g(c$,"addBioMoleculeOperation",function(a,b){this.$isBio=this.spaceGroup.isBio=!0;return this.spaceGroup.addSymmetry((b?
"!":"")+"[[bio"+a,0,!1)},"JU.M4,~B");g(c$,"setLattice",function(a){this.spaceGroup.setLatticeParam(a)},"~N");d(c$,"getSpaceGroup",function(){return this.spaceGroup});g(c$,"setSpaceGroupFrom",function(a){this.spaceGroup=a.getSpaceGroup()},"J.api.SymmetryInterface");g(c$,"createSpaceGroup",function(a,b,c,f){this.spaceGroup=JS.SpaceGroup.createSpaceGroup(a,b,c,f);null!=this.spaceGroup&&JU.Logger.debugging&&JU.Logger.debug("using generated space group "+this.spaceGroup.dumpInfo());return null!=this.spaceGroup},
"~N,~S,~O,~N");g(c$,"getSpaceGroupInfoObj",function(a,b,c){return JS.SpaceGroup.getInfo(this.spaceGroup,a,b,c)},"~S,J.api.SymmetryInterface,~B");g(c$,"getLatticeDesignation",function(){return this.spaceGroup.getLatticeDesignation()});g(c$,"setFinalOperations",function(a,b,c,f,h,e){if(null!=a&&(a.startsWith("bio")||0<=a.indexOf(" *(")))this.spaceGroup.name=a;if(null!=e){var d=new JU.Lst;d.addLast(this.spaceGroup.operations[0]);for(var l=1;l<this.spaceGroup.operationCount;l++)e.contains(" "+(l+1)+" ")&&
d.addLast(this.spaceGroup.operations[l]);this.spaceGroup=JS.SpaceGroup.createSpaceGroup(-1,a+" *("+e.trim()+")",d,-1)}this.spaceGroup.setFinalOperations(b,c,f,h)},"~S,~A,~N,~N,~B,~S");g(c$,"getSpaceGroupOperation",function(a){return null==this.spaceGroup||null==this.spaceGroup.operations||a>=this.spaceGroup.operations.length?null:null==this.spaceGroup.finalOperations?this.spaceGroup.operations[a]:this.spaceGroup.finalOperations[a]},"~N");g(c$,"getSpaceGroupOperationRaw",function(a){return this.spaceGroup.getRawOperation(a)},
"~N");g(c$,"getSpaceGroupXyz",function(a,b){return this.spaceGroup.getXyz(a,b)},"~N,~B");g(c$,"newSpaceGroupPoint",function(a,b,c,f,h,e,d){null==d&&null==this.spaceGroup.finalOperations?(a=this.spaceGroup.operations[a],a.isFinalized||a.doFinalize(),JS.SymmetryOperation.newPoint(a,b,c,f,h,e)):JS.SymmetryOperation.newPoint(null==d?this.spaceGroup.finalOperations[a]:d,b,c,f,h,e)},"~N,JU.P3,JU.P3,~N,~N,~N,JU.M4");g(c$,"rotateAxes",function(a,b,c,f){return 0==a?b:this.spaceGroup.finalOperations[a].rotateAxes(b,
this.unitCell,c,f)},"~N,~A,JU.P3,JU.M3");g(c$,"getSpaceGroupOperationCode",function(a){return this.spaceGroup.operations[a].subZystemCode},"~N");g(c$,"setTimeReversal",function(a,b){this.spaceGroup.operations[a].setTimeReversal(b)},"~N,~N");g(c$,"getSpinOp",function(a){return this.spaceGroup.operations[a].getMagneticOp()},"~N");g(c$,"addLatticeVectors",function(a){return this.spaceGroup.addLatticeVectors(a)},"JU.Lst");g(c$,"getLatticeOp",function(){return this.spaceGroup.latticeOp});g(c$,"getOperationRsVs",
function(a){return(null==this.spaceGroup.finalOperations?this.spaceGroup.operations:this.spaceGroup.finalOperations)[a].rsvs},"~N");g(c$,"getSiteMultiplicity",function(a){return this.spaceGroup.getSiteMultiplicity(a,this.unitCell)},"JU.P3");g(c$,"addOp",function(a,b,c,f){this.spaceGroup.isSSG=!0;b=JS.SymmetryOperation.getXYZFromRsVs(b,c,!1);c=this.spaceGroup.addSymmetry(b,-1,!0);this.spaceGroup.operations[c].setSigma(a,f);return b},"~S,JU.Matrix,JU.Matrix,JU.Matrix");g(c$,"getMatrixFromString",function(a,
b,c){return JS.SymmetryOperation.getMatrixFromString(null,a,b,c)},"~S,~A,~B,~N");g(c$,"getSpaceGroupName",function(){return null!=this.symmetryInfo?this.symmetryInfo.sgName:null!=this.spaceGroup?this.spaceGroup.getName():null!=this.unitCell&&0<this.unitCell.name.length?"cell="+this.unitCell.name:""});g(c$,"setSpaceGroupName",function(a){null!=this.spaceGroup&&this.spaceGroup.setName(a)},"~S");g(c$,"getSpaceGroupOperationCount",function(){return null!=this.symmetryInfo?this.symmetryInfo.symmetryOperations.length:
null!=this.spaceGroup&&null!=this.spaceGroup.finalOperations?this.spaceGroup.finalOperations.length:0});g(c$,"getLatticeType",function(){return null!=this.symmetryInfo?this.symmetryInfo.latticeType:null==this.spaceGroup?"P":this.spaceGroup.latticeType});g(c$,"setLatticeType",function(a){null!=this.spaceGroup&&(this.spaceGroup.latticeType=a)},"~S");g(c$,"getIntTableNumber",function(){return null!=this.symmetryInfo?this.symmetryInfo.intlTableNo:null==this.spaceGroup?null:this.spaceGroup.intlTableNumber});
g(c$,"getCoordinatesAreFractional",function(){return null==this.symmetryInfo||this.symmetryInfo.coordinatesAreFractional});g(c$,"getCellRange",function(){return null==this.symmetryInfo?null:this.symmetryInfo.cellRange});g(c$,"getSymmetryInfoStr",function(){return null==this.symmetryInfo?"":this.symmetryInfo.infoStr});g(c$,"getSymmetryOperations",function(){if(null!=this.symmetryInfo)return this.symmetryInfo.symmetryOperations;null==this.spaceGroup&&(this.spaceGroup=JS.SpaceGroup.getNull(!0,!1,!0));
return this.spaceGroup.finalOperations});g(c$,"isSimple",function(){return null==this.symmetryInfo||null==this.symmetryInfo.symmetryOperations});g(c$,"setSymmetryInfo",function(a,b,c){this.symmetryInfo=new JS.SymmetryInfo;c=this.symmetryInfo.setSymmetryInfo(b,c);null!=c&&(this.setUnitCell(c,b.containsKey("jmolData")),this.unitCell.moreInfo=b.get("moreUnitCellInfo"),b.put("infoUnitCell",this.getUnitCellAsArray(!1)),this.setOffsetPt(b.get("unitCellOffset")),b=b.get("matUnitCellOrientation"),null!=b&&
this.initializeOrientation(b),JU.Logger.debugging&&JU.Logger.debug("symmetryInfos["+a+"]:\n"+this.unitCell.dumpInfo(!0)));return this},"~N,java.util.Map,~A");g(c$,"haveUnitCell",function(){return null!=this.unitCell});g(c$,"checkUnitCell",function(a,b,c,f){a.toFractional(c,f);return c.x>=b.x-1-0.02&&c.x<=b.x+0.02&&c.y>=b.y-1-0.02&&c.y<=b.y+0.02&&c.z>=b.z-1-0.02&&c.z<=b.z+0.02},"J.api.SymmetryInterface,JU.P3,JU.P3,~B");d(c$,"setUnitCell",function(a,b){this.unitCell=JS.UnitCell.fromParams(a,b)},"~A,~B");
g(c$,"unitCellEquals",function(a){return a.unitCell.isSameAs(this.unitCell)},"J.api.SymmetryInterface");g(c$,"getUnitCellState",function(){return null==this.unitCell?"":this.unitCell.getState()});g(c$,"getMoreInfo",function(){return this.unitCell.moreInfo});d(c$,"getUnitsymmetryInfo",function(){return this.unitCell.dumpInfo(!1)});g(c$,"initializeOrientation",function(a){this.unitCell.initOrientation(a)},"JU.M3");g(c$,"unitize",function(a){this.unitCell.unitize(a)},"JU.T3");g(c$,"toUnitCell",function(a,
b){this.unitCell.toUnitCell(a,b)},"JU.T3,JU.T3");g(c$,"toSupercell",function(a){return this.unitCell.toSupercell(a)},"JU.P3");d(c$,"toFractional",function(a,b){this.$isBio||this.unitCell.toFractional(a,b)},"JU.T3,~B");g(c$,"toFractionalM",function(a){this.$isBio||this.unitCell.toFractionalM(a)},"JU.M4");g(c$,"toCartesian",function(a,b){this.$isBio||this.unitCell.toCartesian(a,b)},"JU.T3,~B");g(c$,"getUnitCellParams",function(){return this.unitCell.getUnitCellParams()});g(c$,"getUnitCellAsArray",function(a){return this.unitCell.getUnitCellAsArray(a)},
"~B");g(c$,"getTensor",function(a,b){if(null==b)return null;null==this.unitCell&&(this.unitCell=JS.UnitCell.fromParams(r(-1,[1,1,1,90,90,90]),!0));return this.unitCell.getTensor(a,b)},"JV.Viewer,~A");g(c$,"getUnitCellVerticesNoOffset",function(){return this.unitCell.getVertices()});g(c$,"getCartesianOffset",function(){return this.unitCell.getCartesianOffset()});g(c$,"getFractionalOffset",function(){return this.unitCell.getFractionalOffset()});g(c$,"setOffsetPt",function(a){this.unitCell.setOffset(a)},
"JU.T3");g(c$,"setOffset",function(a){var b=new JU.P3;JU.SimpleUnitCell.ijkToPoint3f(a,b,0,0);this.unitCell.setOffset(b)},"~N");g(c$,"getUnitCellMultiplier",function(){return this.unitCell.getUnitCellMultiplier()});g(c$,"getCanonicalCopy",function(a,b){return this.unitCell.getCanonicalCopy(a,b)},"~N,~B");g(c$,"getUnitCellInfoType",function(a){return this.unitCell.getInfo(a)},"~N");g(c$,"getUnitCellInfo",function(){return this.unitCell.dumpInfo(!1)});g(c$,"isSlab",function(){return this.unitCell.isSlab()});
g(c$,"isPolymer",function(){return this.unitCell.isPolymer()});g(c$,"checkDistance",function(a,b,c,f,h,e,d,l){return this.unitCell.checkDistance(a,b,c,f,h,e,d,l)},"JU.P3,JU.P3,~N,~N,~N,~N,~N,JU.P3");g(c$,"getUnitCellVectors",function(){return this.unitCell.getUnitCellVectors()});g(c$,"getUnitCell",function(a,b,c){if(null==a)return null;this.unitCell=JS.UnitCell.fromOABC(a,b);null!=c&&(this.unitCell.name=c);return this},"~A,~B,~S");g(c$,"isSupercell",function(){return this.unitCell.isSupercell()});
g(c$,"notInCentroid",function(a,b,c){try{var f=new JU.BS,h=b.nextSetBit(0),e=a.getMolecules(),d=e.length,l=a.at,n=e[d-1].firstAtomIndex==a.am[l[h].mi].firstAtomIndex,j=new JU.P3,g=1==c[6];a=d;a:for(;0<=--a&&b.get(e[a].firstAtomIndex);){var k=e[a].atomList;j.set(0,0,0);for(var h=0,q=k.nextSetBit(0);0<=q;q=k.nextSetBit(q+1))if(n||g)if(j.setT(l[q]),this.isNotCentroid(j,1,c,g))n&&f.set(q);else{if(!n)continue a}else j.add(l[q]),h++;(g||0<h&&this.isNotCentroid(j,h,c,!1))&&f.or(k)}return f}catch(v){if(X(v,
Exception))return null;throw v;}},"JM.ModelSet,JU.BS,~A");d(c$,"isNotCentroid",function(a,b,c,f){a.scale(1/b);this.toFractional(a,!1);return f?a.x+5E-6<=c[0]||a.x-5E-6>c[3]||a.y+5E-6<=c[1]||a.y-5E-6>c[4]||a.z+5E-6<=c[2]||a.z-5E-6>c[5]:a.x+5E-6<=c[0]||a.x+5E-5>c[3]||a.y+5E-6<=c[1]||a.y+5E-5>c[4]||a.z+5E-6<=c[2]||a.z+5E-5>c[5]},"JU.P3,~N,~A,~B");d(c$,"getDesc",function(a){return(null==this.desc?this.desc=J.api.Interface.getInterface("JS.SymmetryDesc",a.vwr,"eval"):this.desc).set(a)},"JM.ModelSet");
g(c$,"getSymmetryInfoAtom",function(a,b,c,f,h,e,d,l,n,j,g,k){return this.getDesc(a).getSymopInfo(b,c,f,h,e,d,l,n,j,g,k)},"JM.ModelSet,~N,~S,~N,JU.P3,JU.P3,JU.P3,~S,~N,~N,~N,~N");g(c$,"getSpaceGroupInfo",function(a,b,c,f,h){var e=null==b;if(null==b){var d=a.getModelAuxiliaryInfo(a.vwr.am.cmi);null!=d&&(b=d.get("spaceGroup"))}d=null;null!=h&&(d=new JS.Symmetry,d.setUnitCell(h,!1));return this.getDesc(a).getSpaceGroupInfo(this,c,b,0,null,null,null,0,-1,f,e,0,d)},"JM.ModelSet,~S,~N,~B,~A");g(c$,"fcoord",
function(a){return JS.SymmetryOperation.fcoord(a)},"JU.T3");g(c$,"getV0abc",function(a){return null==this.unitCell?null:this.unitCell.getV0abc(a)},"~O");g(c$,"getQuaternionRotation",function(a){return null==this.unitCell?null:this.unitCell.getQuaternionRotation(a)},"~S");g(c$,"getFractionalOrigin",function(){return this.unitCell.getFractionalOrigin()});g(c$,"getState",function(a){var b=this.getFractionalOffset(),c=!1;if(null!=b&&(0!=b.x||0!=b.y||0!=b.z))a.append("; set unitcell ").append(JU.Escape.eP(b)),
c=!0;b=this.getUnitCellMultiplier();null!=b&&(a.append("; set unitcell ").append(JU.SimpleUnitCell.escapeMultiplier(b)),c=!0);return c},"JU.SB");g(c$,"getIterator",function(a,b,c,f,h){return J.api.Interface.getInterface("JS.UnitCellIterator",a,"script").set(this,b,c,f,h)},"JV.Viewer,JM.Atom,~A,JU.BS,~N");g(c$,"toFromPrimitive",function(a,b,c,f){null==this.unitCell&&(this.unitCell=JS.UnitCell.fromOABC(c,!1));return this.unitCell.toFromPrimitive(a,b,c,f)},"~B,~S,~A,JU.M3");g(c$,"generateCrystalClass",
function(a){var b=this.getSymmetryOperations(),c=new JU.Lst,f=null==a,h=0,e=0,d=0;f?(h=2.718281828459045,e=3.141592653589793,d=Math.log10(2E3),a=JU.P3.new3(h+1,e+2,d+3)):a=JU.P3.newP(a);if(null==b||null==this.unitCell)c.addLast(a);else{this.unitCell.toFractional(a,!0);var l=null,g=null,j=null;f&&(l=JU.P3.new3(e+4,d+5,h+6),this.unitCell.toFractional(l,!0),g=JU.P3.new3(d+7,h+8,e+9),this.unitCell.toFractional(g,!0));var h=new J.bspt.Bspt(3,0),e=h.allocateCubeIterator(),d=new JU.P3,m=b.length;a:for(;0<=
--m;){b[m].rotate2(a,d);e.initialize(d,0.001,!1);if(e.hasMoreElements())continue a;j=JU.P3.newP(d);c.addLast(j);h.addTuple(j);f&&(null!=g&&(j=new JU.P3,b[m].rotate2(g,j),c.addLast(j)),null!=l&&(j=new JU.P3,b[m].rotate2(l,j),c.addLast(j)))}for(a=c.size();0<=--a;)this.unitCell.toCartesian(c.get(a),!0)}return c},"JU.P3");g(c$,"calculateCIPChiralityForAtoms",function(a,b){a.setCursor(3);var c=this.getCIPChirality(a),f=a.getBoolean(603979960)?"CIPData":"CIPDataTracker",f=J.api.Interface.getInterface("JS."+
f,a,"script").set(a,b);f.setRule6Full(a.getBoolean(603979823));c.getChiralityForAtoms(f);a.setCursor(0)},"JV.Viewer,JU.BS");g(c$,"calculateCIPChiralityForSmiles",function(a,b){a.setCursor(3);var c=this.getCIPChirality(a),f=J.api.Interface.getInterface("JS.CIPDataSmiles",a,"script").setAtomsForSmiles(a,b);c.getChiralityForAtoms(f);a.setCursor(0);return f.getSmilesChiralityArray()},"JV.Viewer,~S");d(c$,"getCIPChirality",function(a){return null==this.cip?this.cip=J.api.Interface.getInterface("JS.CIPChirality",
a,"script"):this.cip},"JV.Viewer");g(c$,"getConventionalUnitCell",function(a,b){return null==this.unitCell||null==a?null:this.unitCell.getConventionalUnitCell(a,b)},"~S,JU.M3");g(c$,"getUnitCellInfoMap",function(){return null==this.unitCell?null:this.unitCell.getInfo()});d(c$,"setUnitCell",function(a){this.unitCell=JS.UnitCell.cloneUnitCell(a.unitCell)},"JS.Symmetry")});M("JS");B(["JU.M3","$.V3"],"JS.PointGroup","java.lang.Float java.util.Hashtable JU.Lst $.P3 $.PT $.Quat $.SB J.bspt.Bspt JU.BSUtil $.Escape $.Logger $.Node $.Point3fi".split(" "),
function(){c$=N(function(){this.isAtoms=!1;this.iter=this.textInfo=this.info=this.drawInfo=null;this.drawType="";this.drawIndex=0;this.scale=NaN;this.axes=this.nAxes=null;this.radius=this.nAtoms=0;this.distanceTolerance=0.25;this.distanceTolerance2=0;this.linearTolerance=8;this.cosTolerance=0.99;this.name="C_1?";this.vTemp=this.principalPlane=this.principalAxis=null;this.centerAtomIndex=-1;this.haveInversionCenter=!1;this.bsAtoms=this.atomMap=this.elements=this.points=this.center=null;this.localEnvOnly=
this.haveVibration=!1;this.maxElement=0;this.eCounts=null;this.nOps=0;ha("JS.PointGroup.Operation")||JS.PointGroup.$PointGroup$Operation$();R(this,arguments)},JS,"PointGroup");U(c$,function(){this.nAxes=Q(JS.PointGroup.maxAxis,0);this.axes=Array(JS.PointGroup.maxAxis);this.vTemp=new JU.V3});d(c$,"getName",function(){return this.name});c$.getPointGroup=d(c$,"getPointGroup",function(a,b,c,f,h,e,d,l){var g=new JS.PointGroup;0==e&&(e=0.01,d=0.5);g.distanceTolerance=e;g.distanceTolerance2=e*e;g.linearTolerance=
d;g.isAtoms=null!=f;g.bsAtoms=g.isAtoms?f:JU.BSUtil.newBitSet2(0,c.length);g.haveVibration=h;g.center=b;g.localEnvOnly=l;return g.set(a,c)?g:a},"JS.PointGroup,JU.T3,~A,JU.BS,~B,~N,~N,~B");D(c$,function(){});d(c$,"isEqual",function(a){if(null==a||(this.linearTolerance!=a.linearTolerance||this.distanceTolerance!=a.distanceTolerance||this.nAtoms!=a.nAtoms||this.localEnvOnly!=a.localEnvOnly||this.haveVibration!=a.haveVibration||null==this.bsAtoms?null!=a.bsAtoms:!this.bsAtoms.equals(a.bsAtoms)))return!1;
for(var b=0;b<this.nAtoms;b++)if(this.elements[b]!=a.elements[b]||!this.points[b].equals(a.points[b]))return!1;return!0},"JS.PointGroup");d(c$,"set",function(a,b){this.cosTolerance=Math.cos(3.141592653589793*(this.linearTolerance/180));if(!this.getPointsAndElements(b))return JU.Logger.error("Too many atoms for point group calculation"),this.name="point group not determined -- ac > 100 -- select fewer atoms and try again.",!0;this.getElementCounts();if(this.haveVibration){for(var c=Array(this.points.length),
f=this.points.length;0<=--f;){c[f]=JU.P3.newP(this.points[f]);var h=this.points[f].getVibrationVector();null!=h&&c[f].add(h)}this.points=c}if(this.isEqual(a))return!1;try{this.findInversionCenter();if(this.isLinear(this.points))return this.name=this.haveInversionCenter?"D(infinity)h":"C(infinity)v",this.vTemp.sub2(this.points[1],this.points[0]),this.addAxis(16,this.vTemp),this.principalAxis=this.axes[16][0],this.haveInversionCenter&&(this.axes[0]=Array(1),this.principalPlane=this.axes[0][this.nAxes[0]++]=
Z(JS.PointGroup.Operation,this,null,this.vTemp)),!0;this.axes[0]=Array(15);c=0;this.findCAxes();c=this.findPlanes();this.findAdditionalAxes(c);var e=this.getHighestOrder();if(1<this.nAxes[17])this.name=1<this.nAxes[19]?this.haveInversionCenter?"Ih":"I":1<this.nAxes[18]?this.haveInversionCenter?"Oh":"O":0<c?this.haveInversionCenter?"Th":"Td":"T";else if(2>e){if(1==c)return this.name="Cs",!0;if(this.haveInversionCenter)return this.name="Ci",!0;this.name="C1"}else 1==e%2&&0<this.nAxes[16]||0==e%2&&1<
this.nAxes[16]?(this.principalAxis=this.setPrincipalAxis(e,c),0==c?this.name=14>e?"S"+e:"D"+(e-14):(e=14>e?u(e/2):e-14,this.name=c==e?"D"+e+"d":"D"+e+"h")):0==c?(this.principalAxis=this.axes[e][0],this.name=14>e?"S"+e:"C"+(e-14)):c==e-14?(this.principalAxis=this.axes[e][0],this.name="C"+c+"v"):(this.principalAxis=this.axes[14>e?e+14:e][0],this.principalPlane=this.axes[0][0],this.name="C"+(14>e?e/2:e-14)+"h")}catch(d){if(X(d,Exception))this.name="??";else throw d;}finally{JU.Logger.info("Point group found: "+
this.name)}return!0},"JS.PointGroup,~A");d(c$,"setPrincipalAxis",function(a,b){var c=this.setPrincipalPlane(a,b);if(0==b&&14>a||1==this.nAxes[a])return 0<b&&14>a&&(a=14+u(a/2)),this.axes[a][0];if(null==c)return null;for(var f=0;f<this.nAxes[16];f++)if(this.isParallel(c.normalOrAxis,this.axes[16][f].normalOrAxis))return 0!=f&&(c=this.axes[16][0],this.axes[16][0]=this.axes[16][f],this.axes[16][f]=c),this.axes[16][0];return null},"~N,~N");d(c$,"setPrincipalPlane",function(a,b){if(1==b)return this.principalPlane=
this.axes[0][0];if(0==b||b==a-14)return null;for(var c=0;c<b;c++)for(var f=0,h=0;f<b;f++)if(this.isPerpendicular(this.axes[0][c].normalOrAxis,this.axes[0][f].normalOrAxis)&&2<++h)return 0!=c&&(f=this.axes[0][0],this.axes[0][0]=this.axes[0][c],this.axes[0][c]=f),this.principalPlane=this.axes[0][0];return null},"~N,~N");d(c$,"getPointsAndElements",function(a){var b=this.bsAtoms.cardinality();if(this.isAtoms&&100<b)return!1;this.points=Array(b);this.elements=Q(b,0);if(0==b)return!0;for(var c=0,b=this.bsAtoms.nextSetBit(0);0<=
b;b=this.bsAtoms.nextSetBit(b+1)){var f=a[b];z(f,JU.Node)&&(c=Math.max(c,f.i))}this.atomMap=Q(c+1,0);this.nAtoms=0;if(c=null==this.center)this.center=new JU.P3;for(var h=new J.bspt.Bspt(3,0),b=this.bsAtoms.nextSetBit(0);0<=b;b=this.bsAtoms.nextSetBit(b+1),this.nAtoms++){f=a[b];if(z(f,JU.Node)){var e=this.localEnvOnly?1:1+Math.max(3,f.getCovalentBondCount());this.elements[this.nAtoms]=f.getElementNumber()*e;this.atomMap[f.i]=this.nAtoms+1}else e=new JU.Point3fi,e.setT(f),e.i=-1-this.nAtoms,z(f,JU.Point3fi)&&
(this.elements[this.nAtoms]=Math.max(0,f.sD)),f=e;h.addTuple(f);c&&this.center.add(f);this.points[this.nAtoms]=f}this.iter=h.allocateCubeIterator();c&&this.center.scale(1/this.nAtoms);for(b=this.nAtoms;0<=--b;)a=this.center.distanceSquared(this.points[b]),this.isAtoms&&a<this.distanceTolerance2&&(this.centerAtomIndex=b),this.radius=Math.max(this.radius,a);this.radius=Math.sqrt(this.radius);1.5>this.radius&&0.15<this.distanceTolerance&&(this.distanceTolerance=this.radius/10,this.distanceTolerance2=
this.distanceTolerance*this.distanceTolerance,Zystem.out.println("PointGroup calculation adjusting distanceTolerance to "+this.distanceTolerance));return!0},"~A");d(c$,"findInversionCenter",function(){if(this.haveInversionCenter=this.checkOperation(null,this.center,-1))this.axes[1]=Array(1),this.axes[1][0]=Z(JS.PointGroup.Operation,this,null)});d(c$,"checkOperation",function(a,b,c){var f=new JU.P3,h=0;c=14>c;var e=this.points.length,d=e;a:for(;0<=--d&&h<e;)if(d!=this.centerAtomIndex){var l=this.points[d],
g=this.elements[d];null!=a?(f.sub2(l,b),a.transform2(f,f).add(b)):f.setT(l);c&&(this.vTemp.sub2(b,f),f.scaleAdd2(2,this.vTemp,f));if((null!=a||c)&&f.distanceSquared(l)<this.distanceTolerance2)h++;else{for(this.iter.initialize(f,this.distanceTolerance,!1);this.iter.hasMoreElements();){var j=this.iter.nextElement();if(j!==l){var m=this.getPointIndex(j.i);if(!(0<=this.centerAtomIndex&&m==this.centerAtomIndex||m>=this.elements.length||this.elements[m]!=g)&&f.distanceSquared(j)<this.distanceTolerance2){h++;
continue a}}}return!1}}return!0},"JU.Quat,JU.T3,~N");d(c$,"getPointIndex",function(a){return(0>a?-a:this.atomMap[a])-1},"~N");d(c$,"isLinear",function(a){var b=null;if(2>a.length)return!1;for(var c=a.length;0<=--c;)if(c!=this.centerAtomIndex)if(null==b)b=new JU.V3,b.sub2(a[c],this.center),b.normalize(),this.vTemp.setT(b);else if(this.vTemp.sub2(a[c],this.center),this.vTemp.normalize(),!this.isParallel(b,this.vTemp))return!1;return!0},"~A");d(c$,"isParallel",function(a,b){return Math.abs(a.dot(b))>=
this.cosTolerance},"JU.V3,JU.V3");d(c$,"isPerpendicular",function(a,b){return Math.abs(a.dot(b))<=1-this.cosTolerance},"JU.V3,JU.V3");d(c$,"getElementCounts",function(){for(var a=this.points.length;0<=--a;){var b=this.elements[a];b>this.maxElement&&(this.maxElement=b)}this.eCounts=Q(++this.maxElement,0);for(a=this.points.length;0<=--a;)this.eCounts[this.elements[a]]++});d(c$,"findCAxes",function(){for(var a=new JU.V3,b=new JU.V3,c=new JU.V3,f=this.points.length;0<=--f;)if(f!=this.centerAtomIndex)for(var h=
this.points[f],e=this.elements[f],d=this.points.length;--d>f;){var g=this.points[d];if(this.elements[d]==e)if(a.sub2(h,this.center),b.sub2(g,this.center),a.normalize(),b.normalize(),this.isParallel(a,b))this.getAllAxes(a);else{this.nAxes[16]<JS.PointGroup.axesMaxN[16]&&(c.ave(h,g),c.sub(this.center),this.getAllAxes(c));var g=6.283185307179586/a.angle(b),n=u(Math.floor(g+0.01));if(0.02>=g-n&&!((n+=14)>=JS.PointGroup.maxAxis)&&this.nAxes[n]<JS.PointGroup.axesMaxN[n])c.cross(a,b),this.checkAxisOrder(n,
c,this.center)}}h=Array(2*this.nAxes[16]);for(f=0;f<h.length;f++)h[f]=new JU.V3;for(f=d=0;f<this.nAxes[16];f++)h[d++].setT(this.axes[16][f].normalOrAxis),h[d].setT(this.axes[16][f].normalOrAxis),h[d++].scale(-1);for(f=h.length;2<=--f;)for(d=f;1<=--d;)for(e=d;0<=--e;)c.add2(h[f],h[d]),c.add(h[e]),1>c.length()||this.checkAxisOrder(17,c,this.center);d=2147483647;h=-1;for(f=0;f<this.maxElement;f++)this.eCounts[f]<d&&2<this.eCounts[f]&&(d=this.eCounts[f],h=f);f=0;a:for(;f<this.points.length-2;f++)if(this.elements[f]==
h)for(d=f+1;d<this.points.length-1;d++)if(this.elements[d]==h)for(e=d+1;e<this.points.length;e++)if(this.elements[e]==h&&(a.sub2(this.points[f],this.points[d]),b.sub2(this.points[f],this.points[e]),a.normalize(),b.normalize(),c.cross(a,b),this.getAllAxes(c),a.add2(this.points[f],this.points[d]),a.add(this.points[e]),a.normalize(),this.isParallel(a,c)||this.getAllAxes(a),this.nAxes[19]==JS.PointGroup.axesMaxN[19]))break a;h=Array(this.maxElement);for(f=this.points.length;0<=--f;){e=this.elements[f];
if(null==h[e])h[e]=new JU.V3;else if(this.haveInversionCenter)continue;h[e].add(this.points[f])}if(!this.haveInversionCenter)for(f=0;f<this.maxElement;f++)null!=h[f]&&h[f].scale(1/this.eCounts[f]);for(f=0;f<this.maxElement;f++)if(null!=h[f])for(d=0;d<this.maxElement;d++)f==d||null==h[d]||(this.haveInversionCenter?a.cross(h[f],h[d]):a.sub2(h[f],h[d]),this.checkAxisOrder(16,a,this.center));return this.getHighestOrder()});d(c$,"getAllAxes",function(a){for(var b=16;b<JS.PointGroup.maxAxis;b++)this.nAxes[b]<
JS.PointGroup.axesMaxN[b]&&this.checkAxisOrder(b,a,this.center)},"JU.V3");d(c$,"getHighestOrder",function(){for(var a=0,a=14;1<--a&&0==this.nAxes[a];);if(1<a)return a+14<JS.PointGroup.maxAxis&&0<this.nAxes[a+14]?a+14:a;for(a=JS.PointGroup.maxAxis;1<--a&&0==this.nAxes[a];);return a});d(c$,"checkAxisOrder",function(a,b,c){switch(a){case 22:if(0<this.nAxes[17])return!1;case 20:case 18:if(0<this.nAxes[19])return!1;break;case 17:if(0<this.nAxes[22])return!1;break;case 19:if(0<this.nAxes[18]||0<this.nAxes[20]||
0<this.nAxes[22])return!1}b.normalize();if(this.haveAxis(a,b))return!1;var f=JS.PointGroup.getQuaternion(b,a);if(!this.checkOperation(f,c,a))return!1;this.addAxis(a,b);switch(a){case 16:this.checkAxisOrder(4,b,c);break;case 17:this.checkAxisOrder(3,b,c);this.haveInversionCenter&&this.addAxis(6,b);break;case 18:this.addAxis(16,b);this.checkAxisOrder(4,b,c);this.checkAxisOrder(8,b,c);break;case 19:this.checkAxisOrder(5,b,c);this.haveInversionCenter&&this.addAxis(10,b);break;case 20:this.addAxis(16,
b);this.addAxis(17,b);this.checkAxisOrder(3,b,c);this.checkAxisOrder(6,b,c);this.checkAxisOrder(12,b,c);break;case 22:this.addAxis(16,b),this.addAxis(18,b)}return!0},"~N,JU.V3,JU.T3");d(c$,"addAxis",function(a,b){this.haveAxis(a,b)||(null==this.axes[a]&&(this.axes[a]=Array(JS.PointGroup.axesMaxN[a])),this.axes[a][this.nAxes[a]++]=Z(JS.PointGroup.Operation,this,null,b,a))},"~N,JU.V3");d(c$,"haveAxis",function(a,b){if(this.nAxes[a]==JS.PointGroup.axesMaxN[a])return!0;if(0<this.nAxes[a])for(var c=this.nAxes[a];0<=
--c;)if(this.isParallel(b,this.axes[a][c].normalOrAxis))return!0;return!1},"~N,JU.V3");d(c$,"findPlanes",function(){for(var a=new JU.P3,b=new JU.V3,c=new JU.V3,f=new JU.V3,h=0,e=1<this.getHighestOrder(),d=this.points.length;0<=--d;)if(d!=this.centerAtomIndex)for(var g=this.points[d],n=this.elements[d],j=this.points.length;--j>d;)if(!(e&&this.elements[j]!=n)&&(h=this.points[j],a.add2(g,h),a.scale(0.5),b.sub2(g,this.center),c.sub2(h,this.center),this.isParallel(b,c)||(f.cross(b,c),f.normalize(),this.getPlane(f)),
f.sub2(h,g),f.normalize(),h=this.getPlane(f),h==JS.PointGroup.axesMaxN[0]))return h;if(e)for(d=16;d<JS.PointGroup.maxAxis;d++)for(j=0;j<this.nAxes[d];j++)h=this.getPlane(this.axes[d][j].normalOrAxis);return h});d(c$,"getPlane",function(a){!this.haveAxis(0,a)&&this.checkOperation(JU.Quat.newVA(a,180),this.center,-1)&&(this.axes[0][this.nAxes[0]++]=Z(JS.PointGroup.Operation,this,null,a));return this.nAxes[0]},"JU.V3");d(c$,"findAdditionalAxes",function(a){var b=this.axes[0],c=0;if(1<a&&(c=a+14)<JS.PointGroup.maxAxis&&
0==this.nAxes[c])this.vTemp.cross(b[0].normalOrAxis,b[1].normalOrAxis),!this.checkAxisOrder(c,this.vTemp,this.center)&&2<a&&(this.vTemp.cross(b[1].normalOrAxis,b[2].normalOrAxis),this.checkAxisOrder(c-1,this.vTemp,this.center));if(0==this.nAxes[16]&&2<a)for(c=0;c<a-1;c++)for(var f=c+1;f<a;f++)this.vTemp.add2(b[1].normalOrAxis,b[2].normalOrAxis),this.checkAxisOrder(16,this.vTemp,this.center)},"~N");c$.getQuaternion=d(c$,"getQuaternion",function(a,b){return JU.Quat.newVA(a,(14>b?180:0)+(0==b?0:u(360/
(b%14))))},"JU.V3,~N");d(c$,"getInfo",function(a,b,c,f,h,e){var d=null!=b;this.info=c?new java.util.Hashtable:null;var g=new JU.V3,n;0==e&&(e=1);this.scale=e;for(var j=Q(4,2,0),m=1;m<JS.PointGroup.maxAxis;m++)for(var k=this.nAxes[m];0<=--k;)j[this.axes[m][k].type][0]++;var q=(new JU.SB).append("# ").appendI(this.nAtoms).append(" atoms\n");if(d){b="draw "+b;this.drawType=f=(c=null!=f&&0<f.length)?f:"";this.drawIndex=h;var d=f.equalsIgnoreCase("Cn"),v=f.equalsIgnoreCase("Sn");q.append("set perspectivedepth off;\n");
var p="_"+a+"_";c||q.append(b+"pg0").append(p).append("* delete;draw pgva").append(p).append("* delete;draw pgvp").append(p).append("* delete;");if(!c||f.equalsIgnoreCase("Ci"))q.append(b+"pg0").append(p).append(this.haveInversionCenter?"inv ":" ").append(JU.Escape.eP(this.center)).append(this.haveInversionCenter?'"i";\n':";\n");for(var s=0.1,m=2;m<JS.PointGroup.maxAxis;m++)if(14==m&&(s=0.1),0!=this.nAxes[m]){a=this.axes[m][0].getLabel();var s=s+0.25,r=e*this.radius+s,k=14<=m;if(!c||f.equalsIgnoreCase(a)||
d&&k||v&&!k)for(k=0;k<this.nAxes[m];k++)if(!(0<h&&k+1!=h)){n=this.axes[m][k];g.add2(n.normalOrAxis,this.center);2==n.type&&(r=-r);q.append(b+"pgva").append(p).append(a).append("_").appendI(k+1).append(" width 0.05 scale ").appendF(r).append(" ").append(JU.Escape.eP(g));g.scaleAdd2(-2,n.normalOrAxis,g);var t=null!=this.principalAxis&&n.index==this.principalAxis.index;q.append(JU.Escape.eP(g)).append('"').append(a).append(t?"*":"").append('" color ').append(t?"red":2==n.type?"blue":"orange").append(";\n")}}if(!c||
f.equalsIgnoreCase("Cs"))for(k=0;k<this.nAxes[0];k++)0<h&&k+1!=h||(n=this.axes[0][k],q.append(b+"pgvp").append(p).appendI(k+1).append("disk scale ").appendF(2*e*this.radius).append(" CIRCLE PLANE ").append(JU.Escape.eP(this.center)),g.add2(n.normalOrAxis,this.center),q.append(JU.Escape.eP(g)).append(" color translucent yellow;\n"),g.add2(n.normalOrAxis,this.center),q.append(b+"pgvp").append(p).appendI(k+1).append("ring width 0.05 scale ").appendF(2*e*this.radius).append(" arc ").append(JU.Escape.eP(g)),
g.scaleAdd2(-2,n.normalOrAxis,g),q.append(JU.Escape.eP(g)),g.add3(0.011,0.012,0.013),q.append(JU.Escape.eP(g)).append("{0 360 0.5} color ").append(null!=this.principalPlane&&n.index==this.principalPlane.index?"red":"blue").append(";\n"));q.append("# name=").append(this.name);q.append(", nCi=").appendI(this.haveInversionCenter?1:0);q.append(", nCs=").appendI(this.nAxes[0]);q.append(", nCn=").appendI(j[1][0]);q.append(", nSn=").appendI(j[2][0]);q.append(": ");for(m=JS.PointGroup.maxAxis;2<=--m;)0<this.nAxes[m]&&
(q.append(" n").append(14>m?"S":"C").appendI(m%14),q.append("=").appendI(this.nAxes[m]));q.append(";\n");q.append("print '"+this.name+"';\n");this.drawInfo=q.toString();JU.Logger.debugging&&JU.Logger.info(this.drawInfo);return this.drawInfo}k=0;b=1;f=0;m=this.haveInversionCenter?"Ci":"center";this.haveInversionCenter&&(b++,f++);c?(this.info.put(m,this.center),this.haveInversionCenter&&this.info.put("center",this.center),this.info.put(m,this.center)):q.append("\n\n").append(this.name).append("\t").append(m).append("\t").append(JU.Escape.eP(this.center));
for(m=JS.PointGroup.maxAxis;0<=--m;)if(0<this.nAxes[m]){k=JS.PointGroup.nUnique[m];a=this.axes[m][0].getLabel();c?this.info.put("n"+a,Integer.$valueOf(this.nAxes[m])):q.append("\n\n").append(this.name).append("\tn").append(a).append("\t").appendI(this.nAxes[m]).append("\t").appendI(k);k*=this.nAxes[m];b+=k;f+=this.nAxes[m];j[this.axes[m][0].type][1]+=k;h=c?new JU.Lst:null;e=c?new JU.Lst:null;for(k=0;k<this.nAxes[m];k++)g=this.axes[m][k],c?(h.addLast(g.normalOrAxis),e.addLast(g.getM3())):q.append("\n").append(this.name).append("\t").append(a).append("_").appendI(k+
1).append("\t").appendO(g.normalOrAxis);c&&(this.info.put(a,h),this.info.put(a+"_m",e))}if(!c)return q.append("\n"),q.append("\n").append(this.name).append("\ttype\tnElements\tnUnique"),q.append("\n").append(this.name).append("\tE\t  1\t  1"),k=this.haveInversionCenter?1:0,q.append("\n").append(this.name).append("\tCi\t  ").appendI(k).append("\t  ").appendI(k),q.append("\n").append(this.name).append("\tCs\t"),JU.PT.rightJustify(q,"    ",this.nAxes[0]+"\t"),JU.PT.rightJustify(q,"    ",this.nAxes[0]+
"\n"),q.append(this.name).append("\tCn\t"),JU.PT.rightJustify(q,"    ",j[1][0]+"\t"),JU.PT.rightJustify(q,"    ",j[1][1]+"\n"),q.append(this.name).append("\tSn\t"),JU.PT.rightJustify(q,"    ",j[2][0]+"\t"),JU.PT.rightJustify(q,"    ",j[2][1]+"\n"),q.append(this.name).append("\t\tTOTAL\t"),JU.PT.rightJustify(q,"    ",b+"\n"),this.textInfo=q.toString();this.info.put("name",this.name);this.info.put("nAtoms",Integer.$valueOf(this.nAtoms));this.info.put("nTotal",Integer.$valueOf(b));this.info.put("nElements",
Integer.$valueOf(f));this.info.put("nCi",Integer.$valueOf(this.haveInversionCenter?1:0));this.haveInversionCenter&&this.info.put("Ci_m",JU.M3.newM3(JS.PointGroup.mInv));this.info.put("nCs",Integer.$valueOf(this.nAxes[0]));this.info.put("nCn",Integer.$valueOf(j[1][0]));this.info.put("nSn",Integer.$valueOf(j[2][0]));this.info.put("distanceTolerance",Float.$valueOf(this.distanceTolerance));this.info.put("linearTolerance",Float.$valueOf(this.linearTolerance));this.info.put("points",this.points);this.info.put("detail",
q.toString().$replace("\n",";"));null!=this.principalAxis&&0<this.principalAxis.index&&this.info.put("principalAxis",this.principalAxis.normalOrAxis);null!=this.principalPlane&&0<this.principalPlane.index&&this.info.put("principalPlane",this.principalPlane.normalOrAxis);return this.info},"~N,~S,~B,~S,~N,~N");d(c$,"isDrawType",function(a,b,c){return null!=this.drawInfo&&this.drawType.equals(null==a?"":a)&&this.drawIndex==b&&this.scale==c},"~S,~N,~N");c$.$PointGroup$Operation$=function(){ga(self.c$);
c$=N(function(){ia(this,arguments);this.index=this.order=this.type=0;this.normalOrAxis=null;this.typeOrder=0;this.mat=null;R(this,arguments)},JS.PointGroup,"Operation");D(c$,function(){this.index=++this.b$["JS.PointGroup"].nOps;this.type=3;this.typeOrder=this.order=1;JU.Logger.debugging&&JU.Logger.debug("new operation -- "+JS.PointGroup.typeNames[this.type])});D(c$,function(a,b){this.index=++this.b$["JS.PointGroup"].nOps;this.type=14>b?2:1;this.typeOrder=b;this.order=b%14;this.normalOrAxis=JU.Quat.newVA(a,
180).getNormal();JU.Logger.debugging&&JU.Logger.debug("new operation -- "+(this.order==b?"S":"C")+this.order+" "+this.normalOrAxis)},"JU.V3,~N");D(c$,function(a){null!=a&&(this.index=++this.b$["JS.PointGroup"].nOps,this.type=0,this.normalOrAxis=JU.Quat.newVA(a,180).getNormal(),JU.Logger.debugging&&JU.Logger.debug("new operation -- plane "+this.normalOrAxis))},"JU.V3");d(c$,"getLabel",function(){switch(this.type){case 0:return"Cs";case 2:return"S"+this.order;default:return"C"+this.order}});d(c$,"getM3",
function(){if(null!=this.mat)return this.mat;var a=JU.M3.newM3(JS.PointGroup.getQuaternion(this.normalOrAxis,this.typeOrder).getMatrix());(0==this.type||2==this.type)&&a.mul(JS.PointGroup.mInv);this.cleanMatrix(a);return this.mat=a});d(c$,"cleanMatrix",function(a){for(var b=0;3>b;b++)for(var c=0;3>c;c++)a.setElement(b,c,this.approx0(a.getElement(b,c)))},"JU.M3");d(c$,"approx0",function(a){return 1E-15<a||-1E-15>a?a:0},"~N");c$=fa()};G(c$,"axesMaxN",Q(-1,[15,0,0,1,3,1,10,0,1,0,6,0,1,0,0,0,15,10,6,
6,10,0,1]),"nUnique",Q(-1,[1,0,0,2,2,4,2,0,4,0,4,0,4,0,0,0,1,2,2,4,2,0,4]),"s3",3,"s4",4,"s5",5,"s6",6,"s8",8,"s10",10,"s12",12,"firstProper",14,"c2",16,"c3",17,"c4",18,"c5",19,"c6",20,"c8",22);c$.maxAxis=c$.prototype.maxAxis=JS.PointGroup.axesMaxN.length;G(c$,"ATOM_COUNT_MAX",100,"OPERATION_PLANE",0,"OPERATION_PROPER_AXIS",1,"OPERATION_IMPROPER_AXIS",2,"OPERATION_INVERSION_CENTER",3,"typeNames",K(-1,["plane","proper axis","improper axis","center of inversion"]));c$.mInv=c$.prototype.mInv=JU.M3.newA9(r(-1,
[-1,0,0,0,-1,0,0,0,-1]))});M("JS");B(["java.util.Hashtable"],"JS.SpaceGroup","java.util.Arrays JU.AU $.Lst $.M4 $.P3 $.PT $.SB JS.HallInfo $.HallTranslation $.SymmetryOperation JU.Logger".split(" "),function(){c$=N(function(){this.index=0;this.isSSG=!1;this.name="unknown!";this.hmSymbolAbbrShort=this.hmSymbolAlternative=this.hmSymbolAbbr=this.hmSymbolExt=this.hmSymbolFull=this.hmSymbol=this.crystalClass=this.hallSymbol=null;this.axisChoice=this.uniqueAxis=this.ambiguityType="\x00";this.hallInfo=this.intlTableNumberExt=
this.intlTableNumberFull=this.intlTableNumber=null;this.latticeParameter=0;this.finalOperations=this.operations=null;this.operationCount=0;this.latticeOp=-1;this.xyzList=null;this.modDim=0;this.doNormalize=!0;this.isBilbao=this.isBio=!1;this.latticeType="P";this.nHallOperators=null;R(this,arguments)},JS,"SpaceGroup");c$.getNull=d(c$,"getNull",function(a,b,c){JS.SpaceGroup.getSpaceGroups();a=new JS.SpaceGroup(-1,null,a);a.doNormalize=b;c&&a.setFinalOperations(null,0,0,!1);return a},"~B,~B,~B");D(c$,
function(a,b,c){++JS.SpaceGroup.sgIndex;0>a&&(a=JS.SpaceGroup.sgIndex);this.index=a;this.init(c&&null==b);c&&null!=b&&this.buildSpaceGroup(b)},"~N,~S,~B");d(c$,"init",function(a){this.xyzList=new java.util.Hashtable;this.operationCount=0;a&&this.addSymmetry("x,y,z",0,!1)},"~B");c$.createSpaceGroup=d(c$,"createSpaceGroup",function(a,b,c,f){var h=null;0<=a?h=JS.SpaceGroup.getSpaceGroups()[a]:(h=z(c,JU.Lst)?JS.SpaceGroup.createSGFromList(b,c):JS.SpaceGroup.determineSpaceGroupNA(b,c),null==h&&(h=JS.SpaceGroup.createSpaceGroupN(0>=
f?b:"x1,x2,x3,x4,x5,x6,x7,x8,x9".substring(0,3*f+8))));null!=h&&h.generateAllOperators(null);return h},"~N,~S,~O,~N");c$.createSGFromList=d(c$,"createSGFromList",function(a,b){var c=new JS.SpaceGroup(-1,"0;0;--;--;--",!0);c.doNormalize=!1;c.name=a;for(var f=b.size(),h=0;h<f;h++){var e=b.get(h);if(z(e,JS.SymmetryOperation)){var d=c.addOp(e,e.xyz,!1);c.operations[d].setTimeReversal(e.timeReversal)}else c.addSymmetrySM("xyz matrix:"+e,e)}f=c.getDerivedSpaceGroup();null!=f&&(c=f);return c},"~S,JU.Lst");
d(c$,"addSymmetry",function(a,b,c){a=a.toLowerCase();return 0>a.indexOf("[[")&&0>a.indexOf("x4")&&0>a.indexOf(";")&&(0>a.indexOf("x")||0>a.indexOf("y")||0>a.indexOf("z"))?-1:this.addOperation(a,b,c)},"~S,~N,~B");d(c$,"setFinalOperations",function(a,b,c,f){if(null==this.hallInfo&&0!=this.latticeParameter){var h=new JS.HallInfo(JS.HallTranslation.getHallLatticeEquivalent(this.latticeParameter));this.generateAllOperators(h)}this.finalOperations=null;this.isBio=0<=this.name.indexOf("bio");this.index>=
JS.SpaceGroup.getSpaceGroups().length&&(!this.isBio&&0>this.name.indexOf("SSG:")&&0>this.name.indexOf("[subZystem"))&&(h=this.getDerivedSpaceGroup(),null!=h&&(this.name=h.getName(),this.latticeType=h.latticeType,this.intlTableNumber=h.intlTableNumber));this.finalOperations=Array(this.operationCount);if(f&&0<c&&null!=a){this.finalOperations[0]=new JS.SymmetryOperation(this.operations[0],a,b,c,!0);var h=a[b],e=JU.P3.newP(h);this.finalOperations[0].rotTrans(e);if(1E-4<e.distance(h))for(var d=0;d<c;d++)h=
a[b+d],e.setT(h),this.finalOperations[0].rotTrans(e),h.setT(e)}for(d=0;d<this.operationCount;d++)this.finalOperations[d]=new JS.SymmetryOperation(this.operations[d],a,b,c,f),this.finalOperations[d].getCentering()},"~A,~N,~N,~B");d(c$,"getOperationCount",function(){return this.finalOperations.length});d(c$,"getOperation",function(a){return this.finalOperations[a]},"~N");d(c$,"getXyz",function(a,b){return null==this.finalOperations?this.operations[a].getXyz(b):this.finalOperations[a].getXyz(b)},"~N,~B");
d(c$,"newPoint",function(a,b,c,f,d,e){JS.SymmetryOperation.newPoint(this.finalOperations[a],b,c,f,d,e)},"~N,JU.P3,JU.P3,~N,~N,~N");c$.getInfo=d(c$,"getInfo",function(a,b,c,f){if(null!=a&&a.index>=JS.SpaceGroup.SG.length){var d=JS.SpaceGroup.findSpaceGroup(a.operationCount,a.getCanonicalSeitzList());null!=d&&(a=d)}if(null!=c){if(null==a){0<=b.indexOf("[")&&(b=b.substring(0,b.indexOf("[")).trim());if(b.equals("unspecified!"))return"no space group identified in file";a=JS.SpaceGroup.determineSpaceGroupNA(b,
c.getUnitCellParams())}}else{if(b.equalsIgnoreCase("ALL"))return JS.SpaceGroup.dumpAll();if(b.equalsIgnoreCase("ALLSEITZ"))return JS.SpaceGroup.dumpAllSeitz();a=JS.SpaceGroup.determineSpaceGroupN(b)}null==a&&(c=JS.SpaceGroup.createSpaceGroupN(b),c=JS.SpaceGroup.findSpaceGroup(c.operationCount,c.getCanonicalSeitzList()),null!=c&&(a=c));if(null!=a){if(f)return a.dumpInfoObj();for(f=new JU.SB;null!=a;){f.append(a.dumpInfo());if(a.index>=JS.SpaceGroup.SG.length)break;a=JS.SpaceGroup.determineSpaceGroupNS(b,
a)}return f.toString()}return f?null:"?"},"JS.SpaceGroup,~S,J.api.SymmetryInterface,~B");d(c$,"dumpInfo",function(){var a=this.dumpCanonicalSeitzList();if(z(a,JS.SpaceGroup))return a.dumpInfo();var b=(new JU.SB).append("\nHermann-Mauguin symbol: ");null==this.hmSymbol||null==this.hmSymbolExt?b.append("?"):b.append(this.hmSymbol).append(0<this.hmSymbolExt.length?":"+this.hmSymbolExt:"");null!=this.intlTableNumber&&b.append("\ninternational table number: ").append(this.intlTableNumber).append(0<this.intlTableNumberExt.length?
":"+this.intlTableNumberExt:"").append("\ncrystal class: "+this.crystalClass).append("\n\n").appendI(this.operationCount).append(" operators").append(!this.hallInfo.hallSymbol.equals("--")?" from Hall symbol "+this.hallInfo.hallSymbol+"  #"+this.intlTableNumberFull:"").append(": ");for(var c=0;c<this.operationCount;c++)b.append("\n").append(this.operations[c].xyz);b.append("\n\n").append(null==this.hallInfo?"Hall symbol unknown":this.hallInfo.dumpInfo());b.append("\n\ncanonical Seitz: ").append(a).append("\n----------------------------------------------------\n");
return b.toString()});d(c$,"dumpInfoObj",function(){var a=this.dumpCanonicalSeitzList();if(z(a,JS.SpaceGroup))return a.dumpInfoObj();a=new java.util.Hashtable;a.put("HermannMauguinSymbol",null==this.hmSymbol||null==this.hmSymbolExt?"?":this.hmSymbol+(0<this.hmSymbolExt.length?":"+this.hmSymbolExt:""));null!=this.intlTableNumber&&(a.put("ita",Integer.$valueOf(this.intlTableNumber)),a.put("itaFull",this.intlTableNumberFull),a.put("crystalClass",this.crystalClass),a.put("operationCount",Integer.$valueOf(this.operationCount)));
for(var b=new JU.Lst,c=0;c<this.operationCount;c++)b.addLast(this.operations[c].xyz);a.put("operationsXYZ",b);a.put("HallSymbol",null==this.hallInfo?"?":this.hallInfo.hallSymbol);return a});d(c$,"getName",function(){return this.name});d(c$,"getLatticeDesignation",function(){return JS.HallTranslation.getLatticeDesignation(this.latticeParameter)});d(c$,"setLatticeParam",function(a){this.latticeParameter=a;10<a&&(this.latticeParameter=-JS.HallTranslation.getLatticeIndex(JS.HallTranslation.getLatticeCode(a)))},
"~N");d(c$,"dumpCanonicalSeitzList",function(){null!=this.nHallOperators&&(null==this.hallInfo&&(this.hallInfo=new JS.HallInfo(this.hallSymbol)),this.generateAllOperators(null));var a=this.getCanonicalSeitzList();if(this.index>=JS.SpaceGroup.SG.length){var b=JS.SpaceGroup.findSpaceGroup(this.operationCount,a);if(null!=b)return b.getCanonicalSeitzList()}return(0<=this.index&&this.index<JS.SpaceGroup.SG.length?this.hallSymbol+" = ":"")+a});d(c$,"getDerivedSpaceGroup",function(){if(0<=this.index&&this.index<
JS.SpaceGroup.SG.length||0<this.modDim||null==this.operations||0==this.operations.length||0!=this.operations[0].timeReversal)return this;null!=this.finalOperations&&this.setFinalOperations(null,0,0,!1);var a=this.getCanonicalSeitzList();return null==a?null:JS.SpaceGroup.findSpaceGroup(this.operationCount,a)});d(c$,"getCanonicalSeitzList",function(){for(var a=Array(this.operationCount),b=0;b<this.operationCount;b++)a[b]=JS.SymmetryOperation.dumpSeitz(this.operations[b],!0);java.util.Arrays.sort(a,
0,this.operationCount);for(var c=(new JU.SB).append("\n["),b=0;b<this.operationCount;b++)c.append(a[b].$replace("\t"," ").$replace("\n"," ")).append("; ");c.append("]");return c.toString()});c$.findSpaceGroup=d(c$,"findSpaceGroup",function(a,b){JS.SpaceGroup.getSpaceGroups();var c=JS.SpaceGroup.htByOpCount.get(Integer.$valueOf(a));if(null!=c)for(var f=0,d=c.size();f<d;f++){var e=c.get(f);if(0<=JS.SpaceGroup.getCanonicalSeitz(e.index).indexOf(b))return JS.SpaceGroup.SG[e.index]}return null},"~N,~S");
c$.dumpAll=d(c$,"dumpAll",function(){var a=new JU.SB;JS.SpaceGroup.getSpaceGroups();for(var b=0;b<JS.SpaceGroup.SG.length;b++)a.append("\n----------------------\n"+JS.SpaceGroup.SG[b].dumpInfo());return a.toString()});c$.dumpAllSeitz=d(c$,"dumpAllSeitz",function(){JS.SpaceGroup.getSpaceGroups();for(var a=new JU.SB,b=0;b<JS.SpaceGroup.SG.length;b++)a.append("\n").appendO(JS.SpaceGroup.getCanonicalSeitz(b));return a.toString()});c$.getCanonicalSeitz=d(c$,"getCanonicalSeitz",function(a){null==JS.SpaceGroup.canonicalSeitzList&&
(JS.SpaceGroup.canonicalSeitzList=Array(JS.SpaceGroup.SG.length));var b=JS.SpaceGroup.canonicalSeitzList[a];return null==b?JS.SpaceGroup.canonicalSeitzList[a]=JS.SpaceGroup.SG[a].dumpCanonicalSeitzList().toString():b},"~N");d(c$,"setLattice",function(a,b){this.latticeParameter=JS.HallTranslation.getLatticeIndex(a);b||(this.latticeParameter=-this.latticeParameter)},"~S,~B");c$.createSpaceGroupN=d(c$,"createSpaceGroupN",function(a){JS.SpaceGroup.getSpaceGroups();a=a.trim();var b=JS.SpaceGroup.determineSpaceGroupN(a),
c;null==b&&(c=new JS.HallInfo(a),0<c.nRotations?(b=new JS.SpaceGroup(-1,"0;0;--;--;"+a,!0),b.hallInfo=c):0<=a.indexOf(",")&&(b=new JS.SpaceGroup(-1,"0;0;--;--;--",!0),b.doNormalize=!1,b.generateOperatorsFromXyzInfo(a)));null!=b&&b.generateAllOperators(null);return b},"~S");d(c$,"addOperation",function(a,b,c){if(null==a||3>a.length)return this.init(!1),-1;var f="="==a.charAt(0);f&&(a=a.substring(1));var d=this.checkXYZlist(a);if(0<=d)return d;if(a.startsWith("x1,x2,x3,x4")&&0==this.modDim)this.xyzList.clear(),
this.operationCount=0,this.modDim=JU.PT.parseInt(a.substring(a.lastIndexOf("x")+1))-3;else if(0<=a.indexOf("m")&&(a=JU.PT.rep(a,"+m","m"),a.equals("x,y,z,m")||a.equals("x,y,z(mx,my,mz)")))this.xyzList.clear(),this.operationCount=0;b=new JS.SymmetryOperation(null,null,0,b,this.doNormalize);return!b.setMatrixFromXYZ(a,this.modDim,c)?(JU.Logger.error("couldn't interpret symmetry operation: "+a),-1):this.addOp(b,a,f)},"~S,~N,~B");d(c$,"checkXYZlist",function(a){return this.xyzList.containsKey(a)?this.xyzList.get(a).intValue():
-1},"~S");d(c$,"addOp",function(a,b,c){var f=a.xyz;if(!c){c=this.checkXYZlist(f);if(0<=c)return c;0>this.latticeOp&&(c=JU.PT.replaceAllCharacters(0<this.modDim?JS.SymmetryOperation.replaceXn(f,this.modDim+3):f,"+123/",""),this.xyzList.containsKey(c+"!")?this.latticeOp=this.operationCount:this.xyzList.put(c+"!",Integer.$valueOf(this.operationCount)));this.xyzList.put(f,Integer.$valueOf(this.operationCount))}f.equals(b)||this.xyzList.put(b,Integer.$valueOf(this.operationCount));null==this.operations&&
(this.operations=Array(4));this.operationCount==this.operations.length&&(this.operations=JU.AU.arrayCopyObject(this.operations,2*this.operationCount));this.operations[this.operationCount++]=a;a.number=this.operationCount;0!=a.timeReversal&&(this.operations[0].timeReversal=1);JU.Logger.debugging&&JU.Logger.debug("\naddOperation "+this.operationCount+a.dumpInfo());return this.operationCount-1},"JS.SymmetryOperation,~S,~B");d(c$,"generateOperatorsFromXyzInfo",function(a){this.init(!0);a=JU.PT.split(a.toLowerCase(),
";");for(var b=0;b<a.length;b++)this.addSymmetry(a[b],0,!1)},"~S");d(c$,"generateAllOperators",function(a){if(null==a){if(0<this.operationCount)return;a=this.hallInfo;this.operations=Array(4);if(null==this.hallInfo||0==this.hallInfo.nRotations)a=this.hallInfo=new JS.HallInfo(this.hallSymbol);this.setLattice(this.hallInfo.latticeCode,this.hallInfo.isCentrosymmetric);this.init(!0)}switch(a.latticeCode){case "\x00":case "S":case "T":case "P":this.latticeType="P";break;default:this.latticeType=""+a.latticeCode}for(var b=
new JU.M4,c=new JU.M4,f=Array(7),d=0;7>d;d++)f[d]=new JU.M4;for(d=0;d<a.nRotations;d++){var e=a.rotationTerms[d];b.setM4(e.seitzMatrix12ths);e=e.order;f[0].setIdentity();for(var g=this.operationCount,l=1;l<=e;l++){var n=f[l];n.mul2(b,f[0]);f[0].setM4(n);for(var j=0;j<g;j++){c.mul2(n,this.operations[j]);c.m03=(p(c.m03)+12)%12;c.m13=(p(c.m13)+12)%12;c.m23=(p(c.m23)+12)%12;var m=JS.SymmetryOperation.getXYZFromMatrix(c,!0,!0,!0);this.addSymmetrySM(m,c)}}}null!=this.nHallOperators&&this.operationCount!=
this.nHallOperators.intValue()&&JU.Logger.error("Operator mismatch "+this.operationCount+" for "+this)},"JS.HallInfo");d(c$,"addSymmetrySM",function(a,b){var c=this.addOperation(a,0,!1);0<=c&&this.operations[c].setM4(b);return c},"~S,JU.M4");c$.determineSpaceGroupN=d(c$,"determineSpaceGroupN",function(a){return JS.SpaceGroup.determineSpaceGroup(a,0,0,0,0,0,0,-1)},"~S");c$.determineSpaceGroupNS=d(c$,"determineSpaceGroupNS",function(a,b){return JS.SpaceGroup.determineSpaceGroup(a,0,0,0,0,0,0,b.index)},
"~S,JS.SpaceGroup");c$.determineSpaceGroupNA=d(c$,"determineSpaceGroupNA",function(a,b){return null==b?JS.SpaceGroup.determineSpaceGroup(a,0,0,0,0,0,0,-1):JS.SpaceGroup.determineSpaceGroup(a,b[0],b[1],b[2],b[3],b[4],b[5],-1)},"~S,~A");c$.determineSpaceGroup=d(c$,"determineSpaceGroup",function(a,b,c,f,d,e,g,l){a=JS.SpaceGroup.determineSpaceGroupIndex(a,b,c,f,d,e,g,l);return 0<=a?JS.SpaceGroup.SG[a]:null},"~S,~N,~N,~N,~N,~N,~N,~N");c$.determineSpaceGroupIndex=d(c$,"determineSpaceGroupIndex",function(a,
b,c,f,d,e,g,l){JS.SpaceGroup.getSpaceGroups();0>l&&(l=JS.SpaceGroup.SG.length);a=a.trim().toLowerCase();var n=!1;a.startsWith("bilbao:")&&(n=!0,a=a.substring(7));var j=a.startsWith("hall:")?5:a.startsWith("hm:")?3:0;switch(j){case 3:case 5:a=a.substring(j);break;case 0:a.contains("[")&&(j=5,a=a.substring(0,a.indexOf("[")).trim())}var m=a,k,q=!1;a=a.$replace("_"," ");2<=a.length&&(k=0==a.indexOf("-")?2:1,k<a.length&&" "!=a.charAt(k)&&(a=a.substring(0,k)+" "+a.substring(k)),a=JS.SpaceGroup.toCap(a,
2));var p="";if(0<(k=a.indexOf(":")))p=a.substring(k+1),a=a.substring(0,k).trim(),q=!0;5!=j&&(!q&&JU.PT.isOneOf(a,JS.SpaceGroup.ambiguousNames))&&(p="?",q=!0);var r=JU.PT.replaceAllCharacters(a," ()",""),s;if(3!=j&&!q)for(k=l;0<=--k;)if(JS.SpaceGroup.SG[k].hallSymbol.equalsIgnoreCase(a))return k;if(5!=j){if(3!=j)for(k=l;0<=--k;)if(JS.SpaceGroup.SG[k].intlTableNumberFull.equalsIgnoreCase(m))return k;for(k=l;0<=--k;)if(JS.SpaceGroup.SG[k].hmSymbolFull.equalsIgnoreCase(m))return k;for(k=l;0<=--k;)if(null!=
(s=JS.SpaceGroup.SG[k]).hmSymbolAlternative&&s.hmSymbolAlternative.equalsIgnoreCase(m))return k;if(q){for(k=l;0<=--k;)if((s=JS.SpaceGroup.SG[k]).hmSymbolAbbr.equalsIgnoreCase(r)&&s.intlTableNumberExt.equalsIgnoreCase(p))return k;for(k=l;0<=--k;)if((s=JS.SpaceGroup.SG[k]).hmSymbolAbbrShort.equalsIgnoreCase(r)&&s.intlTableNumberExt.equalsIgnoreCase(p))return k}a=JS.SpaceGroup.determineUniqueAxis(b,c,f,d,e,g);if(!q||"?"==p.charAt(0))for(k=0;k<l;k++)if(((s=JS.SpaceGroup.SG[k]).hmSymbolAbbr.equalsIgnoreCase(r)||
s.hmSymbolAbbrShort.equalsIgnoreCase(r))&&(!n||s.isBilbao))switch(s.ambiguityType){case "\x00":return k;case "a":if(s.uniqueAxis==a||"\x00"==a)return k;break;case "o":if(0==p.length){if(s.hmSymbolExt.equals("2"))return k}else if(s.hmSymbolExt.equalsIgnoreCase(p))return k;break;case "t":if(0==p.length){if("h"==s.axisChoice)return k}else if((s.axisChoice+"").equalsIgnoreCase(p))return k}}if(0==p.length)for(k=0;k<l;k++)if((s=JS.SpaceGroup.SG[k]).intlTableNumber.equals(m)&&(!n||s.isBilbao))return k;return-1},
"~S,~N,~N,~N,~N,~N,~N,~N");c$.determineUniqueAxis=d(c$,"determineUniqueAxis",function(a,b,c,f,d,e){return a==b?b==c?"\x00":"c":b==c?"a":c==a?"b":f==d?d==e?"\x00":"c":d==e?"a":e==f?"b":"\x00"},"~N,~N,~N,~N,~N,~N");d(c$,"buildSpaceGroup",function(a){a=JU.PT.split(a.toLowerCase(),";");this.intlTableNumberFull=a[0].trim();this.isBilbao=6>a.length&&!this.intlTableNumberFull.equals("0");var b=JU.PT.split(this.intlTableNumberFull,":");this.intlTableNumber=b[0];this.intlTableNumberExt=1==b.length?"":b[1];
this.ambiguityType="\x00";0<this.intlTableNumberExt.length&&(this.intlTableNumberExt.equals("h")||this.intlTableNumberExt.equals("r")?(this.ambiguityType="t",this.axisChoice=this.intlTableNumberExt.charAt(0)):this.intlTableNumberExt.startsWith("1")||this.intlTableNumberExt.startsWith("2")?this.ambiguityType="o":2>=this.intlTableNumberExt.length?(this.ambiguityType="a",this.uniqueAxis=this.intlTableNumberExt.charAt(0)):this.intlTableNumberExt.contains("-")&&(this.ambiguityType="-"));a[1].equals("0")||
(this.nHallOperators=Integer.$valueOf(a[1]),b=JS.SpaceGroup.htByOpCount.get(this.nHallOperators),null==b&&JS.SpaceGroup.htByOpCount.put(this.nHallOperators,b=new JU.Lst),b.addLast(this));this.crystalClass=JS.SpaceGroup.toCap(JU.PT.split(a[2],"^")[0],1);this.setHMSymbol(a[3]);this.hallSymbol=a[4];1<this.hallSymbol.length&&(this.hallSymbol=JS.SpaceGroup.toCap(this.hallSymbol,2));a=this.intlTableNumber+this.hallSymbol;"0"!=this.intlTableNumber.charAt(0)&&JS.SpaceGroup.lastInfo.equals(a)&&(JS.SpaceGroup.ambiguousNames+=
this.hmSymbol+";");JS.SpaceGroup.lastInfo=a;this.name=this.hallSymbol+" ["+this.hmSymbolFull+"] #"+this.intlTableNumber},"~S");d(c$,"setHMSymbol",function(a){this.hmSymbolFull=JS.SpaceGroup.toCap(a,1);this.latticeType=this.hmSymbolFull.substring(0,1);a=JU.PT.split(this.hmSymbolFull,":");this.hmSymbol=a[0];this.hmSymbolExt=1==a.length?"":a[1];a=this.hmSymbol.indexOf(" -3");1<=a&&0<="admn".indexOf(this.hmSymbol.charAt(a-1))&&(this.hmSymbolAlternative=(this.hmSymbol.substring(0,a)+" 3"+this.hmSymbol.substring(a+
3)).toLowerCase());this.hmSymbolAbbr=JU.PT.rep(this.hmSymbol," ","");this.hmSymbolAbbrShort=JU.PT.rep(this.hmSymbol," 1","");this.hmSymbolAbbrShort=JU.PT.rep(this.hmSymbolAbbrShort," ","")},"~S");c$.toCap=d(c$,"toCap",function(a,b){return a.substring(0,b).toUpperCase()+a.substring(b)},"~S,~N");d(c$,"toString",function(){return""+this.intlTableNumberFull+"["+this.index+","+this.nHallOperators+"] "+this.hmSymbolFull+" "+this.hallSymbol});c$.getSpaceGroups=d(c$,"getSpaceGroups",function(){return null==
JS.SpaceGroup.SG?JS.SpaceGroup.SG=JS.SpaceGroup.createSpaceGroups():JS.SpaceGroup.SG});c$.createSpaceGroups=d(c$,"createSpaceGroups",function(){for(var a=JS.SpaceGroup.STR_SG.length,b=Array(a),c=0;c<a;c++)b[c]=new JS.SpaceGroup(c,JS.SpaceGroup.STR_SG[c],!0);JS.SpaceGroup.STR_SG=null;return b});d(c$,"addLatticeVectors",function(a){if(0<=this.latticeOp||0==a.size())return!1;for(var b=this.latticeOp=this.operationCount,c=a.get(0).length==this.modDim+4,f=-2,d=0;d<a.size();d++){var e=a.get(d);c&&(f=p(e[this.modDim+
3]),e=JU.AU.arrayCopyF(e,this.modDim+3));if(e.length>this.modDim+3)return!1;for(var g=0;g<b;g++){var l=new JS.SymmetryOperation(null,null,0,0,!0);l.modDim=this.modDim;var n=this.operations[g];l.divisor=n.divisor;l.linearRotTrans=JU.AU.arrayCopyF(n.linearRotTrans,-1);l.setFromMatrix(e,!1);-2!=f&&l.setTimeReversal(n.timeReversal*f);l.xyzOriginal=l.xyz;this.addOp(l,l.xyz,!0)}}return!0},"JU.Lst");d(c$,"getSiteMultiplicity",function(a,b){for(var c=this.finalOperations.length,f=new JU.Lst,d=c;0<=--d;){var e=
JU.P3.newP(a);this.finalOperations[d].rotTrans(e);b.unitize(e);for(var g=f.size();0<=--g;){var l=f.get(g);if(1E-6>e.distanceSquared(l)){e=null;break}}null!=e&&f.addLast(e)}return u(c/f.size())},"JU.P3,JS.UnitCell");d(c$,"setName",function(a){this.name=a;null!=a&&a.startsWith("HM:")&&this.setHMSymbol(a.substring(3))},"~S");d(c$,"getRawOperation",function(a){var b=new JS.SymmetryOperation(null,null,0,0,!1);b.setMatrixFromXYZ(this.operations[a].xyzOriginal,0,!1);b.doFinalize();return b},"~N");G(c$,"canonicalSeitzList",
null,"NAME_UNK",0,"NAME_HM",3,"NAME_HALL",5,"sgIndex",-1,"ambiguousNames","","lastInfo","","SG",null);c$.htByOpCount=c$.prototype.htByOpCount=new java.util.Hashtable;G(c$,"STR_SG",K(-1,'1;1;c1^1;p 1;p 1,2;2;ci^1;p -1;-p 1,3:b;2;c2^1;p 1 2 1;p 2y,3:b;2;c2^1;p 2;p 2y,3:c;2;c2^1;p 1 1 2;p 2,3:a;2;c2^1;p 2 1 1;p 2x,4:b;2;c2^2;p 1 21 1;p 2yb,4:b;2;c2^2;p 21;p 2yb,4:b*;2;c2^2;p 1 21 1*;p 2y1,4:c;2;c2^2;p 1 1 21;p 2c,4:c*;2;c2^2;p 1 1 21*;p 21,4:a;2;c2^2;p 21 1 1;p 2xa,4:a*;2;c2^2;p 21 1 1*;p 2x1,5:b1;4;c2^3;c 1 2 1;c 2y,5:b1;4;c2^3;c 2;c 2y,5:b2;4;c2^3;a 1 2 1;a 2y,5:b3;4;c2^3;i 1 2 1;i 2y,5:c1;4;c2^3;a 1 1 2;a 2,5:c2;4;c2^3;b 1 1 2;b 2,5:c3;4;c2^3;i 1 1 2;i 2,5:a1;4;c2^3;b 2 1 1;b 2x,5:a2;4;c2^3;c 2 1 1;c 2x,5:a3;4;c2^3;i 2 1 1;i 2x,6:b;2;cs^1;p 1 m 1;p -2y,6:b;2;cs^1;p m;p -2y,6:c;2;cs^1;p 1 1 m;p -2,6:a;2;cs^1;p m 1 1;p -2x,7:b1;2;cs^2;p 1 c 1;p -2yc,7:b1;2;cs^2;p c;p -2yc,7:b2;2;cs^2;p 1 n 1;p -2yac,7:b2;2;cs^2;p n;p -2yac,7:b3;2;cs^2;p 1 a 1;p -2ya,7:b3;2;cs^2;p a;p -2ya,7:c1;2;cs^2;p 1 1 a;p -2a,7:c2;2;cs^2;p 1 1 n;p -2ab,7:c3;2;cs^2;p 1 1 b;p -2b,7:a1;2;cs^2;p b 1 1;p -2xb,7:a2;2;cs^2;p n 1 1;p -2xbc,7:a3;2;cs^2;p c 1 1;p -2xc,8:b1;4;cs^3;c 1 m 1;c -2y,8:b1;4;cs^3;c m;c -2y,8:b2;4;cs^3;a 1 m 1;a -2y,8:b3;4;cs^3;i 1 m 1;i -2y,8:b3;4;cs^3;i m;i -2y,8:c1;4;cs^3;a 1 1 m;a -2,8:c2;4;cs^3;b 1 1 m;b -2,8:c3;4;cs^3;i 1 1 m;i -2,8:a1;4;cs^3;b m 1 1;b -2x,8:a2;4;cs^3;c m 1 1;c -2x,8:a3;4;cs^3;i m 1 1;i -2x,9:b1;4;cs^4;c 1 c 1;c -2yc,9:b1;4;cs^4;c c;c -2yc,9:b2;4;cs^4;a 1 n 1;a -2yab,9:b3;4;cs^4;i 1 a 1;i -2ya,9:-b1;4;cs^4;a 1 a 1;a -2ya,9:-b2;4;cs^4;c 1 n 1;c -2yac,9:-b3;4;cs^4;i 1 c 1;i -2yc,9:c1;4;cs^4;a 1 1 a;a -2a,9:c2;4;cs^4;b 1 1 n;b -2ab,9:c3;4;cs^4;i 1 1 b;i -2b,9:-c1;4;cs^4;b 1 1 b;b -2b,9:-c2;4;cs^4;a 1 1 n;a -2ab,9:-c3;4;cs^4;i 1 1 a;i -2a,9:a1;4;cs^4;b b 1 1;b -2xb,9:a2;4;cs^4;c n 1 1;c -2xac,9:a3;4;cs^4;i c 1 1;i -2xc,9:-a1;4;cs^4;c c 1 1;c -2xc,9:-a2;4;cs^4;b n 1 1;b -2xab,9:-a3;4;cs^4;i b 1 1;i -2xb,10:b;4;c2h^1;p 1 2/m 1;-p 2y,10:b;4;c2h^1;p 2/m;-p 2y,10:c;4;c2h^1;p 1 1 2/m;-p 2,10:a;4;c2h^1;p 2/m 1 1;-p 2x,11:b;4;c2h^2;p 1 21/m 1;-p 2yb,11:b;4;c2h^2;p 21/m;-p 2yb,11:b*;4;c2h^2;p 1 21/m 1*;-p 2y1,11:c;4;c2h^2;p 1 1 21/m;-p 2c,11:c*;4;c2h^2;p 1 1 21/m*;-p 21,11:a;4;c2h^2;p 21/m 1 1;-p 2xa,11:a*;4;c2h^2;p 21/m 1 1*;-p 2x1,12:b1;8;c2h^3;c 1 2/m 1;-c 2y,12:b1;8;c2h^3;c 2/m;-c 2y,12:b2;8;c2h^3;a 1 2/m 1;-a 2y,12:b3;8;c2h^3;i 1 2/m 1;-i 2y,12:b3;8;c2h^3;i 2/m;-i 2y,12:c1;8;c2h^3;a 1 1 2/m;-a 2,12:c2;8;c2h^3;b 1 1 2/m;-b 2,12:c3;8;c2h^3;i 1 1 2/m;-i 2,12:a1;8;c2h^3;b 2/m 1 1;-b 2x,12:a2;8;c2h^3;c 2/m 1 1;-c 2x,12:a3;8;c2h^3;i 2/m 1 1;-i 2x,13:b1;4;c2h^4;p 1 2/c 1;-p 2yc,13:b1;4;c2h^4;p 2/c;-p 2yc,13:b2;4;c2h^4;p 1 2/n 1;-p 2yac,13:b2;4;c2h^4;p 2/n;-p 2yac,13:b3;4;c2h^4;p 1 2/a 1;-p 2ya,13:b3;4;c2h^4;p 2/a;-p 2ya,13:c1;4;c2h^4;p 1 1 2/a;-p 2a,13:c2;4;c2h^4;p 1 1 2/n;-p 2ab,13:c3;4;c2h^4;p 1 1 2/b;-p 2b,13:a1;4;c2h^4;p 2/b 1 1;-p 2xb,13:a2;4;c2h^4;p 2/n 1 1;-p 2xbc,13:a3;4;c2h^4;p 2/c 1 1;-p 2xc,14:b1;4;c2h^5;p 1 21/c 1;-p 2ybc,14:b1;4;c2h^5;p 21/c;-p 2ybc,14:b2;4;c2h^5;p 1 21/n 1;-p 2yn,14:b2;4;c2h^5;p 21/n;-p 2yn,14:b3;4;c2h^5;p 1 21/a 1;-p 2yab,14:b3;4;c2h^5;p 21/a;-p 2yab,14:c1;4;c2h^5;p 1 1 21/a;-p 2ac,14:c2;4;c2h^5;p 1 1 21/n;-p 2n,14:c3;4;c2h^5;p 1 1 21/b;-p 2bc,14:a1;4;c2h^5;p 21/b 1 1;-p 2xab,14:a2;4;c2h^5;p 21/n 1 1;-p 2xn,14:a3;4;c2h^5;p 21/c 1 1;-p 2xac,15:b1;8;c2h^6;c 1 2/c 1;-c 2yc,15:b1;8;c2h^6;c 2/c;-c 2yc,15:b2;8;c2h^6;a 1 2/n 1;-a 2yab,15:b3;8;c2h^6;i 1 2/a 1;-i 2ya,15:b3;8;c2h^6;i 2/a;-i 2ya,15:-b1;8;c2h^6;a 1 2/a 1;-a 2ya,15:-b2;8;c2h^6;c 1 2/n 1;-c 2yac,15:-b2;8;c2h^6;c 2/n;-c 2yac,15:-b3;8;c2h^6;i 1 2/c 1;-i 2yc,15:-b3;8;c2h^6;i 2/c;-i 2yc,15:c1;8;c2h^6;a 1 1 2/a;-a 2a,15:c2;8;c2h^6;b 1 1 2/n;-b 2ab,15:c3;8;c2h^6;i 1 1 2/b;-i 2b,15:-c1;8;c2h^6;b 1 1 2/b;-b 2b,15:-c2;8;c2h^6;a 1 1 2/n;-a 2ab,15:-c3;8;c2h^6;i 1 1 2/a;-i 2a,15:a1;8;c2h^6;b 2/b 1 1;-b 2xb,15:a2;8;c2h^6;c 2/n 1 1;-c 2xac,15:a3;8;c2h^6;i 2/c 1 1;-i 2xc,15:-a1;8;c2h^6;c 2/c 1 1;-c 2xc,15:-a2;8;c2h^6;b 2/n 1 1;-b 2xab,15:-a3;8;c2h^6;i 2/b 1 1;-i 2xb,16;4;d2^1;p 2 2 2;p 2 2,17;4;d2^2;p 2 2 21;p 2c 2,17*;4;d2^2;p 2 2 21*;p 21 2,17:cab;4;d2^2;p 21 2 2;p 2a 2a,17:bca;4;d2^2;p 2 21 2;p 2 2b,18;4;d2^3;p 21 21 2;p 2 2ab,18:cab;4;d2^3;p 2 21 21;p 2bc 2,18:bca;4;d2^3;p 21 2 21;p 2ac 2ac,19;4;d2^4;p 21 21 21;p 2ac 2ab,20;8;d2^5;c 2 2 21;c 2c 2,20*;8;d2^5;c 2 2 21*;c 21 2,20:cab;8;d2^5;a 21 2 2;a 2a 2a,20:cab*;8;d2^5;a 21 2 2*;a 2a 21,20:bca;8;d2^5;b 2 21 2;b 2 2b,21;8;d2^6;c 2 2 2;c 2 2,21:cab;8;d2^6;a 2 2 2;a 2 2,21:bca;8;d2^6;b 2 2 2;b 2 2,22;16;d2^7;f 2 2 2;f 2 2,23;8;d2^8;i 2 2 2;i 2 2,24;8;d2^9;i 21 21 21;i 2b 2c,25;4;c2v^1;p m m 2;p 2 -2,25:cab;4;c2v^1;p 2 m m;p -2 2,25:bca;4;c2v^1;p m 2 m;p -2 -2,26;4;c2v^2;p m c 21;p 2c -2,26*;4;c2v^2;p m c 21*;p 21 -2,26:ba-c;4;c2v^2;p c m 21;p 2c -2c,26:ba-c*;4;c2v^2;p c m 21*;p 21 -2c,26:cab;4;c2v^2;p 21 m a;p -2a 2a,26:-cba;4;c2v^2;p 21 a m;p -2 2a,26:bca;4;c2v^2;p b 21 m;p -2 -2b,26:a-cb;4;c2v^2;p m 21 b;p -2b -2,27;4;c2v^3;p c c 2;p 2 -2c,27:cab;4;c2v^3;p 2 a a;p -2a 2,27:bca;4;c2v^3;p b 2 b;p -2b -2b,28;4;c2v^4;p m a 2;p 2 -2a,28*;4;c2v^4;p m a 2*;p 2 -21,28:ba-c;4;c2v^4;p b m 2;p 2 -2b,28:cab;4;c2v^4;p 2 m b;p -2b 2,28:-cba;4;c2v^4;p 2 c m;p -2c 2,28:-cba*;4;c2v^4;p 2 c m*;p -21 2,28:bca;4;c2v^4;p c 2 m;p -2c -2c,28:a-cb;4;c2v^4;p m 2 a;p -2a -2a,29;4;c2v^5;p c a 21;p 2c -2ac,29:ba-c;4;c2v^5;p b c 21;p 2c -2b,29:cab;4;c2v^5;p 21 a b;p -2b 2a,29:-cba;4;c2v^5;p 21 c a;p -2ac 2a,29:bca;4;c2v^5;p c 21 b;p -2bc -2c,29:a-cb;4;c2v^5;p b 21 a;p -2a -2ab,30;4;c2v^6;p n c 2;p 2 -2bc,30:ba-c;4;c2v^6;p c n 2;p 2 -2ac,30:cab;4;c2v^6;p 2 n a;p -2ac 2,30:-cba;4;c2v^6;p 2 a n;p -2ab 2,30:bca;4;c2v^6;p b 2 n;p -2ab -2ab,30:a-cb;4;c2v^6;p n 2 b;p -2bc -2bc,31;4;c2v^7;p m n 21;p 2ac -2,31:ba-c;4;c2v^7;p n m 21;p 2bc -2bc,31:cab;4;c2v^7;p 21 m n;p -2ab 2ab,31:-cba;4;c2v^7;p 21 n m;p -2 2ac,31:bca;4;c2v^7;p n 21 m;p -2 -2bc,31:a-cb;4;c2v^7;p m 21 n;p -2ab -2,32;4;c2v^8;p b a 2;p 2 -2ab,32:cab;4;c2v^8;p 2 c b;p -2bc 2,32:bca;4;c2v^8;p c 2 a;p -2ac -2ac,33;4;c2v^9;p n a 21;p 2c -2n,33*;4;c2v^9;p n a 21*;p 21 -2n,33:ba-c;4;c2v^9;p b n 21;p 2c -2ab,33:ba-c*;4;c2v^9;p b n 21*;p 21 -2ab,33:cab;4;c2v^9;p 21 n b;p -2bc 2a,33:cab*;4;c2v^9;p 21 n b*;p -2bc 21,33:-cba;4;c2v^9;p 21 c n;p -2n 2a,33:-cba*;4;c2v^9;p 21 c n*;p -2n 21,33:bca;4;c2v^9;p c 21 n;p -2n -2ac,33:a-cb;4;c2v^9;p n 21 a;p -2ac -2n,34;4;c2v^10;p n n 2;p 2 -2n,34:cab;4;c2v^10;p 2 n n;p -2n 2,34:bca;4;c2v^10;p n 2 n;p -2n -2n,35;8;c2v^11;c m m 2;c 2 -2,35:cab;8;c2v^11;a 2 m m;a -2 2,35:bca;8;c2v^11;b m 2 m;b -2 -2,36;8;c2v^12;c m c 21;c 2c -2,36*;8;c2v^12;c m c 21*;c 21 -2,36:ba-c;8;c2v^12;c c m 21;c 2c -2c,36:ba-c*;8;c2v^12;c c m 21*;c 21 -2c,36:cab;8;c2v^12;a 21 m a;a -2a 2a,36:cab*;8;c2v^12;a 21 m a*;a -2a 21,36:-cba;8;c2v^12;a 21 a m;a -2 2a,36:-cba*;8;c2v^12;a 21 a m*;a -2 21,36:bca;8;c2v^12;b b 21 m;b -2 -2b,36:a-cb;8;c2v^12;b m 21 b;b -2b -2,37;8;c2v^13;c c c 2;c 2 -2c,37:cab;8;c2v^13;a 2 a a;a -2a 2,37:bca;8;c2v^13;b b 2 b;b -2b -2b,38;8;c2v^14;a m m 2;a 2 -2,38:ba-c;8;c2v^14;b m m 2;b 2 -2,38:cab;8;c2v^14;b 2 m m;b -2 2,38:-cba;8;c2v^14;c 2 m m;c -2 2,38:bca;8;c2v^14;c m 2 m;c -2 -2,38:a-cb;8;c2v^14;a m 2 m;a -2 -2,39;8;c2v^15;a e m 2;a 2 -2b,39;8;c2v^15;a b m 2;a 2 -2b,39:ba-c;8;c2v^15;b m a 2;b 2 -2a,39:cab;8;c2v^15;b 2 c m;b -2a 2,39:-cba;8;c2v^15;c 2 m b;c -2a 2,39:bca;8;c2v^15;c m 2 a;c -2a -2a,39:a-cb;8;c2v^15;a c 2 m;a -2b -2b,40;8;c2v^16;a m a 2;a 2 -2a,40:ba-c;8;c2v^16;b b m 2;b 2 -2b,40:cab;8;c2v^16;b 2 m b;b -2b 2,40:-cba;8;c2v^16;c 2 c m;c -2c 2,40:bca;8;c2v^16;c c 2 m;c -2c -2c,40:a-cb;8;c2v^16;a m 2 a;a -2a -2a,41;8;c2v^17;a e a 2;a 2 -2ab,41;8;c2v^17;a b a 2;a 2 -2ab;-b,41:ba-c;8;c2v^17;b b a 2;b 2 -2ab,41:cab;8;c2v^17;b 2 c b;b -2ab 2,41:-cba;8;c2v^17;c 2 c b;c -2ac 2,41:bca;8;c2v^17;c c 2 a;c -2ac -2ac,41:a-cb;8;c2v^17;a c 2 a;a -2ab -2ab,42;16;c2v^18;f m m 2;f 2 -2,42:cab;16;c2v^18;f 2 m m;f -2 2,42:bca;16;c2v^18;f m 2 m;f -2 -2,43;16;c2v^19;f d d 2;f 2 -2d,43:cab;16;c2v^19;f 2 d d;f -2d 2,43:bca;16;c2v^19;f d 2 d;f -2d -2d,44;8;c2v^20;i m m 2;i 2 -2,44:cab;8;c2v^20;i 2 m m;i -2 2,44:bca;8;c2v^20;i m 2 m;i -2 -2,45;8;c2v^21;i b a 2;i 2 -2c,45:cab;8;c2v^21;i 2 c b;i -2a 2,45:bca;8;c2v^21;i c 2 a;i -2b -2b,46;8;c2v^22;i m a 2;i 2 -2a,46:ba-c;8;c2v^22;i b m 2;i 2 -2b,46:cab;8;c2v^22;i 2 m b;i -2b 2,46:-cba;8;c2v^22;i 2 c m;i -2c 2,46:bca;8;c2v^22;i c 2 m;i -2c -2c,46:a-cb;8;c2v^22;i m 2 a;i -2a -2a,47;8;d2h^1;p m m m;-p 2 2,48:1;8;d2h^2;p n n n:1;p 2 2 -1n;-b,48:2;8;d2h^2;p n n n:2;-p 2ab 2bc,49;8;d2h^3;p c c m;-p 2 2c,49:cab;8;d2h^3;p m a a;-p 2a 2,49:bca;8;d2h^3;p b m b;-p 2b 2b,50:1;8;d2h^4;p b a n:1;p 2 2 -1ab;-b,50:2;8;d2h^4;p b a n:2;-p 2ab 2b,50:1cab;8;d2h^4;p n c b:1;p 2 2 -1bc,50:2cab;8;d2h^4;p n c b:2;-p 2b 2bc,50:1bca;8;d2h^4;p c n a:1;p 2 2 -1ac,50:2bca;8;d2h^4;p c n a:2;-p 2a 2c,51;8;d2h^5;p m m a;-p 2a 2a,51:ba-c;8;d2h^5;p m m b;-p 2b 2,51:cab;8;d2h^5;p b m m;-p 2 2b,51:-cba;8;d2h^5;p c m m;-p 2c 2c,51:bca;8;d2h^5;p m c m;-p 2c 2,51:a-cb;8;d2h^5;p m a m;-p 2 2a,52;8;d2h^6;p n n a;-p 2a 2bc,52:ba-c;8;d2h^6;p n n b;-p 2b 2n,52:cab;8;d2h^6;p b n n;-p 2n 2b,52:-cba;8;d2h^6;p c n n;-p 2ab 2c,52:bca;8;d2h^6;p n c n;-p 2ab 2n,52:a-cb;8;d2h^6;p n a n;-p 2n 2bc,53;8;d2h^7;p m n a;-p 2ac 2,53:ba-c;8;d2h^7;p n m b;-p 2bc 2bc,53:cab;8;d2h^7;p b m n;-p 2ab 2ab,53:-cba;8;d2h^7;p c n m;-p 2 2ac,53:bca;8;d2h^7;p n c m;-p 2 2bc,53:a-cb;8;d2h^7;p m a n;-p 2ab 2,54;8;d2h^8;p c c a;-p 2a 2ac,54:ba-c;8;d2h^8;p c c b;-p 2b 2c,54:cab;8;d2h^8;p b a a;-p 2a 2b,54:-cba;8;d2h^8;p c a a;-p 2ac 2c,54:bca;8;d2h^8;p b c b;-p 2bc 2b,54:a-cb;8;d2h^8;p b a b;-p 2b 2ab,55;8;d2h^9;p b a m;-p 2 2ab,55:cab;8;d2h^9;p m c b;-p 2bc 2,55:bca;8;d2h^9;p c m a;-p 2ac 2ac,56;8;d2h^10;p c c n;-p 2ab 2ac,56:cab;8;d2h^10;p n a a;-p 2ac 2bc,56:bca;8;d2h^10;p b n b;-p 2bc 2ab,57;8;d2h^11;p b c m;-p 2c 2b,57:ba-c;8;d2h^11;p c a m;-p 2c 2ac,57:cab;8;d2h^11;p m c a;-p 2ac 2a,57:-cba;8;d2h^11;p m a b;-p 2b 2a,57:bca;8;d2h^11;p b m a;-p 2a 2ab,57:a-cb;8;d2h^11;p c m b;-p 2bc 2c,58;8;d2h^12;p n n m;-p 2 2n,58:cab;8;d2h^12;p m n n;-p 2n 2,58:bca;8;d2h^12;p n m n;-p 2n 2n,59:1;8;d2h^13;p m m n:1;p 2 2ab -1ab;-b,59:2;8;d2h^13;p m m n:2;-p 2ab 2a,59:1cab;8;d2h^13;p n m m:1;p 2bc 2 -1bc,59:2cab;8;d2h^13;p n m m:2;-p 2c 2bc,59:1bca;8;d2h^13;p m n m:1;p 2ac 2ac -1ac,59:2bca;8;d2h^13;p m n m:2;-p 2c 2a,60;8;d2h^14;p b c n;-p 2n 2ab,60:ba-c;8;d2h^14;p c a n;-p 2n 2c,60:cab;8;d2h^14;p n c a;-p 2a 2n,60:-cba;8;d2h^14;p n a b;-p 2bc 2n,60:bca;8;d2h^14;p b n a;-p 2ac 2b,60:a-cb;8;d2h^14;p c n b;-p 2b 2ac,61;8;d2h^15;p b c a;-p 2ac 2ab,61:ba-c;8;d2h^15;p c a b;-p 2bc 2ac,62;8;d2h^16;p n m a;-p 2ac 2n,62:ba-c;8;d2h^16;p m n b;-p 2bc 2a,62:cab;8;d2h^16;p b n m;-p 2c 2ab,62:-cba;8;d2h^16;p c m n;-p 2n 2ac,62:bca;8;d2h^16;p m c n;-p 2n 2a,62:a-cb;8;d2h^16;p n a m;-p 2c 2n,63;16;d2h^17;c m c m;-c 2c 2,63:ba-c;16;d2h^17;c c m m;-c 2c 2c,63:cab;16;d2h^17;a m m a;-a 2a 2a,63:-cba;16;d2h^17;a m a m;-a 2 2a,63:bca;16;d2h^17;b b m m;-b 2 2b,63:a-cb;16;d2h^17;b m m b;-b 2b 2,64;16;d2h^18;c m c e;-c 2ac 2,64;16;d2h^18;c m c a;-c 2ac 2,64:ba-c;16;d2h^18;c c m b;-c 2ac 2ac,64:cab;16;d2h^18;a b m a;-a 2ab 2ab,64:-cba;16;d2h^18;a c a m;-a 2 2ab,64:bca;16;d2h^18;b b c m;-b 2 2ab,64:a-cb;16;d2h^18;b m a b;-b 2ab 2,65;16;d2h^19;c m m m;-c 2 2,65:cab;16;d2h^19;a m m m;-a 2 2,65:bca;16;d2h^19;b m m m;-b 2 2,66;16;d2h^20;c c c m;-c 2 2c,66:cab;16;d2h^20;a m a a;-a 2a 2,66:bca;16;d2h^20;b b m b;-b 2b 2b,67;16;d2h^21;c m m e;-c 2a 2,67;16;d2h^21;c m m a;-c 2a 2,67:ba-c;16;d2h^21;c m m b;-c 2a 2a,67:cab;16;d2h^21;a b m m;-a 2b 2b,67:-cba;16;d2h^21;a c m m;-a 2 2b,67:bca;16;d2h^21;b m c m;-b 2 2a,67:a-cb;16;d2h^21;b m a m;-b 2a 2,68:1;16;d2h^22;c c c e:1;c 2 2 -1ac;-b,68:1;16;d2h^22;c c c a:1;c 2 2 -1ac;-b,68:2;16;d2h^22;c c c e:2;-c 2a 2ac,68:2;16;d2h^22;c c c a:2;-c 2a 2ac,68:1ba-c;16;d2h^22;c c c b:1;c 2 2 -1ac,68:2ba-c;16;d2h^22;c c c b:2;-c 2a 2c,68:1cab;16;d2h^22;a b a a:1;a 2 2 -1ab,68:2cab;16;d2h^22;a b a a:2;-a 2a 2b,68:1-cba;16;d2h^22;a c a a:1;a 2 2 -1ab,68:2-cba;16;d2h^22;a c a a:2;-a 2ab 2b,68:1bca;16;d2h^22;b b c b:1;b 2 2 -1ab,68:2bca;16;d2h^22;b b c b:2;-b 2ab 2b,68:1a-cb;16;d2h^22;b b a b:1;b 2 2 -1ab,68:2a-cb;16;d2h^22;b b a b:2;-b 2b 2ab,69;32;d2h^23;f m m m;-f 2 2,70:1;32;d2h^24;f d d d:1;f 2 2 -1d;-b,70:2;32;d2h^24;f d d d:2;-f 2uv 2vw,71;16;d2h^25;i m m m;-i 2 2,72;16;d2h^26;i b a m;-i 2 2c,72:cab;16;d2h^26;i m c b;-i 2a 2,72:bca;16;d2h^26;i c m a;-i 2b 2b,73;16;d2h^27;i b c a;-i 2b 2c,73:ba-c;16;d2h^27;i c a b;-i 2a 2b,74;16;d2h^28;i m m a;-i 2b 2,74:ba-c;16;d2h^28;i m m b;-i 2a 2a,74:cab;16;d2h^28;i b m m;-i 2c 2c,74:-cba;16;d2h^28;i c m m;-i 2 2b,74:bca;16;d2h^28;i m c m;-i 2 2a,74:a-cb;16;d2h^28;i m a m;-i 2c 2,75;4;c4^1;p 4;p 4,76;4;c4^2;p 41;p 4w,76*;4;c4^2;p 41*;p 41,77;4;c4^3;p 42;p 4c,77*;4;c4^3;p 42*;p 42,78;4;c4^4;p 43;p 4cw,78*;4;c4^4;p 43*;p 43,79;8;c4^5;i 4;i 4,80;8;c4^6;i 41;i 4bw,81;4;s4^1;p -4;p -4,82;8;s4^2;i -4;i -4,83;8;c4h^1;p 4/m;-p 4,84;8;c4h^2;p 42/m;-p 4c,84*;8;c4h^2;p 42/m*;-p 42,85:1;8;c4h^3;p 4/n:1;p 4ab -1ab;-b,85:2;8;c4h^3;p 4/n:2;-p 4a,86:1;8;c4h^4;p 42/n:1;p 4n -1n;-b,86:2;8;c4h^4;p 42/n:2;-p 4bc,87;16;c4h^5;i 4/m;-i 4,88:1;16;c4h^6;i 41/a:1;i 4bw -1bw;-b,88:2;16;c4h^6;i 41/a:2;-i 4ad,89;8;d4^1;p 4 2 2;p 4 2,90;8;d4^2;p 4 21 2;p 4ab 2ab,91;8;d4^3;p 41 2 2;p 4w 2c,91*;8;d4^3;p 41 2 2*;p 41 2c,92;8;d4^4;p 41 21 2;p 4abw 2nw,93;8;d4^5;p 42 2 2;p 4c 2,93*;8;d4^5;p 42 2 2*;p 42 2,94;8;d4^6;p 42 21 2;p 4n 2n,95;8;d4^7;p 43 2 2;p 4cw 2c,95*;8;d4^7;p 43 2 2*;p 43 2c,96;8;d4^8;p 43 21 2;p 4nw 2abw,97;16;d4^9;i 4 2 2;i 4 2,98;16;d4^10;i 41 2 2;i 4bw 2bw,99;8;c4v^1;p 4 m m;p 4 -2,100;8;c4v^2;p 4 b m;p 4 -2ab,101;8;c4v^3;p 42 c m;p 4c -2c,101*;8;c4v^3;p 42 c m*;p 42 -2c,102;8;c4v^4;p 42 n m;p 4n -2n,103;8;c4v^5;p 4 c c;p 4 -2c,104;8;c4v^6;p 4 n c;p 4 -2n,105;8;c4v^7;p 42 m c;p 4c -2,105*;8;c4v^7;p 42 m c*;p 42 -2,106;8;c4v^8;p 42 b c;p 4c -2ab,106*;8;c4v^8;p 42 b c*;p 42 -2ab,107;16;c4v^9;i 4 m m;i 4 -2,108;16;c4v^10;i 4 c m;i 4 -2c,109;16;c4v^11;i 41 m d;i 4bw -2,110;16;c4v^12;i 41 c d;i 4bw -2c,111;8;d2d^1;p -4 2 m;p -4 2,112;8;d2d^2;p -4 2 c;p -4 2c,113;8;d2d^3;p -4 21 m;p -4 2ab,114;8;d2d^4;p -4 21 c;p -4 2n,115;8;d2d^5;p -4 m 2;p -4 -2,116;8;d2d^6;p -4 c 2;p -4 -2c,117;8;d2d^7;p -4 b 2;p -4 -2ab,118;8;d2d^8;p -4 n 2;p -4 -2n,119;16;d2d^9;i -4 m 2;i -4 -2,120;16;d2d^10;i -4 c 2;i -4 -2c,121;16;d2d^11;i -4 2 m;i -4 2,122;16;d2d^12;i -4 2 d;i -4 2bw,123;16;d4h^1;p 4/m m m;-p 4 2,124;16;d4h^2;p 4/m c c;-p 4 2c,125:1;16;d4h^3;p 4/n b m:1;p 4 2 -1ab;-b,125:2;16;d4h^3;p 4/n b m:2;-p 4a 2b,126:1;16;d4h^4;p 4/n n c:1;p 4 2 -1n;-b,126:2;16;d4h^4;p 4/n n c:2;-p 4a 2bc,127;16;d4h^5;p 4/m b m;-p 4 2ab,128;16;d4h^6;p 4/m n c;-p 4 2n,129:1;16;d4h^7;p 4/n m m:1;p 4ab 2ab -1ab;-b,129:2;16;d4h^7;p 4/n m m:2;-p 4a 2a,130:1;16;d4h^8;p 4/n c c:1;p 4ab 2n -1ab;-b,130:2;16;d4h^8;p 4/n c c:2;-p 4a 2ac,131;16;d4h^9;p 42/m m c;-p 4c 2,132;16;d4h^10;p 42/m c m;-p 4c 2c,133:1;16;d4h^11;p 42/n b c:1;p 4n 2c -1n;-b,133:2;16;d4h^11;p 42/n b c:2;-p 4ac 2b,134:1;16;d4h^12;p 42/n n m:1;p 4n 2 -1n;-b,134:2;16;d4h^12;p 42/n n m:2;-p 4ac 2bc,135;16;d4h^13;p 42/m b c;-p 4c 2ab,135*;16;d4h^13;p 42/m b c*;-p 42 2ab,136;16;d4h^14;p 42/m n m;-p 4n 2n,137:1;16;d4h^15;p 42/n m c:1;p 4n 2n -1n;-b,137:2;16;d4h^15;p 42/n m c:2;-p 4ac 2a,138:1;16;d4h^16;p 42/n c m:1;p 4n 2ab -1n;-b,138:2;16;d4h^16;p 42/n c m:2;-p 4ac 2ac,139;32;d4h^17;i 4/m m m;-i 4 2,140;32;d4h^18;i 4/m c m;-i 4 2c,141:1;32;d4h^19;i 41/a m d:1;i 4bw 2bw -1bw;-b,141:2;32;d4h^19;i 41/a m d:2;-i 4bd 2,142:1;32;d4h^20;i 41/a c d:1;i 4bw 2aw -1bw;-b,142:2;32;d4h^20;i 41/a c d:2;-i 4bd 2c,143;3;c3^1;p 3;p 3,144;3;c3^2;p 31;p 31,145;3;c3^3;p 32;p 32,146:h;9;c3^4;r 3:h;r 3,146:r;3;c3^4;r 3:r;p 3*,147;6;c3i^1;p -3;-p 3,148:h;18;c3i^2;r -3:h;-r 3,148:r;6;c3i^2;r -3:r;-p 3*,149;6;d3^1;p 3 1 2;p 3 2,150;6;d3^2;p 3 2 1;p 3 2",151;6;d3^3;p 31 1 2;p 31 2 (0 0 4),152;6;d3^4;p 31 2 1;p 31 2",153;6;d3^5;p 32 1 2;p 32 2 (0 0 2),154;6;d3^6;p 32 2 1;p 32 2",155:h;18;d3^7;r 3 2:h;r 3 2",155:r;6;d3^7;r 3 2:r;p 3* 2,156;6;c3v^1;p 3 m 1;p 3 -2",157;6;c3v^2;p 3 1 m;p 3 -2,158;6;c3v^3;p 3 c 1;p 3 -2"c,159;6;c3v^4;p 3 1 c;p 3 -2c,160:h;18;c3v^5;r 3 m:h;r 3 -2",160:r;6;c3v^5;r 3 m:r;p 3* -2,161:h;18;c3v^6;r 3 c:h;r 3 -2"c,161:r;6;c3v^6;r 3 c:r;p 3* -2n,162;12;d3d^1;p -3 1 m;-p 3 2,163;12;d3d^2;p -3 1 c;-p 3 2c,164;12;d3d^3;p -3 m 1;-p 3 2",165;12;d3d^4;p -3 c 1;-p 3 2"c,166:h;36;d3d^5;r -3 m:h;-r 3 2",166:r;12;d3d^5;r -3 m:r;-p 3* 2,167:h;36;d3d^6;r -3 c:h;-r 3 2"c,167:r;12;d3d^6;r -3 c:r;-p 3* 2n,168;6;c6^1;p 6;p 6,169;6;c6^2;p 61;p 61,170;6;c6^3;p 65;p 65,171;6;c6^4;p 62;p 62,172;6;c6^5;p 64;p 64,173;6;c6^6;p 63;p 6c,173*;6;c6^6;p 63*;p 63 ,174;6;c3h^1;p -6;p -6,175;12;c6h^1;p 6/m;-p 6,176;12;c6h^2;p 63/m;-p 6c,176*;12;c6h^2;p 63/m*;-p 63,177;12;d6^1;p 6 2 2;p 6 2,178;12;d6^2;p 61 2 2;p 61 2 (0 0 5),179;12;d6^3;p 65 2 2;p 65 2 (0 0 1),180;12;d6^4;p 62 2 2;p 62 2 (0 0 4),181;12;d6^5;p 64 2 2;p 64 2 (0 0 2),182;12;d6^6;p 63 2 2;p 6c 2c,182*;12;d6^6;p 63 2 2*;p 63 2c,183;12;c6v^1;p 6 m m;p 6 -2,184;12;c6v^2;p 6 c c;p 6 -2c,185;12;c6v^3;p 63 c m;p 6c -2,185*;12;c6v^3;p 63 c m*;p 63 -2,186;12;c6v^4;p 63 m c;p 6c -2c,186*;12;c6v^4;p 63 m c*;p 63 -2c,187;12;d3h^1;p -6 m 2;p -6 2,188;12;d3h^2;p -6 c 2;p -6c 2,189;12;d3h^3;p -6 2 m;p -6 -2,190;12;d3h^4;p -6 2 c;p -6c -2c,191;24;d6h^1;p 6/m m m;-p 6 2,192;24;d6h^2;p 6/m c c;-p 6 2c,193;24;d6h^3;p 63/m c m;-p 6c 2,193*;24;d6h^3;p 63/m c m*;-p 63 2,194;24;d6h^4;p 63/m m c;-p 6c 2c,194*;24;d6h^4;p 63/m m c*;-p 63 2c,195;12;t^1;p 2 3;p 2 2 3,196;48;t^2;f 2 3;f 2 2 3,197;24;t^3;i 2 3;i 2 2 3,198;12;t^4;p 21 3;p 2ac 2ab 3,199;24;t^5;i 21 3;i 2b 2c 3,200;24;th^1;p m -3;-p 2 2 3,201:1;24;th^2;p n -3:1;p 2 2 3 -1n;-b,201:2;24;th^2;p n -3:2;-p 2ab 2bc 3,202;96;th^3;f m -3;-f 2 2 3,203:1;96;th^4;f d -3:1;f 2 2 3 -1d;-b,203:2;96;th^4;f d -3:2;-f 2uv 2vw 3,204;48;th^5;i m -3;-i 2 2 3,205;24;th^6;p a -3;-p 2ac 2ab 3,206;48;th^7;i a -3;-i 2b 2c 3,207;24;o^1;p 4 3 2;p 4 2 3,208;24;o^2;p 42 3 2;p 4n 2 3,209;96;o^3;f 4 3 2;f 4 2 3,210;96;o^4;f 41 3 2;f 4d 2 3,211;48;o^5;i 4 3 2;i 4 2 3,212;24;o^6;p 43 3 2;p 4acd 2ab 3,213;24;o^7;p 41 3 2;p 4bd 2ab 3,214;48;o^8;i 41 3 2;i 4bd 2c 3,215;24;td^1;p -4 3 m;p -4 2 3,216;96;td^2;f -4 3 m;f -4 2 3,217;48;td^3;i -4 3 m;i -4 2 3,218;24;td^4;p -4 3 n;p -4n 2 3,219;96;td^5;f -4 3 c;f -4a 2 3,220;48;td^6;i -4 3 d;i -4bd 2c 3,221;48;oh^1;p m -3 m;-p 4 2 3,222:1;48;oh^2;p n -3 n:1;p 4 2 3 -1n;-b,222:2;48;oh^2;p n -3 n:2;-p 4a 2bc 3,223;48;oh^3;p m -3 n;-p 4n 2 3,224:1;48;oh^4;p n -3 m:1;p 4n 2 3 -1n;-b,224:2;48;oh^4;p n -3 m:2;-p 4bc 2bc 3,225;192;oh^5;f m -3 m;-f 4 2 3,226;192;oh^6;f m -3 c;-f 4a 2 3,227:1;192;oh^7;f d -3 m:1;f 4d 2 3 -1d;-b,227:2;192;oh^7;f d -3 m:2;-f 4vw 2vw 3,228:1;192;oh^8;f d -3 c:1;f 4d 2 3 -1ad;-b,228:2;192;oh^8;f d -3 c:2;-f 4ud 2vw 3,229;96;oh^9;i m -3 m;-i 4 2 3,230;96;oh^10;i a -3 d;-i 4bd 2c 3'.split(",")))});
M("JS");B(null,"JS.HallInfo",["JU.P3i","$.SB","JS.HallRotationTerm","$.HallTranslation","JU.Logger"],function(){c$=N(function(){this.primitiveHallSymbol=this.hallSymbol=null;this.latticeCode="\x00";this.latticeExtension=null;this.isCentrosymmetric=!1;this.nRotations=0;this.vectorCode=this.vector12ths=this.rotationTerms=null;R(this,arguments)},JS,"HallInfo");U(c$,function(){this.rotationTerms=Array(16)});D(c$,function(a){try{var b=this.hallSymbol=a.trim(),b=this.extractLatticeInfo(b);if(0!=JS.HallTranslation.getLatticeIndex(this.latticeCode)){this.latticeExtension=
JS.HallTranslation.getLatticeExtension(this.latticeCode,this.isCentrosymmetric);b=this.extractVectorInfo(b)+this.latticeExtension;JU.Logger.debugging&&JU.Logger.debug("Hallinfo: "+a+" "+b);a=0;var c="\x00";for(this.primitiveHallSymbol="P";0<b.length&&16>this.nRotations;){var b=this.extractRotationInfo(b,a,c),f=this.rotationTerms[this.nRotations-1];a=f.order;c=f.axisType;this.primitiveHallSymbol+=" "+f.primitiveCode}this.primitiveHallSymbol+=this.vectorCode}}catch(d){if(X(d,Exception))JU.Logger.error("Invalid Hall symbol "+
d),this.nRotations=0;else throw d;}},"~S");d(c$,"dumpInfo",function(){var a=new JU.SB;a.append("\nHall symbol: ").append(this.hallSymbol).append("\nprimitive Hall symbol: ").append(this.primitiveHallSymbol).append("\nlattice type: ").append(this.getLatticeDesignation());for(var b=0;b<this.nRotations;b++)a.append("\n\nrotation term ").appendI(b+1).append(this.rotationTerms[b].dumpInfo(this.vectorCode));return a.toString()});d(c$,"getLatticeDesignation",function(){return JS.HallTranslation.getLatticeDesignation2(this.latticeCode,
this.isCentrosymmetric)});d(c$,"extractLatticeInfo",function(a){var b=a.indexOf(" ");if(0>b)return"";var c=a.substring(0,b).toUpperCase();this.latticeCode=c.charAt(0);"-"==this.latticeCode&&(this.isCentrosymmetric=!0,this.latticeCode=c.charAt(1));return a.substring(b+1).trim()},"~S");d(c$,"extractVectorInfo",function(a){this.vector12ths=new JU.P3i;this.vectorCode="";var b=a.indexOf("("),c=a.indexOf(")",b);0<b&&c>b&&(c=a.substring(b+1,c),this.vectorCode=" ("+c+")",a=a.substring(0,b).trim(),b=c.indexOf(" "),
0<=b&&(this.vector12ths.x=Integer.parseInt(c.substring(0,b)),c=c.substring(b+1).trim(),b=c.indexOf(" "),0<=b&&(this.vector12ths.y=Integer.parseInt(c.substring(0,b)),c=c.substring(b+1).trim())),this.vector12ths.z=Integer.parseInt(c));return a},"~S");d(c$,"extractRotationInfo",function(a,b,c){var f=a.indexOf(" "),d;0<=f?(d=a.substring(0,f),a=a.substring(f+1).trim()):(d=a,a="");this.rotationTerms[this.nRotations]=new JS.HallRotationTerm(this,d,b,c);this.nRotations++;return a},"~S,~N,~S")});M("JS");B(["JU.M4"],
"JS.HallRotationTerm",["JU.SB","JS.HallRotation","$.HallTranslation","$.SymmetryOperation","JU.Logger"],function(){c$=N(function(){this.seitzMatrix12ths=this.translation=this.rotation=this.translationString=this.lookupCode=this.primitiveCode=this.inputCode=null;this.isImproper=!1;this.order=0;this.diagonalReferenceAxis=this.axisType="\x00";this.allPositive=!0;R(this,arguments)},JS,"HallRotationTerm");U(c$,function(){this.seitzMatrix12ths=new JU.M4});D(c$,function(a,b,c,f){this.inputCode=b;b+="   ";
"-"==b.charAt(0)&&(this.isImproper=!0,b=b.substring(1));this.primitiveCode="";this.order=b.charCodeAt(0)-48;this.axisType=this.diagonalReferenceAxis="\x00";var d=2,e;switch(e=b.charAt(1)){case "x":case "y":case "z":switch(b.charAt(2)){case "'":case '"':this.diagonalReferenceAxis=e,e=b.charAt(2),d++}case "*":this.axisType=e;break;case "'":case '"':this.axisType=e;switch(b.charAt(2)){case "x":case "y":case "z":this.diagonalReferenceAxis=b.charAt(2);d++;break;default:this.diagonalReferenceAxis=f}break;
default:this.axisType=1==this.order?"_":0==a.nRotations?"z":2==a.nRotations?"*":2==c||4==c?"x":"'",b=b.substring(0,1)+this.axisType+b.substring(1)}this.primitiveCode+="_"==this.axisType?"1":b.substring(0,2);"\x00"!=this.diagonalReferenceAxis&&(b=b.substring(0,1)+this.diagonalReferenceAxis+this.axisType+b.substring(d),this.primitiveCode+=this.diagonalReferenceAxis,d=3);this.lookupCode=b.substring(0,d);this.rotation=JS.HallRotation.lookup(this.lookupCode);if(null==this.rotation)JU.Logger.error("Rotation lookup could not find "+
this.inputCode+" ? "+this.lookupCode);else{this.translation=new JS.HallTranslation("\x00",null);this.translationString="";for(c=b.length;d<c;d++)f=b.charAt(d),f=JS.HallTranslation.getHallTranslation(f,this.order),null!=f&&(this.translationString+=""+f.translationCode,this.translation.rotationShift12ths+=f.rotationShift12ths,this.translation.vectorShift12ths.add(f.vectorShift12ths));this.primitiveCode=(this.isImproper?"-":"")+this.primitiveCode+this.translationString;this.seitzMatrix12ths.setM4(this.isImproper?
this.rotation.seitzMatrixInv:this.rotation.seitzMatrix);this.seitzMatrix12ths.m03=this.translation.vectorShift12ths.x;this.seitzMatrix12ths.m13=this.translation.vectorShift12ths.y;this.seitzMatrix12ths.m23=this.translation.vectorShift12ths.z;switch(this.axisType){case "x":this.seitzMatrix12ths.m03+=this.translation.rotationShift12ths;break;case "y":this.seitzMatrix12ths.m13+=this.translation.rotationShift12ths;break;case "z":this.seitzMatrix12ths.m23+=this.translation.rotationShift12ths}0<a.vectorCode.length&&
(d=JU.M4.newM4(null),c=JU.M4.newM4(null),a=a.vector12ths,d.m03=a.x,d.m13=a.y,d.m23=a.z,c.m03=-a.x,c.m13=-a.y,c.m23=-a.z,this.seitzMatrix12ths.mul2(d,this.seitzMatrix12ths),this.seitzMatrix12ths.mul(c));JU.Logger.debugging&&JU.Logger.debug("code = "+b+"; primitive code ="+this.primitiveCode+"\n Seitz Matrix(12ths):"+this.seitzMatrix12ths)}},"JS.HallInfo,~S,~N,~S");d(c$,"dumpInfo",function(a){var b=new JU.SB;b.append("\ninput code: ").append(this.inputCode).append("; primitive code: ").append(this.primitiveCode).append("\norder: ").appendI(this.order).append(this.isImproper?
" (improper axis)":"");"_"!=this.axisType&&(b.append("; axisType: ").appendC(this.axisType),"\x00"!=this.diagonalReferenceAxis&&b.appendC(this.diagonalReferenceAxis));0<this.translationString.length&&b.append("; translation: ").append(this.translationString);0<a.length&&b.append("; vector offset: ").append(a);null!=this.rotation&&b.append("\noperator: ").append(this.getXYZ(this.allPositive)).append("\nSeitz matrix:\n").append(JS.SymmetryOperation.dumpSeitz(this.seitzMatrix12ths,!1));return b.toString()},
"~S");d(c$,"getXYZ",function(a){return JS.SymmetryOperation.getXYZFromMatrix(this.seitzMatrix12ths,!0,a,!0)},"~B")});M("JS");B(["JU.M4"],"JS.HallRotation",null,function(){c$=N(function(){this.seitzMatrixInv=this.seitzMatrix=this.rotCode=null;R(this,arguments)},JS,"HallRotation");U(c$,function(){this.seitzMatrix=new JU.M4;this.seitzMatrixInv=new JU.M4});D(c$,function(a,b){this.rotCode=a;var c=r(16,0),f=r(16,0);c[15]=f[15]=1;for(var d=0,e=0;11>e;d++){var g=0;switch(b.charAt(d)){case " ":e++;continue;
case "+":case "1":g=1;break;case "-":g=-1}c[e]=g;f[e]=-g;e++}this.seitzMatrix.setA(c);this.seitzMatrixInv.setA(f)},"~S,~S");c$.lookup=d(c$,"lookup",function(a){for(var b=JS.HallRotation.getHallTerms().length;0<=--b;)if(JS.HallRotation.hallRotationTerms[b].rotCode.equals(a))return JS.HallRotation.hallRotationTerms[b];return null},"~S");c$.getHallTerms=d(c$,"getHallTerms",function(){return null==JS.HallRotation.hallRotationTerms?JS.HallRotation.hallRotationTerms=K(-1,[new JS.HallRotation("1_","+00 0+0 00+"),
new JS.HallRotation("2x","+00 0-0 00-"),new JS.HallRotation("2y","-00 0+0 00-"),new JS.HallRotation("2z","-00 0-0 00+"),new JS.HallRotation("2'","0-0 -00 00-"),new JS.HallRotation('2"',"0+0 +00 00-"),new JS.HallRotation("2x'","-00 00- 0-0"),new JS.HallRotation('2x"',"-00 00+ 0+0"),new JS.HallRotation("2y'","00- 0-0 -00"),new JS.HallRotation('2y"',"00+ 0-0 +00"),new JS.HallRotation("2z'","0-0 -00 00-"),new JS.HallRotation('2z"',"0+0 +00 00-"),new JS.HallRotation("3x","+00 00- 0+-"),new JS.HallRotation("3y",
"-0+ 0+0 -00"),new JS.HallRotation("3z","0-0 +-0 00+"),new JS.HallRotation("3*","00+ +00 0+0"),new JS.HallRotation("4x","+00 00- 0+0"),new JS.HallRotation("4y","00+ 0+0 -00"),new JS.HallRotation("4z","0-0 +00 00+"),new JS.HallRotation("6x","+00 0+- 0+0"),new JS.HallRotation("6y","00+ 0+0 -0+"),new JS.HallRotation("6z","+-0 +00 00+")]):JS.HallRotation.hallRotationTerms});G(c$,"hallRotationTerms",null)});M("JS");B(null,"JS.HallTranslation",["JU.P3i"],function(){c$=N(function(){this.translationCode=
"\x00";this.rotationShift12ths=this.rotationOrder=0;this.vectorShift12ths=null;R(this,arguments)},JS,"HallTranslation");D(c$,function(a,b){this.translationCode=a;if(null!=b){if(0<=b.z){this.vectorShift12ths=b;return}this.rotationOrder=b.x;this.rotationShift12ths=b.y}this.vectorShift12ths=new JU.P3i},"~S,JU.P3i");c$.getHallLatticeEquivalent=d(c$,"getHallLatticeEquivalent",function(a){var b=JS.HallTranslation.getLatticeCode(a);return(0<a?"-":"")+b+" 1"},"~N");c$.getLatticeIndex=d(c$,"getLatticeIndex",
function(a){for(var b=1,c=3;b<=JS.HallTranslation.nLatticeTypes;b++,c+=3)if(JS.HallTranslation.latticeTranslationData[c].charAt(0)==a)return b;return 0},"~S");c$.getLatticeCode=d(c$,"getLatticeCode",function(a){0>a&&(a=-a);return 0==a?"\x00":a>JS.HallTranslation.nLatticeTypes?JS.HallTranslation.getLatticeCode(JS.HallTranslation.getLatticeIndex(String.fromCharCode(a))):JS.HallTranslation.latticeTranslationData[3*a].charAt(0)},"~N");c$.getLatticeDesignation=d(c$,"getLatticeDesignation",function(a){var b=
0<a;0>a&&(a=-a);return 0==a||a>JS.HallTranslation.nLatticeTypes?"":(b?"-":"")+JS.HallTranslation.getLatticeCode(a)+": "+(b?"centrosymmetric ":"")+JS.HallTranslation.latticeTranslationData[3*a+1]},"~N");c$.getLatticeDesignation2=d(c$,"getLatticeDesignation2",function(a,b){var c=JS.HallTranslation.getLatticeIndex(a);b||(c=-c);return JS.HallTranslation.getLatticeDesignation(c)},"~S,~B");c$.getLatticeExtension=d(c$,"getLatticeExtension",function(a,b){for(var c=1,f=3;c<=JS.HallTranslation.nLatticeTypes;c++,
f+=3)if(JS.HallTranslation.latticeTranslationData[f].charAt(0)==a)return JS.HallTranslation.latticeTranslationData[f+2]+(b?" -1":"");return""},"~S,~B");c$.getHallTerms=d(c$,"getHallTerms",function(){return null==JS.HallTranslation.hallTranslationTerms?JS.HallTranslation.hallTranslationTerms=K(-1,[new JS.HallTranslation("a",JU.P3i.new3(6,0,0)),new JS.HallTranslation("b",JU.P3i.new3(0,6,0)),new JS.HallTranslation("c",JU.P3i.new3(0,0,6)),new JS.HallTranslation("n",JU.P3i.new3(6,6,6)),new JS.HallTranslation("u",
JU.P3i.new3(3,0,0)),new JS.HallTranslation("v",JU.P3i.new3(0,3,0)),new JS.HallTranslation("w",JU.P3i.new3(0,0,3)),new JS.HallTranslation("d",JU.P3i.new3(3,3,3)),new JS.HallTranslation("1",JU.P3i.new3(2,6,-1)),new JS.HallTranslation("1",JU.P3i.new3(3,4,-1)),new JS.HallTranslation("2",JU.P3i.new3(3,8,-1)),new JS.HallTranslation("1",JU.P3i.new3(4,3,-1)),new JS.HallTranslation("3",JU.P3i.new3(4,9,-1)),new JS.HallTranslation("1",JU.P3i.new3(6,2,-1)),new JS.HallTranslation("2",JU.P3i.new3(6,4,-1)),new JS.HallTranslation("4",
JU.P3i.new3(6,8,-1)),new JS.HallTranslation("5",JU.P3i.new3(6,10,-1)),new JS.HallTranslation("r",JU.P3i.new3(4,8,8)),new JS.HallTranslation("s",JU.P3i.new3(8,8,4)),new JS.HallTranslation("t",JU.P3i.new3(8,4,8))]):JS.HallTranslation.hallTranslationTerms});c$.getHallTranslation=d(c$,"getHallTranslation",function(a,b){for(var c=null,f=JS.HallTranslation.getHallTerms().length;0<=--f;){var d=JS.HallTranslation.hallTranslationTerms[f];if(d.translationCode==a&&(0==d.rotationOrder||d.rotationOrder==b)){c=
new JS.HallTranslation(a,null);c.translationCode=a;c.rotationShift12ths=d.rotationShift12ths;c.vectorShift12ths=d.vectorShift12ths;break}}return c},"~S,~N");G(c$,"latticeTranslationData",K(-1,"\x00;unknown;;P;primitive;;I;body-centered; 1n;R;rhombohedral; 1r 1r;F;face-centered; 1ab 1bc 1ac;A;A-centered; 1bc;B;B-centered; 1ac;C;C-centered; 1ab;S;rhombohedral(S); 1s 1s;T;rhombohedral(T); 1t 1t".split(";")));c$.nLatticeTypes=c$.prototype.nLatticeTypes=u(JS.HallTranslation.latticeTranslationData.length/
3)-1;G(c$,"hallTranslationTerms",null)});M("JS");B(["JU.M4"],"JS.SymmetryOperation","java.lang.Boolean $.Character $.Float java.util.Hashtable JU.Matrix $.P3 $.PT $.SB $.V3 JU.Logger $.Parser".split(" "),function(){c$=N(function(){this.xyz=this.xyzCanonical=this.xyzOriginal=null;this.doNormalize=!0;this.isFinalized=!1;this.opId=0;this.myLabels=this.centering=null;this.modDim=0;this.rsvs=this.linearRotTrans=null;this.isBio=!1;this.sigma=null;this.number=0;this.subZystemCode=null;this.timeReversal=
0;this.isCenteringOp=this.unCentered=!1;this.magOp=3.4028235E38;this.divisor=12;this.info=null;R(this,arguments)},JS,"SymmetryOperation",JU.M4);d(c$,"setSigma",function(a,b){this.subZystemCode=a;this.sigma=b},"~S,JU.Matrix");D(c$,function(a,b,c,f,d){da(this,JS.SymmetryOperation,[]);this.doNormalize=d;null==a?this.opId=f:(this.xyzOriginal=a.xyzOriginal,this.xyz=a.xyz,this.divisor=a.divisor,this.opId=a.opId,this.modDim=a.modDim,this.myLabels=a.myLabels,this.number=a.number,this.linearRotTrans=a.linearRotTrans,
this.sigma=a.sigma,this.subZystemCode=a.subZystemCode,this.timeReversal=a.timeReversal,this.setMatrix(!1),a.isFinalized||this.doFinalize(),d&&null==this.sigma&&JS.SymmetryOperation.setOffset(this,b,c,f))},"JS.SymmetryOperation,~A,~N,~N,~B");d(c$,"setGamma",function(a){for(var b=3+this.modDim,c=(this.rsvs=new JU.Matrix(null,b+1,b+1)).getArray(),f=aa(b,0),d=0,e=0;e<b;e++){for(var g=0;g<b;g++)c[e][g]=this.linearRotTrans[d++];f[e]=(a?-1:1)*this.linearRotTrans[d++]}c[b][b]=1;a&&(this.rsvs=this.rsvs.inverse());
for(e=0;e<b;e++)c[e][b]=f[e];c=this.rsvs.getSubmatrix(0,0,3,3).getArray();for(e=0;3>e;e++)for(g=0;4>g;g++)this.setElement(e,g,3>g?c[e][g]:f[e]);this.setElement(3,3,1)},"~B");d(c$,"doFinalize",function(){JS.SymmetryOperation.div12(this,this.divisor);if(0<this.modDim)for(var a=this.rsvs.getArray(),b=a.length-1;0<=--b;)a[b][3+this.modDim]=JS.SymmetryOperation.finalizeD(a[b][3+this.modDim],this.divisor);this.isFinalized=!0});c$.div12=d(c$,"div12",function(a,b){a.m03=JS.SymmetryOperation.finalizeF(a.m03,
b);a.m13=JS.SymmetryOperation.finalizeF(a.m13,b);a.m23=JS.SymmetryOperation.finalizeF(a.m23,b);return a},"JU.M4,~N");c$.finalizeF=d(c$,"finalizeF",function(a,b){if(0==b){if(0==a)return 0;var c=p(a);return 1*(c>>8)/(c&255)}return a/b},"~N,~N");c$.finalizeD=d(c$,"finalizeD",function(a,b){if(0==b){if(0==a)return 0;var c=u(a);return 1*(c>>8)/(c&255)}return a/b},"~N,~N");d(c$,"getXyz",function(a){return a&&0==this.modDim||null==this.xyzOriginal?this.xyz:this.xyzOriginal},"~B");d(c$,"getxyzTrans",function(a){var b=
JU.M4.newM4(this);b.add(a);return JS.SymmetryOperation.getXYZFromMatrix(b,!1,!1,!1)},"JU.P3");c$.newPoint=d(c$,"newPoint",function(a,b,c,f,d,e){a.rotTrans2(b,c);c.add3(f,d,e)},"JU.M4,JU.P3,JU.P3,~N,~N,~N");d(c$,"dumpInfo",function(){return"\n"+this.xyz+"\ninternal matrix representation:\n"+this.toString()});c$.dumpSeitz=d(c$,"dumpSeitz",function(a,b){for(var c=new JU.SB,f=r(4,0),d=0;3>d;d++){a.getRow(d,f);c.append("[\t");for(var e=0;3>e;e++)c.appendI(p(f[e])).append("\t");e=f[3];e!=p(e)&&(e*=12);
c.append(JS.SymmetryOperation.twelfthsOf(b?JS.SymmetryOperation.normalizeTwelfths(e/12,12,!0):p(e))).append("\t]\n")}return c.toString()},"JU.M4,~B");d(c$,"setMatrixFromXYZ",function(a,b,c){if(null==a)return!1;this.xyzOriginal=a;this.divisor=JS.SymmetryOperation.setDivisor(a);a=a.toLowerCase();this.setModDim(b);var f=a.startsWith("!");f&&(a=a.substring(1));if(0==a.indexOf("xyz matrix:"))return this.xyz=a,JU.Parser.parseStringInfestedFloatArray(a,null,this.linearRotTrans),this.setFromMatrix(null,f);
if(0==a.indexOf("[[")){a=a.$replace("["," ").$replace("]"," ").$replace(","," ");JU.Parser.parseStringInfestedFloatArray(a,null,this.linearRotTrans);for(b=this.linearRotTrans.length;0<=--b;)if(Float.isNaN(this.linearRotTrans[b]))return!1;this.setMatrix(f);this.isFinalized=!0;this.xyz=(this.isBio=0<=a.indexOf("bio"))?this.toString():JS.SymmetryOperation.getXYZFromMatrix(this,!1,!1,!1);return!0}if(0==b&&0<=a.indexOf("x4"))for(b=14;4<=--b;)if(0<=a.indexOf("x"+b)){this.setModDim(b-3);break}b=null;a.endsWith("m")?
(this.timeReversal=0<=a.indexOf("-m")?-1:1,c=!0):0<=a.indexOf("mz)")&&(c=a.indexOf("("),b=a.substring(c+1,a.length-1),a=a.substring(0,c),c=!1);c=JS.SymmetryOperation.getMatrixFromString(this,a,this.linearRotTrans,c);if(null==c)return!1;this.xyzCanonical=c;if(null!=b){var d=1==JU.M4.newA16(this.linearRotTrans).determinant3();this.timeReversal=0>a.indexOf("-x")==0>b.indexOf("-mx")==d?1:-1}this.setMatrix(f);this.xyz=f?JS.SymmetryOperation.getXYZFromMatrix(this,!0,!1,!1):this.doNormalize?c:a;0!=this.timeReversal&&
(this.xyz+=1==this.timeReversal?",m":",-m");JU.Logger.debugging&&JU.Logger.debug(""+this);return!0},"~S,~N,~B");c$.setDivisor=d(c$,"setDivisor",function(a){for(var b=a.indexOf("/"),c=a.length;0<b&&b<c-1;){var f=a.charAt(b+1);if(0>"2346".indexOf(f)||b<c-2&&Character.isDigit(a.charAt(b+2)))return 0;b=a.indexOf("/",b+1)}return 12},"~S");d(c$,"setModDim",function(a){this.modDim=a;0<a&&(this.myLabels=JS.SymmetryOperation.labelsXn);this.linearRotTrans=r((a+4)*(a+4),0)},"~N");d(c$,"setMatrix",function(a){16<
this.linearRotTrans.length?this.setGamma(a):(this.setA(this.linearRotTrans),a&&(a=JU.P3.new3(this.m03,this.m13,this.m23),this.invert(),this.rotate(a),a.scale(-1),this.setTranslation(a)))},"~B");d(c$,"setFromMatrix",function(a,b){var c=0,f=0;this.myLabels=0==this.modDim?JS.SymmetryOperation.labelsXYZ:JS.SymmetryOperation.labelsXn;for(var d=0,e=3+this.modDim,g=0;d<e;g++){if(Float.isNaN(this.linearRotTrans[g]))return!1;c=this.linearRotTrans[g];1E-5>Math.abs(c)&&(c=0);if(0==(g+1)%(e+1)){var l=0==this.divisor?
p(c)&255:this.divisor;0==l&&(l=12);c=JS.SymmetryOperation.finalizeF(c,this.divisor);null!=a&&f<a.length&&(c+=a[f++]);c=JS.SymmetryOperation.normalizeTwelfths((0>c?-1:1)*Math.abs(c*l)/l,l,this.doNormalize);0==this.divisor&&(c=JS.SymmetryOperation.toDivisor(c,l));d++}this.linearRotTrans[g]=c}this.linearRotTrans[this.linearRotTrans.length-1]=this.divisor;this.setMatrix(b);this.isFinalized=null==a;this.xyz=JS.SymmetryOperation.getXYZFromMatrix(this,!0,!1,!1);return!0},"~A,~B");c$.getMatrixFromXYZ=d(c$,
"getMatrixFromXYZ",function(a){var b=r(16,0);a=JS.SymmetryOperation.getMatrixFromString(null,"!"+a,b,!1);return null==a?null:JS.SymmetryOperation.div12(JU.M4.newA16(b),JS.SymmetryOperation.setDivisor(a))},"~S");c$.getMatrixFromString=d(c$,"getMatrixFromString",function(a,b,c,f){var d=!1,e=!1,g=!1,l=null==a?0:a.modDim,n=4+l,j=null==a?JS.SymmetryOperation.setDivisor(b):a.divisor,m=null==a?!b.startsWith("!"):a.doNormalize,k=0<l?3:0;c[c.length-1]=1;var q=b.indexOf(";")+1;0!=q&&(f=!0,q==b.length&&(b+=
"0,0,0"));var r=-1;a=null==a||0==l?null:a.myLabels;null==a&&(a=JS.SymmetryOperation.labelsXYZ);b=b.toLowerCase()+",";b=b.$replace("(",",");0<l&&(b=JS.SymmetryOperation.replaceXn(b,l+3));for(var x=0,s=0,u=0,t,w=0,y=0,z=0,B=1,A="",H="",D=Q(1,0),K=b.length,E=0;E<K;E++){switch(t=b.charAt(E)){case ";":break;case "'":case " ":case "{":case "}":case "!":continue;case "-":g=!0;continue;case "+":g=!1;continue;case "/":y=0;d=!0;continue;case "x":case "y":case "z":case "a":case "b":case "c":case "d":case "e":case "f":case "g":case "h":s=
u*n;d="x"<=t?t.charCodeAt(0)-120:t.charCodeAt(0)-97+k;x=s+d;g=g?-1:1;f&&0!=w?(c[x]=w,g=p(w),w=0):c[x]=g;A+=JS.SymmetryOperation.plusMinus(A,g,a[d]);break;case ",":if(0!=q){if(0<q){r=E;E=q-1;q=-E;y=w=0;continue}q=E+1;E=r}w=JS.SymmetryOperation.normalizeTwelfths(w,0==y?12:0==j?y:j,m);c[s+n-1]=0==j&&0<y?w=JS.SymmetryOperation.toDivisor(z,y):w;A+=JS.SymmetryOperation.xyzFraction12(w,0==j?y:j,!1,!0);H+=(""===H?"":",")+A;if(u==n-2)return H;y=z=w=0;A="";if(2<u++&&0==l)return JU.Logger.warn("Symmetry Operation? "+
b),null;break;case ".":e=!0;B=1;continue;case "0":if(!e&&12==j&&(d||!f))continue;default:var G=t.charCodeAt(0)-48;if(0<=G&&9>=G){if(e){B/=10;0>w&&(g=!0);w+=B*G*(g?-1:1);continue}if(d){D[0]=E;y=JU.PT.parseIntNext(b,D);if(0>y)return null;E=D[0]-1;0==w?c[x]/=y:(z=p(w),w/=y)}else w=10*w+(g?-1:1)*G}else JU.Logger.warn("symmetry character?"+t)}e=d=g=!1}return null},"JS.SymmetryOperation,~S,~A,~B");c$.replaceXn=d(c$,"replaceXn",function(a,b){for(var c=b;0<=--c;)a=JU.PT.rep(a,JS.SymmetryOperation.labelsXn[c],
JS.SymmetryOperation.labelsXnSub[c]);return a},"~S,~N");c$.toDivisor=d(c$,"toDivisor",function(a,b){var c=p(a);c!=a&&(c=a-c,b=p(Math.abs(b/c)),c=p(Math.abs(a)/c));return(c<<8)+b},"~N,~N");c$.xyzFraction12=d(c$,"xyzFraction12",function(a,b,c,d){12!=b&&(a=p(a),b=a&255,a>>=8);var h=u(b/2);if(c)for(;0>a;)a+=b;else if(d){for(;a>h;)a-=b;for(;a<-h;)a+=b}b=12==b?JS.SymmetryOperation.twelfthsOf(a):0==a?"0":a+"/"+b;return"0"==b.charAt(0)?"":0<a?"+"+b:b},"~N,~N,~B,~B");c$.twelfthsOf=d(c$,"twelfthsOf",function(a){var b=
"";0>a&&(a=-a,b="-");var c=12,d=Math.round(a);if(0.01<Math.abs(d-a)){a/=12;for(c=3;20>c;c++){var h=a*c,d=Math.round(h);if(0.01>Math.abs(d-h))break}if(20==c)return b+a}else{if(12==d)return b+"1";if(12>d)return b+JS.SymmetryOperation.twelfths[d%12];switch(d%12){case 0:return""+u(d/12);case 2:case 10:c=6;break;case 3:case 9:c=4;break;case 4:case 8:c=3;break;case 6:c=2}d=u(d*c/12)}return b+d+"/"+c},"~N");c$.plusMinus=d(c$,"plusMinus",function(a,b,c){return 0==b?"":(0>b?"-":0==a.length?"":"+")+(1==b||
-1==b?"":""+p(Math.abs(b)))+c},"~S,~N,~S");c$.normalizeTwelfths=d(c$,"normalizeTwelfths",function(a,b,c){a*=b;var d=u(b/2);if(c){for(;a>d;)a-=b;for(;a<=-d;)a+=b}return a},"~N,~N,~B");c$.getXYZFromMatrix=d(c$,"getXYZFromMatrix",function(a,b,c,d){var h="",e=z(a,JS.SymmetryOperation)?a:null;if(null!=e&&0<e.modDim)return JS.SymmetryOperation.getXYZFromRsVs(e.rsvs.getRotation(),e.rsvs.getTranslation(),b);var e=r(4,0),g=p(a.getElement(3,3));1==g?g=12:a.setElement(3,3,1);for(var l=0;3>l;l++){var n=3>l?0:
3;a.getRow(l,e);for(var j="",m=0;3>m;m++)0!=e[m]&&(j+=JS.SymmetryOperation.plusMinus(j,e[m],JS.SymmetryOperation.labelsXYZ[m+n]));j+=JS.SymmetryOperation.xyzFraction12(b?e[3]:e[3]*g,g,c,d);h+=","+j}return h.substring(1)},"JU.M4,~B,~B,~B");c$.setOffset=d(c$,"setOffset",function(a,b,c,d){if(0!=d){var h=0,e=0,g=0;null==JS.SymmetryOperation.atomTest&&(JS.SymmetryOperation.atomTest=new JU.P3);for(var l=c+d;c<l;c++)JS.SymmetryOperation.newPoint(a,b[c],JS.SymmetryOperation.atomTest,0,0,0),h+=JS.SymmetryOperation.atomTest.x,
e+=JS.SymmetryOperation.atomTest.y,g+=JS.SymmetryOperation.atomTest.z;h/=d;e/=d;for(g/=d;-0.001>h||1.001<=h;)a.m03+=0>h?1:-1,h+=0>h?1:-1;for(;-0.001>e||1.001<=e;)a.m13+=0>e?1:-1,e+=0>e?1:-1;for(;-0.001>g||1.001<=g;)a.m23+=0>g?1:-1,g+=0>g?1:-1}},"JU.M4,~A,~N,~N");d(c$,"rotateAxes",function(a,b,c,d){var h=Array(3);this.getRotationScale(d);for(var e=a.length;0<=--e;)c.setT(a[e]),b.toFractional(c,!0),d.rotate(c),b.toCartesian(c,!0),h[e]=JU.V3.newV(c);return h},"~A,JS.UnitCell,JU.P3,JU.M3");d(c$,"fcoord2",
function(a){return 12==this.divisor?JS.SymmetryOperation.fcoord(a):this.fc2(this.linearRotTrans[3])+" "+this.fc2(this.linearRotTrans[7])+" "+this.fc2(this.linearRotTrans[11])},"JU.T3");d(c$,"fc2",function(a){a=p(a);var b=a&255;a>>=8;return 0==a?"0":a+"/"+b},"~N");c$.fcoord=d(c$,"fcoord",function(a){return JS.SymmetryOperation.fc(a.x)+" "+JS.SymmetryOperation.fc(a.y)+" "+JS.SymmetryOperation.fc(a.z)},"JU.T3");c$.fc=d(c$,"fc",function(a){var b=Math.abs(a);a=0>a?"-":"";b=p(JS.SymmetryOperation.approxF(24*
b));return b/24==p(b/24)?a+u(b/24):0!=b%8?a+JS.SymmetryOperation.twelfthsOf(b>>1):0==b?"0":24==b?a+"1":a+u(b/8)+"/3"},"~N");c$.approxF=d(c$,"approxF",function(a){return JU.PT.approx(a,100)},"~N");c$.getXYZFromRsVs=d(c$,"getXYZFromRsVs",function(a,b,c){a=a.getArray();b=b.getArray();for(var d=a.length,h="",e=0;e<d;e++){for(var h=h+",",g=0;g<d;g++){var l=a[e][g];0!=l&&(h+=(0>l?"-":h.endsWith(",")?"":"+")+(1==Math.abs(l)?"":""+u(Math.abs(l)))+"x"+(g+1))}h+=JS.SymmetryOperation.xyzFraction12(u(b[e][0]*
(c?1:12)),12,!1,!0)}return JU.PT.rep(h.substring(1),",+",",")},"JU.Matrix,JU.Matrix,~B");d(c$,"toString",function(){return null==this.rsvs?ba(this,JS.SymmetryOperation,"toString",[]):ba(this,JS.SymmetryOperation,"toString",[])+" "+this.rsvs.toString()});d(c$,"getMagneticOp",function(){return 3.4028235E38==this.magOp?this.magOp=this.determinant3()*this.timeReversal:this.magOp});d(c$,"setTimeReversal",function(a){this.timeReversal=a;0<=this.xyz.indexOf("m")&&(this.xyz=this.xyz.substring(0,this.xyz.indexOf("m")));
0!=a&&(this.xyz+=1==a?",m":",-m")},"~N");d(c$,"getCentering",function(){this.isFinalized||this.doFinalize();null==this.centering&&!this.unCentered&&(0==this.modDim&&1==this.m00&&1==this.m11&&1==this.m22&&0==this.m01&&0==this.m02&&0==this.m10&&0==this.m12&&0==this.m20&&0==this.m21&&(0!=this.m03||0!=this.m13||0!=this.m23)?(this.isCenteringOp=!0,this.centering=JU.V3.new3(this.m03,this.m13,this.m23)):(this.unCentered=!0,this.centering=null));return this.centering});d(c$,"fixMagneticXYZ",function(a,b,
c){if(0==this.timeReversal)return b;var d=b.indexOf("m"),d=d-u((3-this.timeReversal)/2);b=0>d?b:b.substring(0,d);if(!c)return b+(0<this.timeReversal?" +1":" -1");a=JU.M4.newM4(a);a.m03=a.m13=a.m23=0;0>this.getMagneticOp()&&a.scale(-1);return b+="("+JU.PT.rep(JU.PT.rep(JU.PT.rep(JS.SymmetryOperation.getXYZFromMatrix(a,!1,!1,!1),"x","mx"),"y","my"),"z","mz")+")"},"JU.M4,~S,~B");d(c$,"getInfo",function(){null==this.info&&(this.info=new java.util.Hashtable,this.info.put("xyz",this.xyz),null!=this.centering&&
this.info.put("centering",this.centering),this.info.put("index",Integer.$valueOf(this.number-1)),this.info.put("isCenteringOp",Boolean.$valueOf(this.isCenteringOp)),null!=this.linearRotTrans&&this.info.put("linearRotTrans",this.linearRotTrans),this.info.put("modulationDimension",Integer.$valueOf(this.modDim)),this.info.put("matrix",JU.M4.newM4(this)),3.4028235E38!=this.magOp&&this.info.put("magOp",Float.$valueOf(this.magOp)),this.info.put("id",Integer.$valueOf(this.opId)),this.info.put("timeReversal",
Integer.$valueOf(this.timeReversal)),null!=this.xyzOriginal&&this.info.put("xyzOriginal",this.xyzOriginal));return this.info});G(c$,"atomTest",null,"DIVISOR_MASK",255,"DIVISOR_OFFSET",8,"twelfths",K(-1,"0 1/12 1/6 1/4 1/3 5/12 1/2 7/12 2/3 3/4 5/6 11/12".split(" ")));c$.labelsXYZ=c$.prototype.labelsXYZ=K(-1,["x","y","z"]);c$.labelsXn=c$.prototype.labelsXn=K(-1,"x1 x2 x3 x4 x5 x6 x7 x8 x9 x10 x11 x12 x13".split(" "));c$.labelsXnSub=c$.prototype.labelsXnSub=K(-1,"xyzabcdefghij".split(""))});M("JS");
B(null,"JS.SymmetryInfo",["JU.PT","JU.SimpleUnitCell"],function(){c$=N(function(){this.isMultiCell=this.coordinatesAreFractional=!1;this.cellRange=this.infoStr=this.symmetryOperations=this.sgName=null;this.latticeType="P";this.intlTableNo=null;R(this,arguments)},JS,"SymmetryInfo");D(c$,function(){});d(c$,"setSymmetryInfo",function(a,b){this.cellRange=a.get("unitCellRange");this.sgName=a.get("spaceGroup");if(null==this.sgName||""===this.sgName)this.sgName="spacegroup unspecified";this.infoStr="Spacegroup: "+
this.sgName;if(null==(this.latticeType=a.get("latticeType")))this.latticeType="P";this.intlTableNo=a.get("intlTableNo");var c=a.containsKey("symmetryCount")?a.get("symmetryCount").intValue():0;this.symmetryOperations=a.remove("symmetryOps");if(null!=this.symmetryOperations){for(var d="",g="\nNumber of symmetry operations: "+(0==c?1:c)+"\nSymmetry Operations:",e=0;e<c;e++){var p=this.symmetryOperations[e],g=g+("\n"+p.fixMagneticXYZ(p,p.xyz,!0));p.isCenteringOp&&(d+=" ("+JU.PT.rep(JU.PT.replaceAllCharacters(p.xyz,
"xyz","0"),"0+","")+")")}0<d.length&&(this.infoStr+="\nCentering: "+d);this.infoStr+=g;this.infoStr+="\n"}null==b&&(b=a.get("unitCellParams"));if(!JU.SimpleUnitCell.isValid(b))return null;this.isMultiCell=(this.coordinatesAreFractional=a.containsKey("coordinatesAreFractional")?a.get("coordinatesAreFractional").booleanValue():!1)&&null!=this.symmetryOperations;return b},"java.util.Map,~A")});M("JS");B(["JU.SimpleUnitCell","JU.P3","JV.JC"],"JS.UnitCell","java.lang.Double $.Float java.util.Hashtable JU.M3 $.M4 $.P4 $.PT $.Quat $.T4 $.V3 J.api.Interface JS.Symmetry JU.BoxInfo $.Escape".split(" "),
function(){c$=N(function(){this.fractionalOffset=this.vertices=null;this.allFractionalRelative=!1;this.moreInfo=this.unitCellMultiplier=this.cartesianOffset=null;this.name="";R(this,arguments)},JS,"UnitCell",JU.SimpleUnitCell,Cloneable);U(c$,function(){this.cartesianOffset=new JU.P3});c$.fromOABC=d(c$,"fromOABC",function(a,b){var c=new JS.UnitCell;3==a.length&&(a=K(-1,[new JU.P3,a[0],a[1],a[2]]));var d=r(-1,[-1,0,0,0,0,0,a[1].x,a[1].y,a[1].z,a[2].x,a[2].y,a[2].z,a[3].x,a[3].y,a[3].z]);c.init(d);c.allFractionalRelative=
b;c.initUnitcellVertices();c.setCartesianOffset(a[0]);return c},"~A,~B");c$.fromParams=d(c$,"fromParams",function(a,b){var c=new JS.UnitCell;c.init(a);c.initUnitcellVertices();c.allFractionalRelative=b;return c},"~A,~B");d(c$,"initOrientation",function(a){if(null!=a){var b=new JU.M4;b.setToM3(a);this.matrixFractionalToCartesian.mul2(b,this.matrixFractionalToCartesian);this.matrixCartesianToFractional.setM4(this.matrixFractionalToCartesian).invert();this.initUnitcellVertices()}},"JU.M3");d(c$,"toUnitCell",
function(a,b){null!=this.matrixCartesianToFractional&&(null==b?(this.matrixCartesianToFractional.rotTrans(a),this.unitize(a),this.matrixFractionalToCartesian.rotTrans(a)):(this.matrixCtoFNoOffset.rotTrans(a),this.unitize(a),a.add(b),this.matrixFtoCNoOffset.rotTrans(a)))},"JU.T3,JU.T3");d(c$,"unitize",function(a){switch(this.dimension){case 3:a.z=JS.UnitCell.toFractionalX(a.z);case 2:a.y=JS.UnitCell.toFractionalX(a.y);case 1:a.x=JS.UnitCell.toFractionalX(a.x)}},"JU.T3");d(c$,"reset",function(){this.unitCellMultiplier=
null;this.setOffset(JU.P3.new3(0,0,0))});d(c$,"setOffset",function(a){if(null!=a){var b=z(a,JU.T4)?a:null,c=null!=b&&999999<b.w;if(null!=b?0>=b.w||c:100<=a.x||100<=a.y)if(this.unitCellMultiplier=0==a.z&&a.x==a.y&&!c?null:c?JU.P4.newPt(b):JU.P3.newP(a),null==b||0==b.w||c)return;if(this.hasOffset()||0<a.lengthSquared())this.fractionalOffset=new JU.P3,this.fractionalOffset.setT(a);this.matrixCartesianToFractional.m03=-a.x;this.matrixCartesianToFractional.m13=-a.y;this.matrixCartesianToFractional.m23=
-a.z;this.cartesianOffset.setT(a);this.matrixFractionalToCartesian.m03=0;this.matrixFractionalToCartesian.m13=0;this.matrixFractionalToCartesian.m23=0;this.matrixFractionalToCartesian.rotTrans(this.cartesianOffset);this.matrixFractionalToCartesian.m03=this.cartesianOffset.x;this.matrixFractionalToCartesian.m13=this.cartesianOffset.y;this.matrixFractionalToCartesian.m23=this.cartesianOffset.z;this.allFractionalRelative&&(this.matrixCtoFNoOffset.setM4(this.matrixCartesianToFractional),this.matrixFtoCNoOffset.setM4(this.matrixFractionalToCartesian))}},
"JU.T3");d(c$,"setCartesianOffset",function(a){this.cartesianOffset.setT(a);this.matrixFractionalToCartesian.m03=this.cartesianOffset.x;this.matrixFractionalToCartesian.m13=this.cartesianOffset.y;this.matrixFractionalToCartesian.m23=this.cartesianOffset.z;a=this.hasOffset();this.fractionalOffset=new JU.P3;this.fractionalOffset.setT(this.cartesianOffset);this.matrixCartesianToFractional.m03=0;this.matrixCartesianToFractional.m13=0;this.matrixCartesianToFractional.m23=0;this.matrixCartesianToFractional.rotTrans(this.fractionalOffset);
this.matrixCartesianToFractional.m03=-this.fractionalOffset.x;this.matrixCartesianToFractional.m13=-this.fractionalOffset.y;this.matrixCartesianToFractional.m23=-this.fractionalOffset.z;this.allFractionalRelative&&(this.matrixCtoFNoOffset.setM4(this.matrixCartesianToFractional),this.matrixFtoCNoOffset.setM4(this.matrixFractionalToCartesian));!a&&0==this.fractionalOffset.lengthSquared()&&(this.fractionalOffset=null)},"JU.T3");d(c$,"getInfo",function(){var a=new java.util.Hashtable;a.put("params",this.unitCellParams);
a.put("vectors",this.getUnitCellVectors());a.put("volume",Double.$valueOf(this.volume));a.put("matFtoC",this.matrixFractionalToCartesian);a.put("matCtoF",this.matrixCartesianToFractional);return a});d(c$,"dumpInfo",function(a){return"a="+this.a+", b="+this.b+", c="+this.c+", alpha="+this.alpha+", beta="+this.beta+", gamma="+this.gamma+"\n"+JU.Escape.eAP(this.getUnitCellVectors())+"\nvolume="+this.volume+(a?"\nfractional to cartesian: "+this.matrixFractionalToCartesian+"\ncartesian to fractional: "+
this.matrixCartesianToFractional:"")},"~B");d(c$,"getVertices",function(){return this.vertices});d(c$,"getCartesianOffset",function(){return this.cartesianOffset});d(c$,"getFractionalOffset",function(){return this.fractionalOffset});d(c$,"getTensor",function(a,b){var c=J.api.Interface.getUtil("Tensor",a,"file");if(0==b[0]&&0==b[1]&&0==b[2]){var d=b[7],g=r(-1,[d,d,d]);return c.setFromEigenVectors(JS.UnitCell.unitVectors,g,"iso","Uiso="+d,null)}c.parBorU=b;var d=aa(6,0),e=p(b[6]);if(12==e)d[0]=19.739208802178716*
b[0],d[1]=19.739208802178716*b[1],d[2]=19.739208802178716*b[2],d[3]=39.47841760435743*b[3],d[4]=39.47841760435743*b[4],d[5]=39.47841760435743*b[5],b[7]=(b[0]+b[1]+b[3])/3;else{var g=4==e||5==e||8==e||9==e,F=2-e%2,l=8==e||9==e||10==e?19.739208802178716:4==e||5==e?0.25:2==e||3==e?Math.log(2):1,e=b[0]*l*(g?this.a_*this.a_:1),n=b[1]*l*(g?this.b_*this.b_:1),j=b[2]*l*(g?this.c_*this.c_:1),m=b[3]*l*(g?this.a_*this.b_:1)*F,k=b[4]*l*(g?this.a_*this.c_:1)*F,g=b[5]*l*(g?this.b_*this.c_:1)*F;b[7]=Math.pow(e/
19.739208802178716/this.a_/this.a_*n/19.739208802178716/this.b_/this.b_*j/19.739208802178716/this.c_/this.c_,0.3333);d[0]=this.a*this.a*e+this.b*this.b*this.cosGamma*this.cosGamma*n+this.c*this.c*this.cosBeta*this.cosBeta*j+this.a*this.b*this.cosGamma*m+this.b*this.c*this.cosGamma*this.cosBeta*g+this.a*this.c*this.cosBeta*k;d[1]=this.b*this.b*this.sinGamma*this.sinGamma*n+this.c*this.c*this.cA_*this.cA_*j+this.b*this.c*this.cA_*this.sinGamma*g;d[2]=this.c*this.c*this.cB_*this.cB_*j;d[3]=2*this.b*
this.b*this.cosGamma*this.sinGamma*n+2*this.c*this.c*this.cA_*this.cosBeta*j+this.a*this.b*this.sinGamma*m+this.b*this.c*(this.cA_*this.cosGamma+this.sinGamma*this.cosBeta)*g+this.a*this.c*this.cA_*k;d[4]=2*this.c*this.c*this.cB_*this.cosBeta*j+this.b*this.c*this.cosGamma*g+this.a*this.c*this.cB_*k;d[5]=2*this.c*this.c*this.cA_*this.cB_*j+this.b*this.c*this.cB_*this.sinGamma*g}return c.setFromThermalEquation(d,JU.Escape.eAF(b))},"JV.Viewer,~A");d(c$,"getCanonicalCopy",function(a,b){var c=Array(8),
d=null,g=null;b&&null!=this.unitCellMultiplier&&(d=new JU.P3,g=new JU.P3,JU.SimpleUnitCell.ijkToPoint3f(p(this.unitCellMultiplier.x),d,0,0),JU.SimpleUnitCell.ijkToPoint3f(p(this.unitCellMultiplier.y),g,0,0),g.sub(d));for(var e=0;8>e;e++){var r=c[e]=JU.P3.newP(JU.BoxInfo.unitCubePoints[e]);null!=d&&(a*=0==this.unitCellMultiplier.z?1:this.unitCellMultiplier.z,c[e].add3(d.x+g.x*r.x,d.y+g.y*r.y,d.z+g.z*r.z));this.matrixFractionalToCartesian.rotTrans(r);b||r.sub(this.cartesianOffset)}return JU.BoxInfo.getCanonicalCopy(c,
a)},"~N,~B");c$.toFractionalX=d(c$,"toFractionalX",function(a){a-=Math.floor(a);if(0.9999<a||1E-4>a)a=0;return a},"~N");d(c$,"initUnitcellVertices",function(){if(null!=this.matrixFractionalToCartesian){this.matrixCtoFNoOffset=JU.M4.newM4(this.matrixCartesianToFractional);this.matrixFtoCNoOffset=JU.M4.newM4(this.matrixFractionalToCartesian);this.vertices=Array(8);for(var a=8;0<=--a;)this.vertices[a]=this.matrixFractionalToCartesian.rotTrans2(JU.BoxInfo.unitCubePoints[a],new JU.P3)}});d(c$,"checkDistance",
function(a,b,c,d,g,e,p,l){a=JU.P3.newP(a);this.toCartesian(a,!0);for(var n=-g;n<=g;n++)for(var j=-e;j<=e;j++)for(var m=-p;m<=p;m++){l.set(b.x+n,b.y+j,b.z+m);this.toCartesian(l,!0);var k=a.distance(l);if(0<d?Math.abs(k-c)<=d:k<=c&&0.1<k)return l.set(n,j,m),!0}return!1},"JU.P3,JU.P3,~N,~N,~N,~N,~N,JU.P3");d(c$,"getUnitCellMultiplier",function(){return this.unitCellMultiplier});d(c$,"getUnitCellVectors",function(){var a=this.matrixFractionalToCartesian;return K(-1,[JU.P3.newP(this.cartesianOffset),JU.P3.new3(this.fix(a.m00),
this.fix(a.m10),this.fix(a.m20)),JU.P3.new3(this.fix(a.m01),this.fix(a.m11),this.fix(a.m21)),JU.P3.new3(this.fix(a.m02),this.fix(a.m12),this.fix(a.m22))])});d(c$,"fix",function(a){return 0.001>Math.abs(a)?0:a},"~N");d(c$,"isSameAs",function(a){if(a.unitCellParams.length!=this.unitCellParams.length)return!1;for(var b=this.unitCellParams.length;0<=--b;)if(this.unitCellParams[b]!=a.unitCellParams[b]&&(!Float.isNaN(this.unitCellParams[b])||!Float.isNaN(a.unitCellParams[b])))return!1;return null==this.fractionalOffset?
!a.hasOffset():null==a.fractionalOffset?!this.hasOffset():0==this.fractionalOffset.distanceSquared(a.fractionalOffset)},"JS.UnitCell");d(c$,"hasOffset",function(){return null!=this.fractionalOffset&&0!=this.fractionalOffset.lengthSquared()});d(c$,"getState",function(){var a="";null!=this.fractionalOffset&&0!=this.fractionalOffset.lengthSquared()&&(a+="  unitcell offset "+JU.Escape.eP(this.fractionalOffset)+";\n");null!=this.unitCellMultiplier&&(a+="  unitcell range "+JU.SimpleUnitCell.escapeMultiplier(this.unitCellMultiplier)+
";\n");return a});d(c$,"getQuaternionRotation",function(a){var b=JU.V3.newVsub(this.vertices[4],this.vertices[0]),c=JU.V3.newVsub(this.vertices[2],this.vertices[0]),d=JU.V3.newVsub(this.vertices[1],this.vertices[0]),g=new JU.V3,e=new JU.V3,p="-"==a.charAt(0)?-1:1;0>p&&(a=a.substring(1));var l=0;2==a.length&&(l=a.charCodeAt(1)-48,a=a.substring(0,1));var n=0==l%2;switch("abc".indexOf(a)){default:a=b;b=d;0<l&&0<p==n&&(b=c,a.scale(-1));break;case 1:a=c;0<l&&0<p==n&&(b=d,a.scale(-1));break;case 2:a=d,
0<l&&(l=5-l,0<p!=n&&(b=c,a.scale(-1)))}switch(l){case 2:a.scale(-1);b.scale(-1);break;case 3:b.scale(-1);break;case 4:a.scale(-1)}g.cross(a,b);e.cross(g,a);return JU.Quat.getQuaternionFrame(null,e,g).inv()},"~S");d(c$,"getV0abc",function(a){if(z(a,Array))return a;var b=!1,c=Array(4),d=c[0]=JU.V3.new3(0,0,0);c[1]=JU.V3.new3(1,0,0);c[2]=JU.V3.new3(0,1,0);c[3]=JU.V3.new3(0,0,1);var g=new JU.M3;if(z(a,String)){var e="0,0,0";if(0==a.indexOf("a="))return JU.SimpleUnitCell.setOabc(a,null,c);b=a.indexOf(";");
0<=b&&(e=a.substring(b+1),a=a.substring(0,b));a+=";0,0,0";(b=a.startsWith("!"))&&(a=a.substring(1));var p=new JS.Symmetry;p.setSpaceGroup(!1);a=p.addSpaceGroupOperation("="+a,0);if(0>a)return null;a=p.getSpaceGroupOperation(a);a.doFinalize();if(null!=e){e=JU.PT.split(e,",");p=r(3,0);if(3==e.length)for(var l=0;3>l;l++){var n=e[l],j=n.indexOf("/");p[l]=0<=j?JU.PT.parseFloat(n.substring(0,j))/JU.PT.parseFloat(n.substring(j+1)):JU.PT.parseFloat(n)}e=JU.P3.new3(p[0],p[1],p[2]);a.setTranslation(e)}}else if(z(a,
JU.M3))a=JU.M4.newMV(a,new JU.P3);else if(!z(a,JU.M4)){a=a[0];a.getRotationScale(g);this.toCartesian(d,!1);a.rotTrans(d);for(a=1;4>a;a++)this.toCartesian(c[a],!0),g.rotate(c[a]);return c}a.getRotationScale(g);a.getTranslation(d);b?(g.invert(),g.transpose(),g.rotate(d),d.scale(-1)):g.transpose();this.toCartesian(d,!1);for(a=1;4>a;a++)g.rotate(c[a]),this.toCartesian(c[a],!0);return c},"~O");d(c$,"toFromPrimitive",function(a,b,c,d){var g=c.length-3,e=null;if("r"==b||null==d){switch(b){default:return!1;
case "r":return JU.SimpleUnitCell.getReciprocal(c,c,1),!0;case "P":a=!0;e=JU.M3.newA9(r(-1,[1,0,0,0,1,0,0,0,1]));break;case "A":e=JU.M3.newA9(r(-1,[1,0,0,0,0.5,0.5,0,-0.5,0.5]));break;case "B":e=JU.M3.newA9(r(-1,[0.5,0,0.5,0,1,0,-0.5,0,0.5]));break;case "C":e=JU.M3.newA9(r(-1,[0.5,0.5,0,-0.5,0.5,0,0,0,1]));break;case "R":e=JU.M3.newA9(r(-1,[0.6666667,-0.33333334,-0.33333334,0.33333334,0.33333334,-0.6666667,0.33333334,0.33333334,0.33333334]));break;case "I":e=JU.M3.newA9(r(-1,[-0.5,0.5,0.5,0.5,-0.5,
0.5,0.5,0.5,-0.5]));break;case "F":e=JU.M3.newA9(r(-1,[0,0.5,0.5,0.5,0,0.5,0.5,0.5,0]))}a||e.invert()}else e=JU.M3.newM3(d),a&&e.invert();for(a=c.length;--a>=g;)b=c[a],this.toFractional(b,!1),e.rotate(b),this.toCartesian(b,!1);return!0},"~B,~S,~A,JU.M3");d(c$,"getConventionalUnitCell",function(a,b){var c=this.getUnitCellVectors();(!a.equals("P")||null!=b)&&this.toFromPrimitive(!1,a.charAt(0),c,b);return c},"~S,JU.M3");c$.cloneUnitCell=d(c$,"cloneUnitCell",function(a){var b=null;try{b=a.clone()}catch(c){if(!X(c,
CloneNotSupportedException))throw c;}return b},"JS.UnitCell");G(c$,"twoP2",19.739208802178716);c$.unitVectors=c$.prototype.unitVectors=K(-1,[JV.JC.axisX,JV.JC.axisY,JV.JC.axisZ])})})(Clazz,Clazz.getClassName,Clazz.newLongArray,Clazz.doubleToByte,Clazz.doubleToInt,Clazz.doubleToLong,Clazz.declarePackage,Clazz.instanceOf,Clazz.load,Clazz.instantialize,Clazz.decorateAsClass,Clazz.floatToInt,Clazz.floatToLong,Clazz.makeConstructor,Clazz.defineEnumConstant,Clazz.exceptionOf,Clazz.newIntArray,Clazz.defineStatics,
Clazz.newFloatArray,Clazz.declareType,Clazz.prepareFields,Clazz.superConstructor,Clazz.newByteArray,Clazz.declareInterface,Clazz.p0p,Clazz.pu$h,Clazz.newShortArray,Clazz.innerTypeInstance,Clazz.isClassDefined,Clazz.prepareCallback,Clazz.newArray,Clazz.castNullAs,Clazz.floatToShort,Clazz.superCall,Clazz.decorateAsType,Clazz.newBooleanArray,Clazz.newCharArray,Clazz.implementOf,Clazz.newDoubleArray,Clazz.overrideConstructor,Clazz.clone,Clazz.doubleToShort,Clazz.getInheritedLevel,Clazz.getParamsType,
Clazz.isAF,Clazz.isAB,Clazz.isAI,Clazz.isAS,Clazz.isASS,Clazz.isAP,Clazz.isAFloat,Clazz.isAII,Clazz.isAFF,Clazz.isAFFF,Clazz.tryToSearchAndExecute,Clazz.getStackTrace,Clazz.inheritArgs,Clazz.alert,Clazz.defineMethod,Clazz.overrideMethod,Clazz.declareAnonymous,Clazz.cloneFinals);
